// CertInfoDlg.cpp : implementation file
//

#include "stdafx.h"
#include "CertInfo.h"
#include "CertInfoDlg.h"
#include "PasswordDialog.h"
#include "PKIBBox.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCertInfoDlg dialog

CCertInfoDlg::CCertInfoDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCertInfoDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CCertInfoDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDI_ICON1/*IDR_MAINFRAME*/);
	m_imglist.Create(IDB_LISTCTRL, 16, 1, RGB(255, 255, 255));
}

void CCertInfoDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCertInfoDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

void CCertInfoDlg::InitializeTreeCtrl()
{
	CListCtrl* ctrl;
	DWORD style;
	ctrl = (CListCtrl*)this->GetDlgItem(IDC_LISTCERTINFO);
	ctrl->SetImageList(&m_imglist, LVSIL_SMALL);
	ctrl->InsertColumn(0, "Issuer", LVCFMT_LEFT, 150);
	ctrl->InsertColumn(1, "Subject", LVCFMT_LEFT, 150);
	ctrl->InsertColumn(2, "Valid from", LVCFMT_LEFT, 80);
	ctrl->InsertColumn(3, "Valid till", LVCFMT_LEFT, 80);
	ctrl->InsertColumn(4, "Public key", LVCFMT_LEFT, 80);

	style = ctrl->GetExtendedStyle();
	style |= LVS_EX_FULLROWSELECT;
	ctrl->SetExtendedStyle(style);
}

void CCertInfoDlg::InitializeComboBox()
{
	CComboBox* ctrl;
	ctrl = (CComboBox*)this->GetDlgItem(IDC_COMBOCS);
	int nSize, i;
	char** storenames;
	HANDLE hStore;
	
	nSize = 0;
	SBGetAvailableSystemStores(NULL, &nSize);
	storenames = (char**)malloc(nSize);
	SBGetAvailableSystemStores(storenames, &nSize);

	for(i = 0; storenames[i]; i++) {
		ctrl->AddString((LPCTSTR)storenames[i]);

		hStore = SBWinCertStorageCreate();
		if (!hStore) {
			ErrorInfo("Unable to create ElWinCertStorage object. Please contact support@eldos.org.");
		}
		if (!SBWinCertStorageAdd(hStore, storenames[i])) {
			ErrorInfo("Unable to access ROOT certificate store.");
		}
		ctrl->SetItemData(i, (long)hStore);

	}
	ctrl->SetCurSel(0);
}

void CCertInfoDlg::RefreshCertificateList()
{
	CListCtrl* ctrl;
	CComboBox* cbox;
	HANDLE hCert, hRDN, hCertStorage;
	bool found;
	int i, j, nSize, count, alg;
	unsigned char buffer[4096];
	LPCSTR szAlgStr;
	DATE date;
	COleDateTime oledate;

	ctrl = (CListCtrl*)this->GetDlgItem(IDC_LISTCERTINFO);
	ctrl->DeleteAllItems();

	cbox = (CComboBox*)this->GetDlgItem(IDC_COMBOCS);
	hCertStorage = (HANDLE)cbox->GetItemData(cbox->GetCurSel());
	if (!hCertStorage) {
		return;
	}

	count = SBWinCertStorageGetCount(hCertStorage);
	ctrl->SetItemCount(count);
	for(i = 0; i < count; i++) {
		hCert = SBWinCertStorageGetCertificate(hCertStorage, i);
		// note that we should not destroy the hCert handle, as it is destroyed by WinCertStorage handle.
		// inserting issuer's common name
		hRDN = SBX509GetIssuerRDN(hCert);
		// note that we should not destroy the hRDN handle, as it is destroyed by SBX509 handle.
		found = false;
		for (j = 0; j < SBRelativeDistinguishedNameGetCount(hRDN); j++) {
			nSize = 4096;
			if (SBRelativeDistinguishedNameGetOID(hRDN, j, buffer, &nSize)) {
				if ((nSize == strlen(SB_CERT_OID_COMMON_NAME)) && (memcmp(buffer,
					SB_CERT_OID_COMMON_NAME, nSize) == 0)) {
					nSize = 4096;
					if (SBRelativeDistinguishedNameGetValue(hRDN, j, buffer, &nSize)) {
						found = true;
						buffer[nSize] = '\0';
					} 
					break;
				} 
			} 
		}
		if (!found) {
			buffer[0] = '\0';
		}
		ctrl->InsertItem(i, (LPCTSTR)buffer, 0);
		// inserting subject's common name
		hRDN = SBX509GetSubjectRDN(hCert);
		found = false;
		for (j = 0; j < SBRelativeDistinguishedNameGetCount(hRDN); j++) {
			nSize = 4096;
			if (SBRelativeDistinguishedNameGetOID(hRDN, j, buffer, &nSize)) {
				if ((nSize == strlen(SB_CERT_OID_COMMON_NAME)) && (memcmp(buffer,
					SB_CERT_OID_COMMON_NAME, nSize) == 0)) {
					nSize = 4096;
					if (SBRelativeDistinguishedNameGetValue(hRDN, j, buffer, &nSize)) {
						found = true;
						buffer[nSize] = '\0';
					} 
					break;
				} 
			} 
		}
		if (!found) {
			buffer[0] = '\0';
		}
		ctrl->SetItemText(i, 1, (LPCTSTR)buffer);

		// validity period start
		SBX509GetValidityFrom(hCert, &date);
		oledate = date;
		ctrl->SetItemText(i, 2, oledate.Format(VAR_DATEVALUEONLY));

		// validity period end
		SBX509GetValidityTo(hCert, &date);
		oledate = date;
		ctrl->SetItemText(i, 3, oledate.Format(VAR_DATEVALUEONLY));

		// public key info
		alg = SBX509GetPublicKeyAlgorithm(hCert);
		switch(alg) {
		case SB_CERT_ALGORITHM_ID_RSA_ENCRYPTION:
			szAlgStr = "RSA";
			break;
		case SB_CERT_ALGORITHM_ID_DSA:
			szAlgStr = "DSA";
			break;
		case SB_CERT_ALGORITHM_DH_PUBLIC:
			szAlgStr = "DH";
			break;
		default:
			szAlgStr = "Unknown";
		}
		sprintf((char*)buffer, "%s/%d", szAlgStr, SBX509GetPublicKeySize(hCert));
		ctrl->SetItemText(i, 4, (LPCTSTR)buffer);
		ctrl->SetItemData(i, (DWORD)hCert);
	}
}

void CCertInfoDlg::ErrorInfo(LPCTSTR lpszErrorMsg)
{
	MessageBox(lpszErrorMsg, "Error");
}

BOOL CCertInfoDlg::SaveCertDER(HANDLE hCert, LPCTSTR lpszFilename)
{
	unsigned char* pCertBuffer = NULL;
	int nCertSize;
	HANDLE hFile;
	BOOL res;
	DWORD nWritten;
	
	if (!hCert) {
		return FALSE;
	}
	
	nCertSize = 0;
	SBX509SaveToBuffer(hCert, NULL, &nCertSize);
	pCertBuffer = (unsigned char*)malloc(nCertSize);
	SBX509SaveToBuffer(hCert, pCertBuffer, &nCertSize);

	hFile = CreateFile(lpszFilename, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);
	if (hFile != INVALID_HANDLE_VALUE) {
		WriteFile(hFile, pCertBuffer, nCertSize, &nWritten, NULL);
		CloseHandle(hFile);
		res = TRUE;
	} else {
		res = FALSE;
	}
	free(pCertBuffer);
	return res;
}

BOOL CCertInfoDlg::SaveCertPEM(HANDLE hCert, LPCTSTR lpszFilename)
{
	unsigned char* pCertBuffer = NULL;
	int nCertSize;
	HANDLE hFile;
	DWORD nWritten;
	BOOL res;

	if (!hCert) {
		return FALSE;
	}

	nCertSize = 0;
	SBX509SaveToBufferPEM(hCert, NULL, &nCertSize, "");
	pCertBuffer = (unsigned char*)malloc(nCertSize);
	if (!SBX509SaveToBufferPEM(hCert, pCertBuffer, &nCertSize, "")) {
		free(pCertBuffer);
		return FALSE;
	}

	hFile = CreateFile(lpszFilename, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);
	if (hFile != INVALID_HANDLE_VALUE) {
		WriteFile(hFile, pCertBuffer, nCertSize, &nWritten, NULL);
		CloseHandle(hFile);
		res = TRUE;
	} else {
		res = FALSE;
	}
	free(pCertBuffer);
	return res;
}

BOOL CCertInfoDlg::SaveCertPFX(HANDLE hCert, LPCTSTR lpszFilename)
{
	CPasswordDialog dlg;
	unsigned char* pCertBuffer = NULL;
	int nCertSize, nPassSize;
	char password[128];
	HANDLE hFile;
	DWORD nWritten;
	BOOL res;

	if (!hCert) {
		return FALSE;
	}

	// checking up that private key exists and is exportable
	if (!SBX509PrivateKeyExists(hCert)) {
		return FALSE;
	}
	nCertSize = 0;
	SBX509SaveKeyToBuffer(hCert, NULL, &nCertSize);
	if (nCertSize == 0) {
		return FALSE;
	}

	int nResponse = dlg.DoModal();
	if (nResponse) {
		nPassSize = 128;
		dlg.GetPassword(password, &nPassSize);
		password[nPassSize] = '\0';
	} else {
		password[0] = '\0';
	}

	nCertSize = 0;
	SBX509SaveToBufferPFX(hCert, NULL, &nCertSize, password, SB_ALGORITHM_PBE_SHA1_RC4_40, SB_ALGORITHM_PBE_SHA1_RC4_40);
	pCertBuffer = (unsigned char*)malloc(nCertSize);
	if (SBX509SaveToBufferPFX(hCert, pCertBuffer, &nCertSize, password, SB_ALGORITHM_PBE_SHA1_RC4_40, SB_ALGORITHM_PBE_SHA1_RC4_40) != 0) {
		free(pCertBuffer);
		return FALSE;
	}

	hFile = CreateFile(lpszFilename, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);
	if (hFile != INVALID_HANDLE_VALUE) {
		WriteFile(hFile, pCertBuffer, nCertSize, &nWritten, NULL);
		CloseHandle(hFile);
		res = TRUE;
	} else {
		res = FALSE;
	}
	free(pCertBuffer);
	return res;
}

BOOL CCertInfoDlg::LoadCertDER(HANDLE hCert, LPCTSTR lpszFilename)
{
	HANDLE hFile;
	DWORD nSize, nSizeHi;
	void* pCertBuffer;
	BOOL res;

	hFile = CreateFile(lpszFilename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
	if (hFile != INVALID_HANDLE_VALUE) {
		nSize = GetFileSize(hFile, &nSizeHi);
		pCertBuffer = malloc(nSize);
		res = ReadFile(hFile, pCertBuffer, nSize, &nSizeHi, NULL);
		CloseHandle(hFile);

		if (res) {
			SBX509LoadFromBuffer(hCert, pCertBuffer, nSize);
		}

		free(pCertBuffer);
		return res;
	} else {
		return FALSE;
	}
}

BOOL CCertInfoDlg::LoadCertPEM(HANDLE hCert, LPCTSTR lpszFilename)
{
	HANDLE hFile;
	DWORD nSize, nSizeHi;
	void* pCertBuffer;
	BOOL res;

	hFile = CreateFile(lpszFilename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
	if (hFile != INVALID_HANDLE_VALUE) {
		nSize = GetFileSize(hFile, &nSizeHi);
		pCertBuffer = malloc(nSize);
		res = ReadFile(hFile, pCertBuffer, nSize, &nSizeHi, NULL);
		CloseHandle(hFile);

		if (res) {
			res = SBX509LoadFromBufferPEM(hCert, pCertBuffer, nSize, "");
		}

		free(pCertBuffer);
		return res;
	} else {
		return FALSE;
	}
}

BOOL CCertInfoDlg::LoadCertPFX(HANDLE hCert, LPCTSTR lpszFilename)
{
	HANDLE hFile;
	DWORD nSize, nSizeHi;
	void* pCertBuffer;
	BOOL res;
	CPasswordDialog dlg;
	int nPassSize, passcount;
	char password[128];

	hFile = CreateFile(lpszFilename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
	if (hFile != INVALID_HANDLE_VALUE) {
		nSize = GetFileSize(hFile, &nSizeHi);
		pCertBuffer = malloc(nSize);
		res = ReadFile(hFile, pCertBuffer, nSize, &nSizeHi, NULL);
		CloseHandle(hFile);

		if (res) {

			passcount = 3;
			while (passcount > 0) {
				int nResponse = dlg.DoModal();
				if (nResponse) {
					nPassSize = 128;
					dlg.GetPassword(password, &nPassSize);
					password[nPassSize] = '\0';
				} else {
					password[0] = '\0';
				}
				(SBX509LoadFromBufferPFX(hCert, pCertBuffer, nSize, password) == 0) ? res = 1 : res = 0;
				if (res == 1) {
					break;
				}
				passcount--;
			}
		}
		free(pCertBuffer);
		return res;
	} else {
		return FALSE;
	}
}

BEGIN_MESSAGE_MAP(CCertInfoDlg, CDialog)
	//{{AFX_MSG_MAP(CCertInfoDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_CBN_SELCHANGE(IDC_COMBOCS, OnSelchangeCombocs)
	ON_BN_CLICKED(IDC_BUTTONEXPORT, OnButtonexport)
	ON_BN_CLICKED(IDC_BUTTONIMPORT, OnButtonimport)
	ON_BN_CLICKED(IDC_BUTTONENCRYPT, OnButtonencrypt)
	ON_BN_CLICKED(IDC_BUTTONDECRYPT, OnButtondecrypt)
	ON_BN_CLICKED(IDC_BUTTONSIGN, OnButtonsign)
	ON_BN_CLICKED(IDC_BUTTONVERIFY, OnButtonverify)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCertInfoDlg message handlers

BOOL CCertInfoDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here

	InitializeComboBox();
	InitializeTreeCtrl();
	RefreshCertificateList();

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CCertInfoDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CCertInfoDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CCertInfoDlg::OnSelchangeCombocs() 
{
	// TODO: Add your control notification handler code here
	RefreshCertificateList();	
}

void CCertInfoDlg::OnButtonexport() 
{
	const char *szExportFilter = "Plain certificate (*.cer)|*.cer|PEM-encoded certificate (*.pem)|*.pem|Certificate encrypted with private key (*.pfx)|*.pfx|All Files (*.*)|*.*||";
	HANDLE hCert;
	CListCtrl* ctrl;
	POSITION pos;
	int item;
	CString filepath, ext;
	CFileDialog dlg(FALSE, NULL, NULL, OFN_OVERWRITEPROMPT, szExportFilter, this);

	ctrl = (CListCtrl*)this->GetDlgItem(IDC_LISTCERTINFO);
	pos = ctrl->GetFirstSelectedItemPosition();
	if (pos) {
		item = ctrl->GetNextSelectedItem(pos);
		hCert = (HANDLE)ctrl->GetItemData(item);
		if (!hCert) {
			return;
		}
		if (dlg.DoModal() == IDOK) {

			filepath = dlg.GetPathName();
			ext = dlg.GetFileExt();

			if (_stricmp(ext, "pem") == 0) {
				if (!SaveCertPEM(hCert, filepath)) {
					ErrorInfo("Error: unable to save file. Please check up that you have enough rights to write to this file.");
				} else {
					MessageBox("Export done");
				}
			} else if (_stricmp(ext, "pfx") == 0) {
				if (!SaveCertPFX(hCert, filepath)) {
					ErrorInfo("Error: unable to save file. Please check up that certificate has an exportable private key and you have enough rights to write to this file.");
				} else {
					MessageBox("Export done");
				}
			} else {
				if (!SaveCertDER(hCert, filepath)) {
					ErrorInfo("Error: unable to save file. Please check up that you have enough rights to write to this file.");	
				} else {
					MessageBox("Export done");
				}
			}				
		}
	}
}

void CCertInfoDlg::OnButtonimport() 
{
	const char *szImportFilter = "Plain certificate (*.cer)|*.cer|PEM-encoded certificate (*.pem)|*.pem|Certificate encrypted with private key (*.pfx)|*.pfx||";
	HANDLE hCert;
	CFileDialog dlg(TRUE, NULL, NULL, NULL, szImportFilter, this);
	CString filepath, ext;
	BOOL res;
	HANDLE hStore;
	CComboBox* cbox;

	if (dlg.DoModal() == IDOK) {
		filepath = dlg.GetPathName();
		ext = dlg.GetFileExt();
		hCert = SBX509Create();

		res = FALSE;
		if (_stricmp(ext, "pem") == 0) {
			if (!LoadCertPEM(hCert, filepath)) {
				ErrorInfo("Error: unable to load certificate. Please check up that it contains the valid certificate and you have enough rights to access this file.");
			} else {
				res = TRUE;
			}
		} else if (_stricmp(ext, "pfx") == 0) {
			if (!LoadCertPFX(hCert, filepath)) {
				ErrorInfo("Error: unable to load certificate. Please check up that it contains the valid certificate and you have enough rights to access this file.");
			} else {
				res = TRUE;
			}
		} else {
			if (!LoadCertDER(hCert, filepath)) {
				ErrorInfo("Error: unable to load certificate. Please check up that it contains the valid certificate and you have enough rights to access this file.");
			} else {
				res = TRUE;
			}
		}				
		if (res) {
			cbox = (CComboBox*)this->GetDlgItem(IDC_COMBOCS);
			hStore = (HANDLE)cbox->GetItemData(cbox->GetCurSel());
			if (!hStore) {
				return;
			}
			
			if (SBWinCertStorageAddCertificate(hStore, hCert)) {
				RefreshCertificateList();
				MessageBox("Import done");
			} else {
				ErrorInfo("Error: unable to import certificate to Windows certificate store. Please check up that you have enough rights to access this store.");
			}
		}
		SBX509Destroy(hCert);
	}
}

void CCertInfoDlg::OnButtonencrypt() 
{
	const char *szOpenFilter = "All files (*.*)|*.*||";
	const char *szSaveFilter = "PKCS7 encrypted data (*.p7c)|*.p7c||";
	HANDLE hCert, hEncryptor, hStore;
	HANDLE hFile;
	CListCtrl* ctrl;
	POSITION pos;
	int item;
	CString filepath, ext;
	CFileDialog savedlg(FALSE, "p7c", NULL, OFN_OVERWRITEPROMPT, szSaveFilter, this);
	CFileDialog opendlg(TRUE, NULL, NULL, 0, szOpenFilter, this);
	void* pInputBuffer, *pOutputBuffer;
	int nInputSize, nOutputSize;
	BOOL res;
	DWORD dwResult, dwTmp;
	char errorLine[255];

	ctrl = (CListCtrl*)this->GetDlgItem(IDC_LISTCERTINFO);
	pos = ctrl->GetFirstSelectedItemPosition();
	if (pos) {
		item = ctrl->GetNextSelectedItem(pos);
		hCert = (HANDLE)ctrl->GetItemData(item);
		if (!hCert) {
			return;
		}
		if (opendlg.DoModal() == IDOK) {

			filepath = opendlg.GetPathName();
			ext = opendlg.GetFileExt();

			hFile = CreateFile(filepath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
			if (hFile == INVALID_HANDLE_VALUE) {
				ErrorInfo("Error: unable to open source file. Please check up that you have enough rights to access this file.");
				return;
			}
			nInputSize = GetFileSize(hFile, &dwTmp);
			if (dwTmp > 0) {
				CloseHandle(hFile);
				ErrorInfo("Error: file size too large.");
				return;
			}
			pInputBuffer = malloc(nInputSize);
			res = ReadFile(hFile, pInputBuffer, nInputSize, &dwTmp, NULL);
			CloseHandle(hFile);

			hEncryptor = SBMessageEncryptorCreate();
			hStore = SBMemoryCertStorageCreate();
			SBMemoryCertStorageAdd(hStore, hCert);
			SBMessageEncryptorSetCertStorage(hEncryptor, hStore);
			SBMessageEncryptorSetAlgorithm(hEncryptor, SB_ALGORITHM_CNT_3DES);
			nOutputSize = 0;
			SBMessageEncryptorEncrypt(hEncryptor, pInputBuffer, nInputSize, NULL, &nOutputSize);
			pOutputBuffer = malloc(nOutputSize);
			dwResult = SBMessageEncryptorEncrypt(hEncryptor, pInputBuffer, nInputSize, pOutputBuffer, &nOutputSize);

			if (dwResult != 0) {
				sprintf(errorLine, "Error: unable to encrypt (error %x). Please view SecureBlackbox documentation for details.", dwResult);
				ErrorInfo(errorLine);
			} else {
				if (savedlg.DoModal() == IDOK) {
					filepath = savedlg.GetPathName();
					
					hFile = CreateFile(filepath, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);
					if (hFile != INVALID_HANDLE_VALUE) {
						WriteFile(hFile, pOutputBuffer, nOutputSize, &dwTmp, NULL);
						CloseHandle(hFile);
						MessageBox("Encryption process completed successfully");
					} else {
						ErrorInfo("Error: unable to open file for writing. Please check up that you have enough access rights to it.");
					}
				}
			}
			
			SBMessageEncryptorDestroy(hEncryptor);
			SBMemoryCertStorageDestroy(hStore);
			free(pInputBuffer);
			free(pOutputBuffer);
		}
	}
}

void CCertInfoDlg::OnButtondecrypt() 
{
	const char *szSaveFilter = "All files (*.*)|*.*||";
	const char *szOpenFilter = "PKCS7 encrypted data (*.p7c)|*.p7c||";
	HANDLE hDecryptor, hStore;
	HANDLE hFile;
	CComboBox* cbox;
	CString filepath, ext;
	CFileDialog savedlg(FALSE, NULL, NULL, OFN_OVERWRITEPROMPT, szSaveFilter, this);
	CFileDialog opendlg(TRUE, NULL, NULL, 0, szOpenFilter, this);
	void* pInputBuffer, *pOutputBuffer;
	int nInputSize, nOutputSize;
	BOOL res;
	DWORD dwResult, dwTmp;
	char errorLine[255];

	cbox = (CComboBox*)this->GetDlgItem(IDC_COMBOCS);
	hStore = (HANDLE)cbox->GetItemData(cbox->GetCurSel());
	if (!hStore) {
		return;
	}

	if (opendlg.DoModal() == IDOK) {

		filepath = opendlg.GetPathName();
		ext = opendlg.GetFileExt();

		hFile = CreateFile(filepath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
		if (hFile == INVALID_HANDLE_VALUE) {
			ErrorInfo("Error: unable to open source file. Please check up that you have enough rights to access this file.");
			return;
		}
		nInputSize = GetFileSize(hFile, &dwTmp);
		if (dwTmp > 0) {
			CloseHandle(hFile);
			ErrorInfo("Error: file size too large.");
			return;
		}
		pInputBuffer = malloc(nInputSize);
		res = ReadFile(hFile, pInputBuffer, nInputSize, &dwTmp, NULL);
		CloseHandle(hFile);

		hDecryptor = SBMessageDecryptorCreate();
		SBMessageDecryptorSetCertStorage(hDecryptor, hStore);
		nOutputSize = 0;
		SBMessageDecryptorDecrypt(hDecryptor, pInputBuffer, nInputSize, NULL, &nOutputSize);
		pOutputBuffer = malloc(nOutputSize);
		dwResult = SBMessageDecryptorDecrypt(hDecryptor, pInputBuffer, nInputSize, pOutputBuffer, &nOutputSize);

		if (dwResult != 0) {
			sprintf(errorLine, "Error: unable to decrypt (error %x). Please view SecureBlackbox documentation for details.", dwResult);
			ErrorInfo(errorLine);
		} else {
			if (savedlg.DoModal() == IDOK) {
				filepath = savedlg.GetPathName();
				
				hFile = CreateFile(filepath, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);
				if (hFile != INVALID_HANDLE_VALUE) {
					WriteFile(hFile, pOutputBuffer, nOutputSize, &dwTmp, NULL);
					CloseHandle(hFile);
					MessageBox("Decryption process completed successfully");
				} else {
					ErrorInfo("Error: unable to open file for writing. Please check up that you have enough access rights to it.");
				}
			}
		}
		
		SBMessageDecryptorDestroy(hDecryptor);
		free(pInputBuffer);
		free(pOutputBuffer);
	}
}

void CCertInfoDlg::OnButtonsign() 
{
	const char *szOpenFilter = "All files (*.*)|*.*||";
	const char *szSaveFilter = "PKCS7 signed data (*.p7c)|*.p7c||";
	HANDLE hCert, hSigner, hStore;
	HANDLE hFile;
	CListCtrl* ctrl;
	POSITION pos;
	int item;
	CString filepath, ext;
	CFileDialog savedlg(FALSE, "p7c", NULL, OFN_OVERWRITEPROMPT, szSaveFilter, this);
	CFileDialog opendlg(TRUE, NULL, NULL, 0, szOpenFilter, this);
	void* pInputBuffer, *pOutputBuffer;
	int nInputSize, nOutputSize;
	BOOL res;
	DWORD dwResult, dwTmp;
	char errorLine[255];

	ctrl = (CListCtrl*)this->GetDlgItem(IDC_LISTCERTINFO);
	pos = ctrl->GetFirstSelectedItemPosition();
	if (pos) {
		item = ctrl->GetNextSelectedItem(pos);
		hCert = (HANDLE)ctrl->GetItemData(item);
		if (!hCert) {
			return;
		}
		if (opendlg.DoModal() == IDOK) {

			filepath = opendlg.GetPathName();
			ext = opendlg.GetFileExt();

			hFile = CreateFile(filepath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
			if (hFile == INVALID_HANDLE_VALUE) {
				ErrorInfo("Error: unable to open source file. Please check up that you have enough rights to access this file.");
				return;
			}
			nInputSize = GetFileSize(hFile, &dwTmp);
			if (dwTmp > 0) {
				CloseHandle(hFile);
				ErrorInfo("Error: file size too large.");
				return;
			}
			pInputBuffer = malloc(nInputSize);
			res = ReadFile(hFile, pInputBuffer, nInputSize, &dwTmp, NULL);
			CloseHandle(hFile);

			hSigner = SBMessageSignerCreate();
			hStore = SBMemoryCertStorageCreate();
			SBMemoryCertStorageAdd(hStore, hCert);
			SBMessageSignerSetCertStorage(hSigner, hStore);
			SBMessageSignerSetHashAlgorithm(hSigner, SB_ALGORITHM_DGST_SHA1);
			nOutputSize = 0;
			SBMessageSignerSign(hSigner, pInputBuffer, nInputSize, NULL, &nOutputSize);
			pOutputBuffer = malloc(nOutputSize);
			dwResult = SBMessageSignerSign(hSigner, pInputBuffer, nInputSize, pOutputBuffer, &nOutputSize);

			if (dwResult != 0) {
				sprintf(errorLine, "Error: unable to sign (error %x). Please view SecureBlackbox documentation for details.", dwResult);
				ErrorInfo(errorLine);
			} else {
				if (savedlg.DoModal() == IDOK) {
					filepath = savedlg.GetPathName();
					
					hFile = CreateFile(filepath, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);
					if (hFile != INVALID_HANDLE_VALUE) {
						WriteFile(hFile, pOutputBuffer, nOutputSize, &dwTmp, NULL);
						CloseHandle(hFile);
						MessageBox("Signing process completed successfully");
					} else {
						ErrorInfo("Error: unable to open file for writing. Please check up that you have enough access rights to it.");
					}
				}
			}
			
			SBMessageSignerDestroy(hSigner);
			SBMemoryCertStorageDestroy(hStore);
			free(pInputBuffer);
			free(pOutputBuffer);
		}
	}
}

void CCertInfoDlg::OnButtonverify() 
{
	const char *szSaveFilter = "All files (*.*)|*.*||";
	const char *szOpenFilter = "PKCS7 signed data (*.p7c)|*.p7c||";
	HANDLE hVerifier, hStore;
	HANDLE hFile;
	CComboBox* cbox;
	CString filepath, ext;
	CFileDialog savedlg(FALSE, NULL, NULL, OFN_OVERWRITEPROMPT, szSaveFilter, this);
	CFileDialog opendlg(TRUE, NULL, NULL, 0, szOpenFilter, this);
	void* pInputBuffer, *pOutputBuffer;
	int nInputSize, nOutputSize;
	BOOL res;
	DWORD dwResult, dwTmp;
	char errorLine[255];

	cbox = (CComboBox*)this->GetDlgItem(IDC_COMBOCS);
	hStore = (HANDLE)cbox->GetItemData(cbox->GetCurSel());
	if (!hStore) {
		return;
	}

	if (opendlg.DoModal() == IDOK) {

		filepath = opendlg.GetPathName();
		ext = opendlg.GetFileExt();

		hFile = CreateFile(filepath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
		if (hFile == INVALID_HANDLE_VALUE) {
			ErrorInfo("Error: unable to open source file. Please check up that you have enough rights to access this file.");
			return;
		}
		nInputSize = GetFileSize(hFile, &dwTmp);
		if (dwTmp > 0) {
			CloseHandle(hFile);
			ErrorInfo("Error: file size too large.");
			return;
		}
		pInputBuffer = malloc(nInputSize);
		res = ReadFile(hFile, pInputBuffer, nInputSize, &dwTmp, NULL);
		CloseHandle(hFile);

		hVerifier = SBMessageVerifierCreate();
		SBMessageVerifierSetCertStorage(hVerifier, hStore);
		nOutputSize = 0;
		SBMessageVerifierVerify(hVerifier, pInputBuffer, nInputSize, NULL, &nOutputSize);
		pOutputBuffer = malloc(nOutputSize);
		dwResult = SBMessageVerifierVerify(hVerifier, pInputBuffer, nInputSize, pOutputBuffer, &nOutputSize);

		if (dwResult != 0) {
			sprintf(errorLine, "Error: unable to verify (error %x). Please view SecureBlackbox documentation for details.", dwResult);
			ErrorInfo(errorLine);
		} else {
			if (savedlg.DoModal() == IDOK) {
				filepath = savedlg.GetPathName();
				
				hFile = CreateFile(filepath, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);
				if (hFile != INVALID_HANDLE_VALUE) {
					WriteFile(hFile, pOutputBuffer, nOutputSize, &dwTmp, NULL);
					CloseHandle(hFile);
					MessageBox("Signature is OK");
				} else {
					ErrorInfo("Error: unable to open file for writing. Please check up that you have enough access rights to it.");
				}
			}
		}
		
		SBMessageVerifierDestroy(hVerifier);
		free(pInputBuffer);
		free(pOutputBuffer);
	}
}
