unit PKIBBoxWrapper;

interface

uses
  Windows;

const
  // Certificate algorithms
  SB_CERT_ALGORITHM_ID_RSA_ENCRYPTION   = $0000;
  SB_CERT_ALGORITHM_MD2_RSA_ENCRYPTION  = $0001;
  SB_CERT_ALGORITHM_MD5_RSA_ENCRYPTION  = $0002;
  SB_CERT_ALGORITHM_SHA1_RSA_ENCRYPTION = $0003;
  SB_CERT_ALGORITHM_ID_DSA              = $0004;
  SB_CERT_ALGORITHM_ID_DSA_SHA1         = $0005;
  SB_CERT_ALGORITHM_DH_PUBLIC           = $0006;
  SB_CERT_ALGORITHM_UNKNOWN             = 65535;

  // Validity constants
  SB_CERT_VALIDITY_OK                   = $0001;
  SB_CERT_VALIDITY_SELF_SIGNED          = $0002;
  SB_CERT_VALIDITY_INVALID              = $0004;
  SB_CERT_VALIDITY_STORAGE_ERROR        = $0008;
  SB_CERT_VALIDITY_REASON_BAD_DATA      = $0001;
  SB_CERT_VALIDITY_REASON_REVOKED       = $0002;
  SB_CERT_VALIDITY_REASON_NOT_YET_VALID = $0004;
  SB_CERT_VALIDITY_REASON_EXPIRED       = $0008;
  SB_CERT_VALIDITY_REASON_INVALID_SIGNATURE     = $0010;
  SB_CERT_VALIDITY_REASON_UNKNOWN_CA    = $0020;

  // Object identifiers to use in Subject/Issuer fields
  SB_CERT_OID_COMMON_NAME               = #$55#$04#$03;
  SB_CERT_OID_SURNAME                   = #$55#$04#$04;
  SB_CERT_OID_COUNTRY                   = #$55#$04#$06;
  SB_CERT_OID_LOCALITY                  = #$55#$04#$07;
  SB_CERT_OID_STATE_OR_PROVINCE         = #$55#$04#$08;
  SB_CERT_OID_ORGANIZATION              = #$55#$04#$0A;
  SB_CERT_OID_ORGANIZATION_UNIT         = #$55#$04#$0B;
  SB_CERT_OID_TITLE                     = #$55#$04#$0C;
  SB_CERT_OID_NAME                      = #$55#$04#$29;
  SB_CERT_OID_GIVEN_NAME                = #$55#$04#$2A;
  SB_CERT_OID_INITIALS                  = #$55#$04#$2B;
  SB_CERT_OID_GENERATION_QUALIFIER      = #$55#$04#$2C;
  SB_CERT_OID_DN_QUALIFIER              = #$55#$04#$2E;

  // Algorithm constants
  SB_ALGORITHM_CNT_RC4                  = $7001;
  SB_ALGORITHM_CNT_DES                  = $7002;
  SB_ALGORITHM_CNT_3DES                 = $7003;
  SB_ALGORITHM_CNT_RC2                  = $7004;
  SB_ALGORITHM_CNT_AES128               = $7005;
  SB_ALGORITHM_CNT_AES192               = $7006;
  SB_ALGORITHM_CNT_AES256               = $7007;
  SB_ALGORITHM_DGST_SHA1                = $7101;
  SB_ALGORITHM_DGST_MD5                 = $7102;
  SB_ALGORITHM_PBE_SHA1_RC4_128         = $7201;
  SB_ALGORITHM_PBE_SHA1_RC4_40          = $7202;
  SB_ALGORITHM_PBE_SHA1_3DES            = $7203;
  SB_ALGORITHM_PBE_SHA1_RC2_128         = $7204;
  SB_ALGORITHM_PBE_SHA1_RC2_40          = $7205;
  SB_ALGORITHM_UNKNOWN                  = $7FFF;

function SBPKIInitialize: boolean;
procedure SBPKIFinalize;
function SBPKIIsInitialized: boolean;
function SBPKISetLicenseKey(Key : PChar) : boolean;

{ ElX509Certificate functions }
function SBX509GetVersion(Handle : THandle) : integer;
function SBX509GetSerialNumber(Handle : THandle; SerialNumber : Pointer; var Size : longint) : boolean;
function SBX509Create : THandle;
function SBX509LoadFromBuffer(Handle : THandle; Data : Pointer; Size : longint) : boolean;
function SBX509GetCertificateBinary(Handle : THandle; Buffer : Pointer; var Size : longint) : boolean;
function SBX509GetSignature(Handle : THandle; Buffer : Pointer; var Size : longint) : boolean;
function SBX509GetIssuer(Handle : THandle; Buffer : Pointer; var Size : longint) : boolean;
function SBX509GetValidFrom(Handle : THandle; var Date : TDateTime) : boolean;
function SBX509GetValidTo(Handle : THandle; var Date : TDateTime) : boolean;
function SBX509GetSubject(Handle : THandle; Buffer : Pointer; var Size : longint) : boolean;
function SBX509GetSignatureAlgorithm(Handle : THandle) : integer;
function SBX509GetPublicKeyAlgorithm(Handle : THandle) : integer;
function SBX509GetRSAParams(Handle : THandle; RSAModulus : Pointer; var RSAModulusSize : integer;
  RSAPublicKey : Pointer; var RSAPublicKeySize : integer) : boolean;
function SBX509GetDSSSignatureParams(Handle : THandle; DSSS : Pointer; var DSSSSize : integer;
  DSSR : Pointer; var DSSRSize : integer) : boolean;
function SBX509GetDSSParams(Handle : THandle; DSSP : Pointer; var DSSPSize : integer;
  DSSQ : Pointer; var DSSQSize : integer; DSSG : Pointer; var DSSGSize : integer;
  DSSY : Pointer; var DSSYSize : integer) : boolean;
function SBX509GetDHParams(Handle : THandle; DHP : Pointer; var DHPSize : integer;
  DHG : Pointer; var DHGSize : integer; DHY : Pointer; var DHYSize : integer) : boolean;
function SBX509SelfSigned(Handle : THandle) : boolean;
function SBX509Validate(Handle : THandle) : boolean;
function SBX509ValidateRSA(Handle : THandle; RSAModulus : pointer; RSAModulusSize : integer;
  RSAPublicKey : pointer; RSAPublicKeySize : integer) : boolean;
function SBX509ValidateDSS(Handle : THandle; DSSP : pointer; DSSPSize : integer;
  DSSQ : pointer; DSSQSize : integer; DSSG : pointer; DSSGSize : integer;
  DSSY : pointer; DSSYSize : integer) : boolean;
function SBX509ValidateWithCA(Handle : THandle; CACertificate : pointer;
  CACertificateSize : integer) : boolean;
function SBX509SetIssuer(Handle : THandle; Fields : pointer): boolean;
function SBX509SetSubject(Handle : THandle; Fields : pointer): boolean;
function SBX509SaveToBuffer(Handle : THandle; Buffer : pointer; var Size : longint) : boolean;
function SBX509SaveKeyToBuffer(Handle : THandle; Buffer : pointer; var Size : longint) : boolean;
function SBX509Generate(Handle : THandle; Algorithm : integer; DWordsInEncryptKey : integer) : boolean;
function SBX509CAGenerate(Handle : THandle; Algorithm : integer; DWordsInEncryptKey : integer) : boolean;
function SBX509SetCACertificate(Handle : THandle; Buffer : pointer; Size : longint) : boolean;
function SBX509SetCAPrivateKey(Handle : THandle; Buffer : pointer; Size : longint) : boolean;
function SBX509Destroy(Handle : THandle) : boolean;
function SBX509SetValidFrom(Handle : THandle; Date : TDateTime) : boolean;
function SBX509SetValidTo(Handle : THandle; Date : TDateTime) : boolean;
function SBX509LoadKeyFromBuffer(Handle : THandle; Buffer : pointer; Size : longint) : boolean;
function SBX509LoadFromBufferPEM(Handle : THandle; Buffer : pointer; Size : longint; PassPhrase : PChar) : boolean;
function SBX509LoadKeyFromBufferPEM(Handle : THandle; Buffer : pointer; Size : longint; PassPhrase : PChar) : boolean;
function SBX509SaveToBufferPEM(Handle : THandle; Buffer : pointer; var Size : longint; PassPhrase : PChar) : boolean;
function SBX509SaveKeyToBufferPEM(Handle : THandle; Buffer : pointer; var Size : longint; PassPhrase : PChar) : boolean;
function SBX509Clone(Handle : THandle; DestHandle : THandle) : boolean;
function SBX509GetHashMD5(Handle : THandle; Buffer : pointer) : boolean;
function SBX509GetHashSHA1(Handle : THandle; Buffer : pointer) : boolean;
function SBX509GetPublicKeySize(Handle : THandle) : integer;
function SBX509GetPublicKeyBlob(Handle : THandle; Buffer : pointer; var
  Size : integer) : boolean;
function SBX509PrivateKeyExists(Handle : THandle) : boolean;
function SBX509LoadFromBufferPFX(Handle : THandle; Buffer : pointer; Size : integer;
  Password : PChar) : integer;
function SBX509SaveToBufferPFX(Handle : THandle; Buffer : pointer; var Size : integer;
  Password : PChar; KeyEncryptionAlgorithm : integer; CertEncryptionAlgorithm :
  integer) : integer;
function SBX509Sign(Handle : THandle; InBuffer : pointer; InSize : integer;
  OutBuffer : pointer; var OutSize : integer) : integer;
function SBX509Verify(Handle : THandle; InBuffer : pointer; InSize : integer;
  Signature : pointer; SignatureSize : integer) : integer;
function SBX509GenerateByRequest(Handle : THandle; RequestHandle : THandle;
  NewCertHandle : THandle) : boolean;
function SBX509GetIssuerRDN(Handle : THandle) : THandle;
function SBX509GetSubjectRDN(Handle : THandle) : THandle;
function SBX509SignDetached(Handle : THandle; InBuffer: pointer; InSize : integer;
  OutBuffer: pointer; var OutSize : integer) : integer;

{ Certificate storage functions }
function SBGetAvailableSystemStores(Buffer : pointer; var Size : integer) : boolean;
function SBGetAvailablePhysicalStores(SystemStore : string; Buffer : pointer;
  var Size : integer) : boolean;
function SBCertStorageFindByHashMD5(Handle : THandle; Buffer : pointer) : integer;
function SBCertStorageFindByHashSHA1(Handle : THandle; Buffer : pointer) : integer;
function SBCertStorageGetIssuerCertificate(Handle : THandle; CertHandle : THandle) : integer;
function SBWinCertStorageCreate: THandle;
function SBWinCertStorageDestroy(Handle : THandle): boolean;
function SBWinCertStorageValidate(Handle : THandle; CertificateHandle : THandle;
  var Reason : integer) : integer;
function SBWinCertStorageAdd(Handle : THandle; StoreName : PChar) : boolean;
function SBWinCertStorageRemove(Handle : THandle; StoreName : PChar) : boolean;
function SBWinCertStorageRemoveAll(Handle : THandle): boolean;
function SBWinCertStorageAddCertificate(Handle : THandle; CertificateHandle : THandle) : boolean;
function SBWinCertStorageAddCertificateTo(Handle : THandle; CertificateHandle : THandle;
  StoreName : PChar) : boolean;
function SBWinCertStorageRemoveCertificate(Handle : THandle; Index : integer) : boolean;
function SBWinCertStorageGetCertificate(Handle : THandle; Index : integer) : THandle;
function SBWinCertStorageGetCount(Handle : THandle) : integer;
function SBWinCertStorageExportTo(Handle : THandle; ExportToHandle : THandle) : boolean;
function SBFileCertStorageCreate : THandle;
function SBFileCertStorageDestroy(Handle : THandle): boolean;
function SBFileCertStorageValidate(Handle : THandle; CertificateHandle : THandle;
  var Reason : integer) : integer;
function SBFileCertStorageSetFilename(Handle : THandle; Filename : PChar) : boolean;
function SBFileCertStorageAdd(Handle : THandle; CertificateHandle : THandle) : boolean;
function SBFileCertStorageRemove(Handle : THandle; Index : integer) : boolean;
function SBFileCertStorageGetCertificate(Handle : THandle; Index : integer) : THandle;
function SBFileCertStorageGetCount(Handle : THandle) : integer;
function SBFileCertStorageSaveToFile(Handle : THandle; Filename : PChar) : boolean;
function SBFileCertStorageExportTo(Handle : THandle; ExportToHandle : THandle) : boolean;
function SBMemoryCertStorageCreate : THandle;
function SBMemoryCertStorageDestroy(Handle : THandle) : boolean;
function SBMemoryCertStorageValidate(Handle : THandle; CertificateHandle : THandle;
  var Reason : integer) : integer;
function SBMemoryCertStorageAdd(Handle : THandle; CertificateHandle : THandle) : boolean;
function SBMemoryCertStorageRemove(Handle : THandle; Index : integer) : boolean;
function SBMemoryCertStorageGetCertificate(Handle : THandle; Index : integer) : THandle;
function SBMemoryCertStorageGetCount(Handle : THandle) : integer;
function SBMemoryCertStorageExportTo(Handle : THandle; ExportToHandle : THandle) : boolean;

{ Relative Distinguished Name interface }
function SBRelativeDistinguishedNameCreate : THandle;
function SBRelativeDistinguishedNameDestroy(Handle : THandle) : boolean;
function SBRelativeDistinguishedNameSetCount(Handle : THandle; Count : integer) : boolean;
function SBRelativeDistinguishedNameGetCount(Handle : THandle) : integer;
function SBRelativeDistinguishedNameSetOID(Handle : THandle; Index : integer;
  Buffer : pointer; Size : integer) : boolean;
function SBRelativeDistinguishedNameGetOID(Handle : THandle; Index : integer;
  Buffer : pointer; var Size : integer) : boolean;
function SBRelativeDistinguishedNameSetValue(Handle : THandle; Index : integer;
  Buffer : pointer; Size : integer) : boolean;
function SBRelativeDistinguishedNameGetValue(Handle : THandle; Index : integer;
  Buffer : pointer; var Size : integer) : boolean;
function SBRelativeDistinguishedNameRemove(Handle : THandle; Index : integer) : boolean;

{ Certification request interface }
function SBCertificateRequestCreate : THandle;
function SBCertificateRequestDestroy(Handle : THandle) : boolean;
function SBCertificateRequestLoadFromBuffer(Handle : THandle; Buffer : pointer;
  Size : integer) : boolean;
function SBCertificateRequestGenerate(Handle : THandle; PublicKeyAlgorithm :
  integer; Bits : integer; SignatureAlgorithm : integer) : boolean; 
function SBCertificateRequestSaveToBuffer(Handle : THandle; Buffer : pointer;
  var Size : integer) : boolean;
function SBCertificateRequestValidateSignature(Handle : THandle) : boolean; 
function SBCertificateRequestGetRSAParams(Handle : THandle; PublicModulus :
  pointer; var PublicModulusSize : integer; PublicExponent : pointer;
  var PublicExponentSize : integer) : boolean; 
function SBCertificateRequestGetDSAParams(Handle : THandle; P : pointer;
  var PSize : integer; Q : pointer; var QSize : integer; G : pointer; var GSize :
  integer; Y : pointer; var YSize : integer) : boolean; 
function SBCertificateRequestGetPrivateKey(Handle : THandle; Buffer : pointer;
  var Size : integer) : boolean; 
function SBCertificateRequestGetSubject(Handle : THandle) : THandle; 
function SBCertificateRequestGetAttributes(Handle : THandle) : THandle; 
function SBCertificateRequestGetPublicKeyAlgorithm(Handle : THandle) : integer; 
function SBCertificateRequestGetSignatureAlgorithm(Handle : THandle) : integer; 

{ Message Encryptor interface }
function SBMessageEncryptorCreate : THandle;
function SBMessageEncryptorDestroy(Handle : THandle) : boolean;
function SBMessageEncryptorSetCertStorage(Handle : THandle; StorageHandle :
  THandle) : boolean;
function SBMessageEncryptorGetCertStorage(Handle : THandle) : THandle;
function SBMessageEncryptorSetAlgorithm(Handle : THandle; Algorithm : integer) : boolean;
function SBMessageEncryptorGetAlgorithm(Handle : THandle) : integer;
function SBMessageEncryptorSetBitsInKey(Handle : THandle; Bits : integer) : boolean;
function SBMessageEncryptorGetBitsInKey(Handle : THandle) : integer;
function SBMessageEncryptorEncrypt(Handle : THandle; InBuffer : pointer; InSize :
  integer; OutBuffer : pointer; var OutSize : integer) : integer;

{ Message Decryptor interface }
function SBMessageDecryptorCreate : THandle;
function SBMessageDecryptorDestroy(Handle : THandle) : boolean;
function SBMessageDecryptorSetCertStorage(Handle : THandle; StorageHandle : THandle) : boolean;
function SBMessageDecryptorGetCertStorage(Handle : THandle) : THandle;
function SBMessageDecryptorGetAlgorithm(Handle : THandle) : integer;
function SBMessageDecryptorGetBitsInKey(Handle : THandle) : integer;
function SBMessageDecryptorDecrypt(Handle : THandle; InBuffer : pointer; InSize :
  integer; OutBuffer : pointer; var OutSize : integer) : integer;
function SBMessageDecryptorGetUsedCertificate(Handle : THandle) : integer;
function SBMessageDecryptorGetCertID(Handle : THandle; Index : integer) : THandle;
function SBMessageDecryptorGetCertIDCount(Handle : THandle) : integer;

{ Message Signer interface }
function SBMessageSignerCreate : THandle;
function SBMessageSignerDestroy(Handle : THandle) : boolean;
function SBMessageSignerSetHashAlgorithm(Handle : THandle; Algorithm : integer) : boolean;
function SBMessageSignerGetHashAlgorithm(Handle : THandle) : integer;
function SBMessageSignerSetCertStorage(Handle : THandle; StorageHandle : THandle) : boolean;
function SBMessageSignerGetCertStorage(Handle : THandle) : THandle;
function SBMessageSignerSign(Handle : THandle; InBuffer : pointer; InSize : integer;
  OutBuffer : pointer; var OutSize : integer) : integer;
function SBMessageSignerSignDetached(Handle : THandle; InBuffer: pointer;
  InSize : integer; OutBuffer: pointer; var OutSize : integer) : integer;
function SBMessageSignerGetAuthenticatedAttributes(Handle : THandle) : THandle;
function SBMessageSignerGetUnauthenticatedAttributes(Handle : THandle) : THandle;

{ Message Verifier interface }
function SBMessageVerifierCreate : THandle;
function SBMessageVerifierDestroy(Handle : THandle) : boolean;
function SBMessageVerifierSetCertStorage(Handle : THandle; StorageHandle : THandle): boolean;
function SBMessageVerifierGetCertStorage(Handle : Thandle) : THandle;
function SBMessageVerifierGetHashAlgorithm(Handle : THandle) : integer;
function SBMessageVerifierGetCertificates(Handle : THandle) : THandle;
function SBMessageVerifierVerify(Handle : THandle; InBuffer : pointer; InSize :
  integer; OutBuffer : pointer; var OutSize : integer) : integer;
function SBMessageVerifierVerifyDetached(Handle : THandle; InBuffer: pointer;
  InSize : integer; SigBuffer: pointer; SigSize : integer) : integer;
function SBMessageVerifierIsSignatureDetached(SigBuffer: pointer; SigSize: integer) : boolean;
function SBMessageVerifierGetCertID(Handle : THandle; Index: integer) : THandle;
function SBMessageVerifierGetCertIDCount(Handle : THandle) : integer;
function SBMessageVerifierGetAttributes(Handle : THandle) : THandle;

{ Random interface }
function SBRandomCreate : THandle;
function SBRandomDestroy(Handle : THandle) : boolean;
function SBRandomRandomize(Handle : THandle; Buffer : pointer; Size : integer): boolean;
function SBRandomGenerate(Handle : THandle; Buffer : pointer; Size : integer) : boolean;

{ PKCS7 Issuer interface }
function SBPKCS7IssuerGetIssuer(Handle: THandle) : THandle;
function SBPKCS7IssuerGetSerialNumber(Handle : THandle; Buffer : pointer; var Size : integer) : boolean;
function SBPKCS7IssuerSetSerialNumber(Handle : THandle; Buffer : pointer; Size : integer) : boolean;

{ ArrayList interface }
function SBArrayListGetCount(Handle : THandle) : integer;
function SBArrayListGetValue(Handle : THandle; Index : integer; Buffer : pointer;
  var Size : integer) : boolean;
function SBArrayListSetValue(Handle : THandle; Index : integer; Buffer : pointer;
  Size : integer) : boolean;
function SBArrayListAdd(Handle : THandle; Buffer : pointer; Size : integer) : integer;
function SBArrayListRemove(Handle : THandle; Index : integer) : boolean;

{ PKCS7 Attributes interface }
function SBPKCS7AttributesRemove(Handle : THandle; Index : integer) : boolean;
function SBPKCS7AttributesGetAttribute(Handle : THandle; Index : integer;
  Buffer : pointer; var Size : integer) : boolean;
function SBPKCS7AttributesSetAttribute(Handle : THandle; Index : integer;
  Buffer : pointer; Size : integer) : boolean;
function SBPKCS7AttributesGetValues(Handle : THandle; Index : integer) : THandle;
function SBPKCS7AttributesGetCount(Handle : THandle) : integer;
function SBPKCS7AttributesSetCount(Handle : THandle; Count : integer) : boolean;
function SBFormatAttributeValue(TagID : integer; InBuffer: pointer; InSize : integer;
  OutBuffer: pointer; var OutSize : integer) : boolean;
function SBUnformatAttributeValue(InBuffer: pointer; InSize : integer; OutBuffer:
  pointer; var OutSize : integer; var TagID : integer) : boolean;

{ Certificate Revocation Lists interface }
function SBCertificateRevocationListCreate : THandle;
function SBCertificateRevocationListDestroy(Handle: THandle) : boolean;
function SBCertificateRevocationListAdd(Handle : THandle; CertHandle : THandle): boolean;
function SBCertificateRevocationListRemoveByIndex(Handle : THandle; Index : integer): boolean;
function SBCertificateRevocationListRemove(Handle : THandle; CertHandle : THandle): boolean;
function SBCertificateRevocationListIsPresent(Handle: THandle; CertHandle : THandle) : boolean;
function SBCertificateRevocationListLoadFromBuffer(Handle : THandle; Buffer: pointer;
  Size : integer) : integer;
function SBCertificateRevocationListSaveToBuffer(Handle : THandle; Buffer: pointer;
  var Size : integer; CertHandle : THandle) : integer;
function SBCertificateRevocationListValidate(Handle : THandle; CertHandle : THandle) : integer;
function SBCertificateRevocationListGetIssuer(Handle : THandle) : THandle;
function SBCertificateRevocationListGetThisUpdate(Handle : THandle; var Date : TDateTime): boolean;
function SBCertificateRevocationListGetNextUpdate(Handle : THandle; var Date : TDateTime): boolean;
function SBCertificateRevocationListSetThisUpdate(Handle : THandle; Date : TDateTime): boolean;
function SBCertificateRevocationListSetNextUpdate(Handle : THandle; Date : TDateTime): boolean;
function SBCertificateRevocationListGetSignatureAlgorithm(Handle : THandle) : integer;
function SBCertificateRevocationListGetItem(Handle : THandle; Index: integer) : THandle;
function SBCertificateRevocationListGetCount(Handle : THandle) : integer;
function SBRevocationItemGetSerialNumber(Handle : THandle; Buffer: pointer;
  var Size : integer) : boolean;
function SBRevocationItemSetSerialNumber(Handle : THandle; Buffer: pointer;
  Size : integer) : boolean;
function SBRevocationItemGetRevocationDate(Handle : THandle; var Date :
  TDateTime) : boolean;
function SBRevocationItemSetRevocationDate(Handle : THandle; Date : TDateTime) : boolean;

implementation

type
  TSBSetLicenseKey = procedure(Key : PChar); stdcall;

  TSBX509GetVersion = function (Handle : THandle) : integer; stdcall;
  TSBX509GetSerialNumber = function (Handle : THandle; SerialNumber : Pointer; var Size : longint) : longbool; stdcall;
  TSBX509Create = function () : THandle; stdcall;
  TSBX509LoadFromBuffer = function (Handle : THandle; Data : Pointer; Size : longint) : longbool; stdcall;
  TSBX509GetCertificateBinary = function (Handle : THandle; Buffer : Pointer; var Size : longint) : longbool; stdcall;
  TSBX509GetSignature = function (Handle : THandle; Buffer : Pointer; var Size : longint) : longbool; stdcall;
  TSBX509GetIssuer = function (Handle : THandle; Buffer : Pointer; var Size : longint) : longbool; stdcall;
  TSBX509GetValidFrom = function(Handle : THandle; var Date : TDateTime) : longbool; stdcall;
  TSBX509GetValidTo = function(Handle : THandle; var Date : TDateTime) : longbool; stdcall;
  TSBX509GetSubject = function (Handle : THandle; Buffer : Pointer; var Size : longint) : longbool; stdcall;
  TSBX509GetSignatureAlgorithm = function(Handle : THandle) : integer; stdcall;
  TSBX509GetPublicKeyAlgorithm = function(Handle : THandle) : integer; stdcall;
  TSBX509GetRSAParams = function(Handle : THandle; RSAModulus : Pointer; var RSAModulusSize : integer;
    RSAPublicKey : Pointer; var RSAPublicKeySize : integer) : longbool; stdcall;
  TSBX509GetDSSSignatureParams = function(Handle : THandle; DSSS : Pointer; var DSSSSize : integer;
    DSSR : Pointer; var DSSRSize : integer) : longbool; stdcall;
  TSBX509GetDSSParams = function(Handle : THandle; DSSP : Pointer; var DSSPSize : integer;
    DSSQ : Pointer; var DSSQSize : integer; DSSG : Pointer; var DSSGSize : integer;
    DSSY : Pointer; var DSSYSize : integer) : longbool; stdcall;
  TSBX509GetDHParams = function(Handle : THandle; DHP : Pointer; var DHPSize : integer;
    DHG : Pointer; var DHGSize : integer; DHY : Pointer; var DHYSize : integer) : longbool; stdcall;
  TSBX509SelfSigned = function(Handle : THandle) : longbool; stdcall;
  TSBX509Validate = function(Handle : THandle) : longbool; stdcall;
  TSBX509ValidateRSA = function(Handle : THandle; RSAModulus : pointer; RSAModulusSize : integer;
    RSAPublicKey : pointer; RSAPublicKeySize : integer) : longbool; stdcall;
  TSBX509ValidateDSS = function(Handle : THandle; DSSP : pointer; DSSPSize : integer;
    DSSQ : pointer; DSSQSize : integer; DSSG : pointer; DSSGSize : integer;
    DSSY : pointer; DSSYSize : integer) : longbool; stdcall;
  TSBX509ValidateWithCA = function(Handle : THandle; CACertificate : pointer;
    CACertificateSize : integer) : longbool; stdcall;
  TSBX509SetIssuer = function(Handle : THandle; Fields : pointer) : longbool; stdcall;
  TSBX509SetSubject = function(Handle : THandle; Fields : pointer) : longbool; stdcall;
  TSBX509SaveToBuffer = function (Handle : THandle; Buffer : pointer; var Size : longint) : longbool; stdcall;
  TSBX509SaveKeyToBuffer = function(Handle : THandle; Buffer : pointer; var Size : longint) : longbool; stdcall;
  TSBX509Generate = function(Handle : THandle; Algorithm : integer; DWordsInEncryptKey : integer) : longbool; stdcall;
  TSBX509CAGenerate = function(Handle : THandle; Algorithm : integer; DWordsInEncryptKey : integer) : longbool; stdcall;
  TSBX509SetCACertificate = function(Handle : THandle; Buffer : pointer; Size : longint) : longbool; stdcall;
  TSBX509SetCAPrivateKey = function(Handle : THandle; Buffer : pointer; Size : longint) : longbool; stdcall;
  TSBX509Destroy = function(Handle : THandle) : longbool; stdcall;
  TSBX509SetValidFrom = function(Handle : THandle; Date : TDateTime) : longbool; stdcall;
  TSBX509SetValidTo = function(Handle : THandle; Date : TDateTime) : longbool; stdcall;
  TSBX509LoadKeyFromBuffer = function(Handle : THandle; Buffer : pointer; Size : longint) : longbool; stdcall;
  TSBX509LoadFromBufferPEM = function(Handle : THandle; Buffer : pointer; Size : longint; PassPhrase : PChar) : longbool; stdcall;
  TSBX509LoadKeyFromBufferPEM = function(Handle : THandle; Buffer : pointer; Size : longint; PassPhrase : PChar) : longbool; stdcall;
  TSBX509SaveToBufferPEM = function(Handle : THandle; Buffer : pointer; var Size : longint; PassPhrase : PChar) : longbool; stdcall;
  TSBX509SaveKeyToBufferPEM = function(Handle : THandle; Buffer : pointer; var Size : longint; PassPhrase : PChar) : longbool; stdcall;
  TSBX509Clone = function(Handle : THandle; DestHandle : THandle) : longbool; stdcall;
  TSBX509GetHashMD5 = function(Handle : THandle; Buffer : pointer) : longbool; stdcall;
  TSBX509GetHashSHA1 = function(Handle : THandle; Buffer : pointer) : longbool; stdcall;
  TSBX509GetPublicKeySize = function(Handle : THandle) : integer; stdcall;
  TSBX509GetPublicKeyBlob = function(Handle : THandle; Buffer : pointer; var Size :
    integer) : longbool; stdcall;
  TSBX509PrivateKeyExists = function(Handle : THandle) : longbool; stdcall;
  TSBGetAvailableSystemStores = function(Buffer : pointer; var Size : integer) : longbool; stdcall;
  TSBGetAvailablePhysicalStores = function(SystemStore : PChar; Buffer : pointer;
    var Size : integer) : longbool; stdcall;
  TSBCertStorageFindByHashMD5 = function(Handle : THandle; Hash : pointer) :integer; stdcall;
  TSBCertStorageFindByHashSHA1 = function(Handle : THandle; Hash : pointer) :integer; stdcall;
  TSBCertStorageGetIssuerCertificate = function(Handle : THandle; CertHandle : THandle) :
    integer; stdcall;
  TSBWinCertStorageCreate = function : THandle; stdcall;
  TSBWinCertStorageDestroy = function(Handle : THandle): longbool; stdcall;
  TSBWinCertStorageValidate = function(Handle : THandle; CertificateHandle : THandle;
    var Reason : integer) : integer; stdcall;
  TSBWinCertStorageAdd = function(Handle : THandle; StoreName : PChar) : longbool; stdcall;
  TSBWinCertStorageRemove = function(Handle : THandle; StoreName : PChar) : longbool; stdcall;
  TSBWinCertStorageRemoveAll = function(Handle : THandle): longbool; stdcall;
  TSBWinCertStorageAddCertificate = function(Handle : THandle; CertificateHandle : THandle) : longbool; stdcall;
  TSBWinCertStorageAddCertificateTo = function(Handle : THandle; CertificateHandle : THandle;
    StoreName : PChar) : longbool; stdcall;
  TSBWinCertStorageRemoveCertificate = function(Handle : THandle; Index : integer) : longbool; stdcall;
  TSBWinCertStorageGetCertificate = function(Handle : THandle; Index : integer) : THandle; stdcall;
  TSBWinCertStorageGetCount = function(Handle : THandle) : integer; stdcall;
  TSBWinCertStorageExportTo = function(Handle : THandle; ExportToHandle : THandle) : longbool; stdcall;
  TSBFileCertStorageCreate = function : THandle; stdcall;
  TSBFileCertStorageDestroy = function(Handle : THandle): longbool; stdcall;
  TSBFileCertStorageValidate = function(Handle : THandle; CertificateHandle : THandle;
    var Reason : integer) : integer; stdcall;
  TSBFileCertStorageSetFilename = function(Handle : THandle; Filename : PChar) : longbool; stdcall;
  TSBFileCertStorageAdd = function(Handle : THandle; CertificateHandle : THandle) : longbool; stdcall;
  TSBFileCertStorageRemove = function(Handle : THandle; Index : integer) : longbool; stdcall;
  TSBFileCertStorageGetCertificate = function(Handle : THandle; Index : integer) : THandle; stdcall;
  TSBFileCertStorageGetCount = function(Handle : THandle) : integer; stdcall;
  TSBFileCertStorageSaveToFile = function(Handle : THandle; Filename : PChar) : longbool; stdcall;
  TSBFileCertStorageExportTo = function(Handle : THandle; ExportToHandle : THandle) : longbool; stdcall;
  TSBMemoryCertStorageCreate = function : THandle; stdcall;
  TSBMemoryCertStorageDestroy = function(Handle : THandle) : longbool; stdcall;
  TSBMemoryCertStorageValidate = function(Handle : THandle; CertificateHandle : THandle;
    var Reason : integer) : integer; stdcall;
  TSBMemoryCertStorageAdd = function(Handle : THandle; CertificateHandle : THandle) : longbool; stdcall;
  TSBMemoryCertStorageRemove = function(Handle : THandle; Index : integer) : longbool; stdcall;
  TSBMemoryCertStorageGetCertificate = function(Handle : THandle; Index : integer) : THandle; stdcall;
  TSBMemoryCertStorageGetCount = function(Handle : THandle) : integer; stdcall;
  TSBMemoryCertStorageExportTo = function(Handle : THandle; ExportToHandle : THandle) : longbool; stdcall;
  TSBX509LoadFromBufferPFX = function (Handle : THandle; Buffer : pointer; Size : integer;
    Password : PChar) : integer; stdcall;
  TSBX509SaveToBufferPFX = function(Handle : THandle; Buffer : pointer; var Size : integer;
    Password : PChar; KeyEncryptionAlgorithm : integer; CertEncryptionAlgorithm :
    integer) : integer; stdcall;
  TSBX509Sign = function(Handle : THandle; InBuffer : pointer; InSize : integer;
    OutBuffer : pointer; var OutSize : integer) : integer; stdcall;
  TSBX509Verify = function(Handle : THandle; InBuffer : pointer; InSize : integer;
    Signature : pointer; SignatureSize : integer) : integer; stdcall;
  TSBX509GenerateByRequest = function(Handle : THandle; RequestHandle : THandle;
    NewCertHandle : THandle) : longbool; stdcall;
  TSBRelativeDistinguishedNameCreate = function : THandle; stdcall;
  TSBRelativeDistinguishedNameDestroy = function(Handle : THandle) : longbool; stdcall;
  TSBRelativeDistinguishedNameSetCount = function(Handle : THandle; Count : integer) : longbool; stdcall;
  TSBRelativeDistinguishedNameGetCount = function(Handle : THandle) : integer; stdcall;
  TSBRelativeDistinguishedNameSetOID = function(Handle : THandle; Index : integer;
    Buffer : pointer; Size : integer) : longbool; stdcall;
  TSBRelativeDistinguishedNameGetOID = function(Handle : THandle; Index : integer;
    Buffer : pointer; var Size : integer) : longbool; stdcall;
  TSBRelativeDistinguishedNameSetValue = function(Handle : THandle; Index : integer;
    Buffer : pointer; Size : integer) : longbool; stdcall;
  TSBRelativeDistinguishedNameGetValue = function(Handle : THandle; Index : integer;
    Buffer : pointer; var Size : integer) : longbool; stdcall;
  TSBRelativeDistinguishedNameRemove = function(Handle : THandle; Index : integer) : longbool; stdcall;
  TSBCertificateRequestCreate = function : THandle; stdcall;
  TSBCertificateRequestDestroy = function(Handle : THandle) : longbool; stdcall;
  TSBCertificateRequestLoadFromBuffer = function(Handle : THandle; Buffer : pointer;
    Size : integer) : longbool; stdcall;
  TSBCertificateRequestGenerate = function(Handle : THandle; PublicKeyAlgorithm :
    integer; Bits : integer; SignatureAlgorithm : integer) : longbool; stdcall;
  TSBCertificateRequestSaveToBuffer = function(Handle : THandle; Buffer : pointer;
    var Size : integer) : longbool; stdcall;
  TSBCertificateRequestValidateSignature = function(Handle : THandle) : longbool; stdcall;
  TSBCertificateRequestGetRSAParams = function(Handle : THandle; PublicModulus :
    pointer; var PublicModulusSize : integer; PublicExponent : pointer;
    var PublicExponentSize : integer) : longbool; stdcall;
  TSBCertificateRequestGetDSAParams = function(Handle : THandle; P : pointer;
    var PSize : integer; Q : pointer; var QSize : integer; G : pointer; var GSize :
    integer; Y : pointer; var YSize : integer) : longbool; stdcall;
  TSBCertificateRequestGetPrivateKey = function(Handle : THandle; Buffer : pointer;
    var Size : integer) : longbool; stdcall;
  TSBCertificateRequestGetSubject = function(Handle : THandle) : THandle; stdcall;
  TSBCertificateRequestGetAttributes = function(Handle : THandle) : THandle; stdcall;
  TSBCertificateRequestGetPublicKeyAlgorithm = function(Handle : THandle) : integer; stdcall;
  TSBCertificateRequestGetSignatureAlgorithm = function(Handle : THandle) : integer; stdcall;
  TSBMessageEncryptorCreate = function : THandle; stdcall;
  TSBMessageEncryptorDestroy = function(Handle : THandle) : longbool; stdcall;
  TSBMessageEncryptorSetCertStorage = function(Handle : THandle; StorageHandle :
    THandle) : longbool; stdcall;
  TSBMessageEncryptorGetCertStorage = function(Handle : THandle) : THandle; stdcall;
  TSBMessageEncryptorSetAlgorithm = function(Handle : THandle; Algorithm : integer) : longbool; stdcall;
  TSBMessageEncryptorGetAlgorithm = function(Handle : THandle) : integer; stdcall;
  TSBMessageEncryptorSetBitsInKey = function(Handle : THandle; Bits : integer) : longbool; stdcall;
  TSBMessageEncryptorGetBitsInKey = function(Handle : THandle) : integer; stdcall;
  TSBMessageEncryptorEncrypt = function(Handle : THandle; InBuffer : pointer; InSize :
    integer; OutBuffer : pointer; var OutSize : integer) : integer; stdcall;
  TSBMessageDecryptorCreate = function : THandle; stdcall;
  TSBMessageDecryptorDestroy = function(Handle : THandle) : longbool; stdcall;
  TSBMessageDecryptorSetCertStorage = function(Handle : THandle; StorageHandle : THandle) : longbool; stdcall;
  TSBMessageDecryptorGetCertStorage = function(Handle : THandle) : THandle; stdcall;
  TSBMessageDecryptorGetAlgorithm = function(Handle : THandle) : integer; stdcall;
  TSBMessageDecryptorGetBitsInKey = function(Handle : THandle) : integer; stdcall;
  TSBMessageDecryptorDecrypt = function(Handle : THandle; InBuffer : pointer; InSize :
    integer; OutBuffer : pointer; var OutSize : integer) : integer; stdcall;
  TSBMessageSignerCreate = function : THandle; stdcall;
  TSBMessageSignerDestroy = function(Handle : THandle) : longbool; stdcall;
  TSBMessageSignerSetHashAlgorithm = function(Handle : THandle; Algorithm : integer) : longbool; stdcall;
  TSBMessageSignerGetHashAlgorithm = function(Handle : THandle) : integer; stdcall;
  TSBMessageSignerSetCertStorage = function(Handle : THandle; StorageHandle : THandle) : longbool; stdcall;
  TSBMessageSignerGetCertStorage = function(Handle : THandle) : THandle; stdcall;
  TSBMessageSignerSign = function(Handle : THandle; InBuffer : pointer; InSize : integer;
    OutBuffer : pointer; var OutSize : integer) : integer; stdcall;
  TSBMessageVerifierCreate = function : THandle; stdcall;
  TSBMessageVerifierDestroy = function(Handle : THandle) : longbool; stdcall;
  TSBMessageVerifierSetCertStorage = function(Handle : THandle; StorageHandle : THandle): longbool; stdcall;
  TSBMessageVerifierGetCertStorage = function(Handle : Thandle) : THandle; stdcall;
  TSBMessageVerifierGetHashAlgorithm = function(Handle : THandle) : integer; stdcall;
  TSBMessageVerifierGetCertificates = function(Handle : THandle) : THandle; stdcall;
  TSBMessageVerifierVerify = function(Handle : THandle; InBuffer : pointer; InSize :
    integer; OutBuffer : pointer; var OutSize : integer) : integer; stdcall;
  TSBRandomCreate = function : THandle; stdcall;
  TSBRandomDestroy = function(Handle : THandle) : longbool; stdcall;
  TSBRandomRandomize = function(Handle : THandle; Buffer : pointer; Size : integer): longbool; stdcall;
  TSBRandomGenerate = function(Handle : THandle; Buffer : pointer; Size : integer) : longbool; stdcall;
  TSBX509GetIssuerRDN = function (Handle : THandle) : THandle; stdcall;
  TSBX509GetSubjectRDN = function(Handle : THandle) : THandle; stdcall;
  TSBX509SignDetached = function(Handle : THandle; InBuffer: pointer; InSize : integer;
    OutBuffer: pointer; var OutSize : integer) : integer; stdcall;
  TSBMessageSignerSignDetached = function(Handle : THandle; InBuffer: pointer;
    InSize : integer; OutBuffer: pointer; var OutSize : integer) : integer; stdcall;
  TSBMessageVerifierVerifyDetached = function(Handle : THandle; InBuffer: pointer;
    InSize : integer; SigBuffer: pointer; SigSize : integer) : integer; stdcall;
  TSBMessageVerifierIsSignatureDetached = function(SigBuffer: pointer; SigSize: integer) : longbool; stdcall;
  TSBMessageVerifierGetCertID = function(Handle : THandle; Index: integer) : THandle; stdcall;
  TSBMessageVerifierGetCertIDCount = function(Handle : THandle) : integer; stdcall;
  TSBMessageVerifierGetAttributes = function(Handle : THandle) : THandle; stdcall;
  TSBMessageSignerGetAuthenticatedAttributes = function(Handle : THandle) : THandle; stdcall;
  TSBMessageSignerGetUnauthenticatedAttributes = function(Handle : THandle) : THandle; stdcall;
  TSBPKCS7IssuerGetIssuer = function(Handle: THandle) : THandle; stdcall;
  TSBPKCS7IssuerGetSerialNumber = function(Handle : THandle; Buffer : pointer; var Size : integer) : longbool; stdcall;
  TSBPKCS7IssuerSetSerialNumber = function(Handle : THandle; Buffer : pointer; Size : integer) : longbool; stdcall;
  TSBArrayListGetCount = function(Handle : THandle) : integer; stdcall;
  TSBArrayListGetValue = function(Handle : THandle; Index : integer; Buffer : pointer;
    var Size : integer) : longbool; stdcall;
  TSBArrayListSetValue = function(Handle : THandle; Index : integer; Buffer : pointer;
    Size : integer) : longbool; stdcall;
  TSBArrayListAdd = function(Handle : THandle; Buffer : pointer; Size : integer) : integer; stdcall;
  TSBArrayListRemove = function(Handle : THandle; Index : integer) : longbool; stdcall;
  TSBPKCS7AttributesRemove = function(Handle : THandle; Index : integer) : longbool; stdcall;
  TSBPKCS7AttributesGetAttribute = function(Handle : THandle; Index : integer;
    Buffer : pointer; var Size : integer) : longbool; stdcall;
  TSBPKCS7AttributesSetAttribute = function(Handle : THandle; Index : integer;
    Buffer : pointer; Size : integer) : longbool; stdcall;
  TSBPKCS7AttributesGetValues = function(Handle : THandle; Index : integer) : THandle; stdcall;
  TSBPKCS7AttributesGetCount = function(Handle : THandle) : integer; stdcall;
  TSBPKCS7AttributesSetCount = function(Handle : THandle; Count : integer) : longbool; stdcall;
  TSBMessageDecryptorGetUsedCertificate = function(Handle : THandle) : integer; stdcall;
  TSBMessageDecryptorGetCertID = function(Handle : THandle; Index : integer) : THandle; stdcall;
  TSBMessageDecryptorGetCertIDCount = function(Handle : THandle) : integer; stdcall;
  TSBFormatAttributeValue = function(TagID : integer; InBuffer: pointer; InSize : integer;
    OutBuffer: pointer; var OutSize : integer) : longbool; stdcall;
  TSBUnformatAttributeValue = function(InBuffer: pointer; InSize : integer; OutBuffer:
    pointer; var OutSize : integer; var TagID : integer) : longbool; stdcall;

  TSBCertificateRevocationListCreate = function : THandle; stdcall; 
  TSBCertificateRevocationListDestroy = function(Handle: THandle) : longbool; stdcall;
  TSBCertificateRevocationListAdd = function(Handle : THandle; CertHandle : THandle): longbool; stdcall;
  TSBCertificateRevocationListRemoveByIndex = function(Handle : THandle; Index : integer): longbool; stdcall;
  TSBCertificateRevocationListRemove = function(Handle : THandle; CertHandle : THandle): longbool; stdcall;
  TSBCertificateRevocationListIsPresent = function(Handle: THandle; CertHandle : THandle) : longbool; stdcall;
  TSBCertificateRevocationListLoadFromBuffer = function(Handle : THandle; Buffer: pointer;
    Size : integer) : integer; stdcall;
  TSBCertificateRevocationListSaveToBuffer = function(Handle : THandle; Buffer: pointer;
    var Size : integer; CertHandle : THandle) : integer; stdcall;
  TSBCertificateRevocationListValidate = function(Handle : THandle; CertHandle : THandle) : integer; stdcall;
  TSBCertificateRevocationListGetIssuer = function(Handle : THandle) : THandle; stdcall;
  TSBCertificateRevocationListGetThisUpdate = function(Handle : THandle; var Date : TDateTime): longbool; stdcall;
  TSBCertificateRevocationListGetNextUpdate = function(Handle : THandle; var Date : TDateTime): longbool; stdcall;
  TSBCertificateRevocationListSetThisUpdate = function(Handle : THandle; Date : TDateTime): longbool; stdcall;
  TSBCertificateRevocationListSetNextUpdate = function(Handle : THandle; Date : TDateTime): longbool; stdcall;
  TSBCertificateRevocationListGetSignatureAlgorithm = function(Handle : THandle) : integer; stdcall;
  TSBCertificateRevocationListGetItem = function(Handle : THandle; Index: integer) : THandle; stdcall;
  TSBCertificateRevocationListGetCount = function(Handle : THandle) : integer; stdcall;
  TSBRevocationItemGetSerialNumber = function(Handle : THandle; Buffer: pointer;
    var Size : integer) : longbool; stdcall;
  TSBRevocationItemSetSerialNumber = function(Handle : THandle; Buffer: pointer;
    Size : integer) : longbool; stdcall;
  TSBRevocationItemGetRevocationDate = function(Handle : THandle; var Date :
    TDateTime) : longbool; stdcall;
  TSBRevocationItemSetRevocationDate = function(Handle : THandle; Date : TDateTime) : longbool; stdcall;

const
  DllName = 'PKIBBox.dll';
  DllRoutineCount = 191;
  DllRoutineNames : array[0..DllRoutineCount - 1] of PChar = (
    'SBX509GetVersion',
    'SBX509GetSerialNumber',
    'SBX509Create',
    'SBX509LoadFromBuffer',
    'SBX509GetCertificateBinary',
    'SBX509GetSignature',
    'SBX509GetIssuer',
    'SBX509GetValidFrom',
    'SBX509GetValidTo',
    'SBX509GetSubject',
    'SBX509GetSignatureAlgorithm',
    'SBX509GetPublicKeyAlgorithm',
    'SBX509GetRSAParams',
    'SBX509GetDSSSignatureParams',
    'SBX509GetDSSParams',
    'SBX509GetDHParams',
    'SBX509SelfSigned',
    'SBX509Validate',
    'SBX509ValidateRSA',
    'SBX509ValidateDSS',
    'SBX509ValidateWithCA',
    'SBX509SetIssuer',
    'SBX509SetSubject',
    'SBX509SaveToBuffer',
    'SBX509SaveKeyToBuffer',
    'SBX509Generate',
    'SBX509CAGenerate',
    'SBX509SetCACertificate',
    'SBX509SetCAPrivateKey',
    'SBX509Destroy',
    'SBX509SetValidFrom',
    'SBX509SetValidTo',
    'SBX509LoadKeyFromBuffer',
    'SBX509LoadFromBufferPEM',
    'SBX509LoadKeyFromBufferPEM',
    'SBX509SaveToBufferPEM',
    'SBX509SaveKeyToBufferPEM',
    'SBX509Clone',
    'SBX509GetHashMD5',
    'SBX509GetHashSHA1',
    'SBX509GetPublicKeySize',
    'SBX509GetPublicKeyBlob',
    'SBX509PrivateKeyExists',
    'SBX509LoadFromBufferPFX',
    'SBX509SaveToBufferPFX',
    'SBX509Sign',
    'SBX509Verify',
    'SBX509GenerateByRequest',
    'SBX509GetIssuerRDN',
    'SBX509GetSubjectRDN',
    'SBX509SignDetached',
    'SBGetAvailableSystemStores',
    'SBGetAvailablePhysicalStores',
    'SBCertStorageFindByHashMD5',
    'SBCertStorageFindByHashSHA1',
    'SBCertStorageGetIssuerCertificate',
    'SBWinCertStorageCreate',
    'SBWinCertStorageDestroy',
    'SBWinCertStorageValidate',
    'SBWinCertStorageAdd',
    'SBWinCertStorageRemove',
    'SBWinCertStorageRemoveAll',
    'SBWinCertStorageAddCertificate',
    'SBWinCertStorageAddCertificateTo',
    'SBWinCertStorageRemoveCertificate',
    'SBWinCertStorageGetCertificate',
    'SBWinCertStorageGetCount',
    'SBWinCertStorageExportTo',
    'SBFileCertStorageCreate',
    'SBFileCertStorageDestroy',
    'SBFileCertStorageValidate',
    'SBFileCertStorageSetFilename',
    'SBFileCertStorageAdd',
    'SBFileCertStorageRemove',
    'SBFileCertStorageGetCertificate',
    'SBFileCertStorageGetCount',
    'SBFileCertStorageSaveToFile',
    'SBFileCertStorageExportTo',
    'SBMemoryCertStorageCreate',
    'SBMemoryCertStorageDestroy',
    'SBMemoryCertStorageValidate',
    'SBMemoryCertStorageAdd',
    'SBMemoryCertStorageRemove',
    'SBMemoryCertStorageGetCertificate',
    'SBMemoryCertStorageGetCount',
    'SBMemoryCertStorageExportTo',
    'SBRelativeDistinguishedNameCreate',
    'SBRelativeDistinguishedNameDestroy',
    'SBRelativeDistinguishedNameSetCount',
    'SBRelativeDistinguishedNameGetCount',
    'SBRelativeDistinguishedNameSetOID',
    'SBRelativeDistinguishedNameGetOID',
    'SBRelativeDistinguishedNameSetValue',
    'SBRelativeDistinguishedNameGetValue',
    'SBRelativeDistinguishedNameRemove',
    'SBCertificateRequestCreate',
    'SBCertificateRequestDestroy',
    'SBCertificateRequestLoadFromBuffer',
    'SBCertificateRequestGenerate',
    'SBCertificateRequestSaveToBuffer',
    'SBCertificateRequestValidateSignature',
    'SBCertificateRequestGetRSAParams',
    'SBCertificateRequestGetDSAParams',
    'SBCertificateRequestGetPrivateKey',
    'SBCertificateRequestGetSubject',
    'SBCertificateRequestGetAttributes',
    'SBCertificateRequestGetPublicKeyAlgorithm',
    'SBCertificateRequestGetSignatureAlgorithm',
    'SBMessageEncryptorCreate',
    'SBMessageEncryptorDestroy',
    'SBMessageEncryptorSetCertStorage',
    'SBMessageEncryptorGetCertStorage',
    'SBMessageEncryptorSetAlgorithm',
    'SBMessageEncryptorGetAlgorithm',
    'SBMessageEncryptorSetBitsInKey',
    'SBMessageEncryptorGetBitsInKey',
    'SBMessageEncryptorEncrypt',
    'SBMessageDecryptorCreate',
    'SBMessageDecryptorDestroy',
    'SBMessageDecryptorSetCertStorage',
    'SBMessageDecryptorGetCertStorage',
    'SBMessageDecryptorGetAlgorithm',
    'SBMessageDecryptorGetBitsInKey',
    'SBMessageDecryptorDecrypt',
    'SBMessageDecryptorGetUsedCertificate',
    'SBMessageDecryptorGetCertID',
    'SBMessageDecryptorGetCertIDCount',
    'SBMessageSignerCreate',
    'SBMessageSignerDestroy',
    'SBMessageSignerSetHashAlgorithm',
    'SBMessageSignerGetHashAlgorithm',
    'SBMessageSignerSetCertStorage',
    'SBMessageSignerGetCertStorage',
    'SBMessageSignerSign',
    'SBMessageSignerSignDetached',
    'SBMessageSignerGetAuthenticatedAttributes',
    'SBMessageSignerGetUnauthenticatedAttributes',
    'SBMessageVerifierCreate',
    'SBMessageVerifierDestroy',
    'SBMessageVerifierSetCertStorage',
    'SBMessageVerifierGetCertStorage',
    'SBMessageVerifierGetHashAlgorithm',
    'SBMessageVerifierGetCertificates',
    'SBMessageVerifierVerify',
    'SBMessageVerifierVerifyDetached',
    'SBMessageVerifierIsSignatureDetached',
    'SBMessageVerifierGetCertID',
    'SBMessageVerifierGetCertIDCount',
    'SBMessageVerifierGetAttributes',
    'SBRandomCreate',
    'SBRandomDestroy',
    'SBRandomRandomize',
    'SBRandomGenerate',
    'SBPKCS7IssuerGetIssuer',
    'SBPKCS7IssuerGetSerialNumber',
    'SBPKCS7IssuerSetSerialNumber',
    'SBArrayListGetCount',
    'SBArrayListGetValue',
    'SBArrayListSetValue',
    'SBArrayListAdd',
    'SBArrayListRemove',
    'SBPKCS7AttributesRemove',
    'SBPKCS7AttributesGetAttribute',
    'SBPKCS7AttributesSetAttribute',
    'SBPKCS7AttributesGetValues',
    'SBPKCS7AttributesGetCount',
    'SBPKCS7AttributesSetCount',
    'SBFormatAttributeValue',
    'SBUnformatAttributeValue',
    'SBCertificateRevocationListCreate',
    'SBCertificateRevocationListDestroy',
    'SBCertificateRevocationListAdd',
    'SBCertificateRevocationListRemoveByIndex',
    'SBCertificateRevocationListRemove',
    'SBCertificateRevocationListIsPresent',
    'SBCertificateRevocationListLoadFromBuffer',
    'SBCertificateRevocationListSaveToBuffer',
    'SBCertificateRevocationListValidate',
    'SBCertificateRevocationListGetIssuer',
    'SBCertificateRevocationListGetThisUpdate',
    'SBCertificateRevocationListGetNextUpdate',
    'SBCertificateRevocationListSetThisUpdate',
    'SBCertificateRevocationListSetNextUpdate',
    'SBCertificateRevocationListGetSignatureAlgorithm',
    'SBCertificateRevocationListGetItem',
    'SBCertificateRevocationListGetCount',
    'SBRevocationItemGetSerialNumber',
    'SBRevocationItemSetSerialNumber',
    'SBRevocationItemGetRevocationDate',
    'SBRevocationItemSetRevocationDate',
    'SetLicenseKey'
  );

var
  DllHandle : THandle;
  DllRefs : integer = 0;
  DllRoutines : array[0..DllRoutineCount - 1] of pointer;

function SBPKIInitialize: boolean;
var
  I : integer;
begin
  if InterlockedExchange(DllRefs, DllRefs) > 0 then
  begin
    InterlockedIncrement(DllRefs);
    Result := True
  end
  else
  begin
    Result := False;
    // loading the library
    DllHandle := LoadLibrary(DllName);
    if DllHandle = 0 then exit;
    InterlockedIncrement(DllRefs);
    // initializing links to routines
    for I := 0 to DllRoutineCount - 1 do
    begin
      DllRoutines[I] := GetProcAddress(DllHandle, DllRoutineNames[I]);
      if DllRoutines[I] = nil then
      begin
        SBPKIFinalize;
        exit;
      end;
    end;
    Result := True;
  end;
end;

procedure SBPKIFinalize;
begin
  if InterlockedExchange(DllRefs, DllRefs) > 0 then
    if InterlockedDecrement(DllRefs) = 0 then
    begin
      FreeLibrary(DllHandle);
      DllHandle := 0;
    end;
end;

function SBPKIIsInitialized: boolean;
begin
  Result := (InterlockedExchange(DllRefs, DllRefs) > 0) and (DllHandle <> 0); 
end;

{ ElX509Certificate functions }
function SBX509GetVersion(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBX509GetVersion(DllRoutines[0])(Handle);
end;

function SBX509GetSerialNumber(Handle : THandle; SerialNumber : Pointer; var Size : longint) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetSerialNumber(DllRoutines[1])(Handle, SerialNumber, Size);
end;

function SBX509Create : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBX509Create(DllRoutines[2])();
end;

function SBX509LoadFromBuffer(Handle : THandle; Data : Pointer; Size : longint) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509LoadFromBuffer(DllRoutines[3])(Handle, Data, Size);
end;

function SBX509GetCertificateBinary(Handle : THandle; Buffer : Pointer; var Size : longint) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetCertificateBinary(DllRoutines[4])(Handle, Buffer, Size);
end;

function SBX509GetSignature(Handle : THandle; Buffer : Pointer; var Size : longint) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetSignature(DllRoutines[5])(Handle, Buffer, Size);
end;

function SBX509GetIssuer(Handle : THandle; Buffer : Pointer; var Size : longint) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetIssuer(DllRoutines[6])(Handle, Buffer, Size);
end;

function SBX509GetValidFrom(Handle : THandle; var Date : TDateTime) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetValidFrom(DllRoutines[7])(Handle, Date);
end;

function SBX509GetValidTo(Handle : THandle; var Date : TDateTime) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetValidTo(DllRoutines[8])(Handle, Date);
end;

function SBX509GetSubject(Handle : THandle; Buffer : Pointer; var Size : longint) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetSubject(DllRoutines[9])(Handle, Buffer, Size);
end;

function SBX509GetSignatureAlgorithm(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBX509GetSignatureAlgorithm(DllRoutines[10])(Handle);
end;

function SBX509GetPublicKeyAlgorithm(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBX509GetPublicKeyAlgorithm(DllRoutines[11])(Handle);
end;

function SBX509GetRSAParams(Handle : THandle; RSAModulus : Pointer; var RSAModulusSize : integer;
  RSAPublicKey : Pointer; var RSAPublicKeySize : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetRSAParams(DllRoutines[12])(Handle, RSAModulus, RSAModulusSize,
      RSAPublicKey, RSAPublicKeySize);
end;

function SBX509GetDSSSignatureParams(Handle : THandle; DSSS : Pointer; var DSSSSize : integer;
  DSSR : Pointer; var DSSRSize : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetDSSSignatureParams(DllRoutines[13])(Handle, DSSS, DSSSSize,
      DSSR, DSSRSize);
end;

function SBX509GetDSSParams(Handle : THandle; DSSP : Pointer; var DSSPSize : integer;
  DSSQ : Pointer; var DSSQSize : integer; DSSG : Pointer; var DSSGSize : integer;
  DSSY : Pointer; var DSSYSize : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetDSSParams(DllRoutines[14])(Handle, DSSP, DSSPSize,
      DSSQ, DSSQSize, DSSG, DSSGSize, DSSY, DSSYSize);
end;

function SBX509GetDHParams(Handle : THandle; DHP : Pointer; var DHPSize : integer;
  DHG : Pointer; var DHGSize : integer; DHY : Pointer; var DHYSize : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetDHParams(DllRoutines[15])(Handle, DHP, DHPSize, DHG, DHGSize,
      DHY, DHYSize);
end;

function SBX509SelfSigned(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509SelfSigned(DllRoutines[16])(Handle);
end;

function SBX509Validate(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509Validate(DllRoutines[17])(Handle);
end;

function SBX509ValidateRSA(Handle : THandle; RSAModulus : pointer; RSAModulusSize : integer;
  RSAPublicKey : pointer; RSAPublicKeySize : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509ValidateRSA(DllRoutines[18])(Handle, RSAModulus, RSAModulusSize,
      RSAPublicKey, RSAPublicKeySize);
end;

function SBX509ValidateDSS(Handle : THandle; DSSP : pointer; DSSPSize : integer;
  DSSQ : pointer; DSSQSize : integer; DSSG : pointer; DSSGSize : integer;
  DSSY : pointer; DSSYSize : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509ValidateDSS(DllRoutines[19])(Handle, DSSP, DSSPSize,
      DSSQ, DSSQSize, DSSG, DSSGSize, DSSY, DSSYSize);
end;

function SBX509ValidateWithCA(Handle : THandle; CACertificate : pointer;
  CACertificateSize : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509ValidateWithCA(DllRoutines[20])(Handle, CACertificate,
      CACertificateSize);
end;

function SBX509SetIssuer(Handle : THandle; Fields : pointer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509SetIssuer(DllRoutines[21])(Handle, Fields);
end;

function SBX509SetSubject(Handle : THandle; Fields : pointer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509SetSubject(DllRoutines[22])(Handle, Fields);
end;

function SBX509SaveToBuffer(Handle : THandle; Buffer : pointer; var Size : longint) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509SaveToBuffer(DllRoutines[23])(Handle, Buffer, Size);
end;

function SBX509SaveKeyToBuffer(Handle : THandle; Buffer : pointer; var Size : longint) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509SaveKeyToBuffer(DllRoutines[24])(Handle, Buffer, Size);
end;

function SBX509Generate(Handle : THandle; Algorithm : integer; DWordsInEncryptKey : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509Generate(DllRoutines[25])(Handle, Algorithm, DWordsInEncryptKey);
end;

function SBX509CAGenerate(Handle : THandle; Algorithm : integer; DWordsInEncryptKey : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509CAGenerate(DllRoutines[26])(Handle, Algorithm, DWordsInEncryptKey);
end;

function SBX509SetCACertificate(Handle : THandle; Buffer : pointer; Size : longint) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509SetCACertificate(DllRoutines[27])(Handle, Buffer, Size);
end;

function SBX509SetCAPrivateKey(Handle : THandle; Buffer : pointer; Size : longint) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509SetCAPrivateKey(DllRoutines[28])(Handle, Buffer, Size);
end;

function SBX509Destroy(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509Destroy(DllRoutines[29])(Handle);
end;

function SBX509SetValidFrom(Handle : THandle; Date : TDateTime) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509SetValidFrom(DllRoutines[30])(Handle, Date);
end;

function SBX509SetValidTo(Handle : THandle; Date : TDateTime) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509SetValidTo(DllRoutines[31])(Handle, Date);
end;

function SBX509LoadKeyFromBuffer(Handle : THandle; Buffer : pointer; Size : longint) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509LoadKeyFromBuffer(DllRoutines[32])(Handle, Buffer, Size);
end;

function SBX509LoadFromBufferPEM(Handle : THandle; Buffer : pointer; Size : longint; PassPhrase : PChar) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509LoadFromBufferPEM(DllRoutines[33])(Handle, Buffer, Size, PassPhrase);
end;

function SBX509LoadKeyFromBufferPEM(Handle : THandle; Buffer : pointer; Size : longint; PassPhrase : PChar) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509LoadKeyFromBufferPEM(DllRoutines[34])(Handle, Buffer, Size, PassPhrase);
end;

function SBX509SaveToBufferPEM(Handle : THandle; Buffer : pointer; var Size : longint; PassPhrase : PChar) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509SaveToBufferPEM(DllRoutines[35])(Handle, Buffer, Size, PassPhrase);
end;

function SBX509SaveKeyToBufferPEM(Handle : THandle; Buffer : pointer; var Size : longint; PassPhrase : PChar) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509SaveKeyToBufferPEM(DllRoutines[36])(Handle, Buffer, Size, PassPhrase);
end;

function SBX509Clone(Handle : THandle; DestHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509Clone(DllRoutines[37])(Handle, DestHandle);
end;

function SBX509GetHashMD5(Handle : THandle; Buffer : pointer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetHashMD5(DllRoutines[38])(Handle, Buffer);
end;

function SBX509GetHashSHA1(Handle : THandle; Buffer : pointer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetHashSHA1(DllRoutines[39])(Handle, Buffer);
end;

function SBX509GetPublicKeySize(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBX509GetPublicKeySize(DllRoutines[40])(Handle);
end;

function SBX509GetPublicKeyBlob(Handle : THandle; Buffer : pointer; var
  Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GetPublicKeyBlob(DllRoutines[41])(Handle, Buffer, Size);
end;

function SBX509PrivateKeyExists(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509PrivateKeyExists(DllRoutines[42])(Handle);
end;

function SBX509LoadFromBufferPFX(Handle : THandle; Buffer : pointer; Size : integer;
  Password : PChar) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBX509LoadFromBufferPFX(DllRoutines[43])(Handle, Buffer, Size, Password);
end;

function SBX509SaveToBufferPFX(Handle : THandle; Buffer : pointer; var Size : integer;
  Password : PChar; KeyEncryptionAlgorithm : integer; CertEncryptionAlgorithm :
  integer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBX509SaveToBufferPFX(DllRoutines[44])(Handle, Buffer, Size,
      Password, KeyEncryptionAlgorithm, CertEncryptionAlgorithm);
end;

function SBX509Sign(Handle : THandle; InBuffer : pointer; InSize : integer;
  OutBuffer : pointer; var OutSize : integer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBX509Sign(DllRoutines[45])(Handle, InBuffer, InSize, OutBuffer,
      OutSize);
end;

function SBX509Verify(Handle : THandle; InBuffer : pointer; InSize : integer;
  Signature : pointer; SignatureSize : integer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBX509Verify(DllRoutines[46])(Handle, InBuffer, InSize, Signature,
      SignatureSize);
end;

function SBX509GenerateByRequest(Handle : THandle; RequestHandle : THandle;
  NewCertHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBX509GenerateByRequest(DllRoutines[47])(Handle, RequestHandle,
      NewCertHandle);
end;

function SBX509GetIssuerRDN(Handle : THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBX509GetIssuerRDN(DllRoutines[48])(Handle);
end;

function SBX509GetSubjectRDN(Handle : THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBX509GetSubjectRDN(DllRoutines[49])(Handle);
end;

function SBX509SignDetached(Handle : THandle; InBuffer: pointer; InSize : integer;
  OutBuffer: pointer; var OutSize : integer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBX509SignDetached(DllRoutines[50])(Handle, InBuffer, InSize,
      OutBuffer, OutSize);
end;

{ Certificate storage functions }
function SBGetAvailableSystemStores(Buffer : pointer; var Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBGetAvailableSystemStores(DllRoutines[51])(Buffer, Size);
end;

function SBGetAvailablePhysicalStores(SystemStore : string; Buffer : pointer;
  var Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBGetAvailablePhysicalStores(DllRoutines[52])(PChar(SystemStore),
      Buffer, Size);
end;

function SBCertStorageFindByHashMD5(Handle : THandle; Buffer : pointer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBCertStorageFindByHashMD5(DllRoutines[53])(Handle, Buffer);
end;

function SBCertStorageFindByHashSHA1(Handle : THandle; Buffer : pointer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBCertStorageFindByHashSHA1(DllRoutines[54])(Handle, Buffer);
end;

function SBCertStorageGetIssuerCertificate(Handle : THandle; CertHandle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBCertStorageGetIssuerCertificate(DllRoutines[55])(Handle, CertHandle);
end;

function SBWinCertStorageCreate: THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBWinCertStorageCreate(DllRoutines[56]);
end;

function SBWinCertStorageDestroy(Handle : THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBWinCertStorageDestroy(DllRoutines[57])(Handle);
end;

function SBWinCertStorageValidate(Handle : THandle; CertificateHandle : THandle;
  var Reason : integer) : integer;
begin
  if DllHandle = 0 then
    Result := SB_CERT_VALIDITY_STORAGE_ERROR
  else
    Result := TSBWinCertStorageValidate(DllRoutines[58])(Handle, CertificateHandle, Reason);
end;

function SBWinCertStorageAdd(Handle : THandle; StoreName : PChar) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBWinCertStorageAdd(DllRoutines[59])(Handle, StoreName);
end;

function SBWinCertStorageRemove(Handle : THandle; StoreName : PChar) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBWinCertStorageRemove(DllRoutines[60])(Handle, StoreName);
end;

function SBWinCertStorageRemoveAll(Handle : THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBWinCertStorageRemoveAll(DllRoutines[61])(Handle);
end;

function SBWinCertStorageAddCertificate(Handle : THandle; CertificateHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBWinCertStorageAddCertificate(DllRoutines[62])(Handle, CertificateHandle);
end;

function SBWinCertStorageAddCertificateTo(Handle : THandle; CertificateHandle : THandle;
  StoreName : PChar) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBWinCertStorageAddCertificateTo(DllRoutines[63])(Handle, CertificateHandle,
      StoreName);
end;

function SBWinCertStorageRemoveCertificate(Handle : THandle; Index : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBWinCertStorageRemoveCertificate(DllRoutines[64])(Handle, Index);
end;

function SBWinCertStorageGetCertificate(Handle : THandle; Index : integer) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBWinCertStorageGetCertificate(DllRoutines[65])(Handle, Index);
end;

function SBWinCertStorageGetCount(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBWinCertStorageGetCount(DllRoutines[66])(Handle);
end;

function SBWinCertStorageExportTo(Handle : THandle; ExportToHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBWinCertStorageExportTo(DllRoutines[67])(Handle, ExportToHandle);
end;

function SBFileCertStorageCreate : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBFileCertStorageCreate(DllRoutines[68]);
end;

function SBFileCertStorageDestroy(Handle : THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBFileCertStorageDestroy(DllRoutines[69])(Handle);
end;

function SBFileCertStorageValidate(Handle : THandle; CertificateHandle : THandle;
  var Reason : integer) : integer;
begin
  if DllHandle = 0 then
    Result := SB_CERT_VALIDITY_STORAGE_ERROR
  else
    Result := TSBFileCertStorageValidate(DllRoutines[70])(Handle, CertificateHandle, Reason);
end;

function SBFileCertStorageSetFilename(Handle : THandle; Filename : PChar) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBFileCertStorageSetFilename(DllRoutines[71])(Handle, Filename);
end;

function SBFileCertStorageAdd(Handle : THandle; CertificateHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBFileCertStorageAdd(DllRoutines[72])(Handle, CertificateHandle);
end;

function SBFileCertStorageRemove(Handle : THandle; Index : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBFileCertStorageRemove(DllRoutines[73])(Handle, Index);
end;

function SBFileCertStorageGetCertificate(Handle : THandle; Index : integer) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBFileCertStorageGetCertificate(DllRoutines[74])(Handle, Index);
end;

function SBFileCertStorageGetCount(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBFileCertStorageGetCount(DllRoutines[75])(Handle);
end;

function SBFileCertStorageSaveToFile(Handle : THandle; Filename : PChar) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBFileCertStorageSaveToFile(DllRoutines[76])(Handle, Filename);
end;

function SBFileCertStorageExportTo(Handle : THandle; ExportToHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBFileCertStorageExportTo(DllRoutines[77])(Handle, ExportToHandle);
end;

function SBMemoryCertStorageCreate : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMemoryCertStorageCreate(DllRoutines[78]);
end;

function SBMemoryCertStorageDestroy(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMemoryCertStorageDestroy(DllRoutines[79])(Handle);
end;

function SBMemoryCertStorageValidate(Handle : THandle; CertificateHandle : THandle;
  var Reason : integer) : integer;
begin
  if DllHandle = 0 then
    Result := SB_CERT_VALIDITY_STORAGE_ERROR
  else
    Result := TSBMemoryCertStorageValidate(DllRoutines[80])(Handle, CertificateHandle, Reason);
end;

function SBMemoryCertStorageAdd(Handle : THandle; CertificateHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMemoryCertStorageAdd(DllRoutines[81])(Handle, CertificateHandle);
end;

function SBMemoryCertStorageRemove(Handle : THandle; Index : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMemoryCertStorageRemove(DllRoutines[82])(Handle, Index);
end;

function SBMemoryCertStorageGetCertificate(Handle : THandle; Index : integer) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMemoryCertStorageGetCertificate(DllRoutines[83])(Handle, Index);
end;

function SBMemoryCertStorageGetCount(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMemoryCertStorageGetCount(DllRoutines[84])(Handle);
end;

function SBMemoryCertStorageExportTo(Handle : THandle; ExportToHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMemoryCertStorageExportTo(DllRoutines[85])(Handle, ExportToHandle);
end;

{ Relative Distinguished Name interface }
function SBRelativeDistinguishedNameCreate : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBRelativeDistinguishedNameCreate(DllRoutines[86]);
end;

function SBRelativeDistinguishedNameDestroy(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRelativeDistinguishedNameDestroy(DllRoutines[87])(Handle);
end;

function SBRelativeDistinguishedNameSetCount(Handle : THandle; Count : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRelativeDistinguishedNameSetCount(DllRoutines[88])(Handle, Count);
end;

function SBRelativeDistinguishedNameGetCount(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBRelativeDistinguishedNameGetCount(DllRoutines[89])(Handle);
end;

function SBRelativeDistinguishedNameSetOID(Handle : THandle; Index : integer;
  Buffer : pointer; Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRelativeDistinguishedNameSetOID(DllRoutines[90])(Handle, Index,
      Buffer, Size);
end;

function SBRelativeDistinguishedNameGetOID(Handle : THandle; Index : integer;
  Buffer : pointer; var Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRelativeDistinguishedNameGetOID(DllRoutines[91])(Handle, Index,
      Buffer, Size);
end;

function SBRelativeDistinguishedNameSetValue(Handle : THandle; Index : integer;
  Buffer : pointer; Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRelativeDistinguishedNameSetValue(DllRoutines[92])(Handle,
      Index, Buffer, Size);
end;

function SBRelativeDistinguishedNameGetValue(Handle : THandle; Index : integer;
  Buffer : pointer; var Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRelativeDistinguishedNameGetValue(DllRoutines[93])(Handle,
      Index, Buffer, Size);
end;

function SBRelativeDistinguishedNameRemove(Handle : THandle; Index : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRelativeDistinguishedNameRemove(DllRoutines[94])(Handle, Index);
end;

{ Certification request interface }
function SBCertificateRequestCreate : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBCertificateRequestCreate(DllRoutines[95]);
end;

function SBCertificateRequestDestroy(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRequestDestroy(DllRoutines[96])(Handle);
end;

function SBCertificateRequestLoadFromBuffer(Handle : THandle; Buffer : pointer;
  Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRequestLoadFromBuffer(DllRoutines[97])(Handle,
      Buffer, Size);
end;

function SBCertificateRequestGenerate(Handle : THandle; PublicKeyAlgorithm :
  integer; Bits : integer; SignatureAlgorithm : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRequestGenerate(DllRoutines[98])(Handle,
      PublicKeyAlgorithm, Bits, SignatureAlgorithm);
end;

function SBCertificateRequestSaveToBuffer(Handle : THandle; Buffer : pointer;
  var Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRequestSaveToBuffer(DllRoutines[99])(Handle,
      Buffer, Size);
end;

function SBCertificateRequestValidateSignature(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRequestValidateSignature(DllRoutines[100])(Handle);
end;

function SBCertificateRequestGetRSAParams(Handle : THandle; PublicModulus :
  pointer; var PublicModulusSize : integer; PublicExponent : pointer;
  var PublicExponentSize : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRequestGetRSAParams(DllRoutines[101])(Handle,
      PublicModulus, PublicModulusSize, PublicExponent, PublicExponentSize);
end;

function SBCertificateRequestGetDSAParams(Handle : THandle; P : pointer;
  var PSize : integer; Q : pointer; var QSize : integer; G : pointer; var GSize :
  integer; Y : pointer; var YSize : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRequestGetDSAParams(DllRoutines[102])(Handle,
      P, PSize, Q, QSize, G, GSize, Y, YSize);
end;

function SBCertificateRequestGetPrivateKey(Handle : THandle; Buffer : pointer;
  var Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRequestGetPrivateKey(DllRoutines[103])(Handle,
      Buffer, Size);
end;

function SBCertificateRequestGetSubject(Handle : THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBCertificateRequestGetSubject(DllRoutines[104])(Handle);
end;

function SBCertificateRequestGetAttributes(Handle : THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBCertificateRequestGetAttributes(DllRoutines[105])(Handle);
end;

function SBCertificateRequestGetPublicKeyAlgorithm(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBCertificateRequestGetPublicKeyAlgorithm(DllRoutines[106])(Handle);
end;

function SBCertificateRequestGetSignatureAlgorithm(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBCertificateRequestGetSignatureAlgorithm(DllRoutines[107])(Handle);
end;

{ Message Encryptor interface }
function SBMessageEncryptorCreate : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageEncryptorCreate(DllRoutines[108]);
end;

function SBMessageEncryptorDestroy(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMessageEncryptorDestroy(DllRoutines[109])(Handle);
end;

function SBMessageEncryptorSetCertStorage(Handle : THandle; StorageHandle :
  THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMessageEncryptorSetCertStorage(DllRoutines[110])(Handle, StorageHandle);
end;

function SBMessageEncryptorGetCertStorage(Handle : THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageEncryptorGetCertStorage(DllRoutines[111])(Handle);
end;

function SBMessageEncryptorSetAlgorithm(Handle : THandle; Algorithm : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMessageEncryptorSetAlgorithm(DllRoutines[112])(Handle, Algorithm);
end;

function SBMessageEncryptorGetAlgorithm(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageEncryptorGetAlgorithm(DllRoutines[113])(Handle);
end;

function SBMessageEncryptorSetBitsInKey(Handle : THandle; Bits : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMessageEncryptorSetBitsInKey(DllRoutines[114])(Handle, Bits);
end;

function SBMessageEncryptorGetBitsInKey(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageEncryptorGetBitsInKey(DllRoutines[115])(Handle);
end;

function SBMessageEncryptorEncrypt(Handle : THandle; InBuffer : pointer; InSize :
  integer; OutBuffer : pointer; var OutSize : integer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageEncryptorEncrypt(DllRoutines[116])(Handle, InBuffer,
      InSize, OutBuffer, OutSize);
end;

{ Message Decryptor interface }
function SBMessageDecryptorCreate : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageDecryptorCreate(DllRoutines[117]);
end;

function SBMessageDecryptorDestroy(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMessageDecryptorDestroy(DllRoutines[118])(Handle);
end;

function SBMessageDecryptorSetCertStorage(Handle : THandle; StorageHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMessageDecryptorSetCertStorage(DllRoutines[119])(Handle, StorageHandle);
end;

function SBMessageDecryptorGetCertStorage(Handle : THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageDecryptorGetCertStorage(DllRoutines[120])(Handle);
end;

function SBMessageDecryptorGetAlgorithm(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageDecryptorGetAlgorithm(DllRoutines[121])(Handle);
end;

function SBMessageDecryptorGetBitsInKey(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageDecryptorGetBitsInKey(DllRoutines[122])(Handle);
end;

function SBMessageDecryptorDecrypt(Handle : THandle; InBuffer : pointer; InSize :
  integer; OutBuffer : pointer; var OutSize : integer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageDecryptorDecrypt(DllRoutines[123])(Handle, InBuffer, InSize,
      OutBuffer, OutSize);
end;

function SBMessageDecryptorGetUsedCertificate(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageDecryptorGetUsedCertificate(DllRoutines[124])(Handle);
end;

function SBMessageDecryptorGetCertID(Handle : THandle; Index : integer) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageDecryptorGetCertID(DllRoutines[125])(Handle, Index);
end;

function SBMessageDecryptorGetCertIDCount(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageDecryptorGetCertIDCount(DllRoutines[126])(Handle);
end;

{ Message Signer interface }
function SBMessageSignerCreate : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageSignerCreate(DllRoutines[127]);
end;

function SBMessageSignerDestroy(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMessageSignerDestroy(DllRoutines[128])(Handle);
end;

function SBMessageSignerSetHashAlgorithm(Handle : THandle; Algorithm : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMessageSignerSetHashAlgorithm(DllRoutines[129])(Handle, Algorithm);
end;

function SBMessageSignerGetHashAlgorithm(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageSignerGetHashAlgorithm(DllRoutines[130])(Handle);
end;

function SBMessageSignerSetCertStorage(Handle : THandle; StorageHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMessageSignerSetCertStorage(DllRoutines[131])(Handle, StorageHandle);
end;

function SBMessageSignerGetCertStorage(Handle : THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageSignerGetCertStorage(DllRoutines[132])(Handle);
end;

function SBMessageSignerSign(Handle : THandle; InBuffer : pointer; InSize : integer;
  OutBuffer : pointer; var OutSize : integer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageSignerSign(DllRoutines[133])(Handle, InBuffer, InSize,
      OutBuffer, OutSize);
end;

function SBMessageSignerSignDetached(Handle : THandle; InBuffer: pointer;
  InSize : integer; OutBuffer: pointer; var OutSize : integer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageSignerSignDetached(DllRoutines[134])(Handle, InBuffer,
      InSize, OutBuffer, OutSize);
end;

function SBMessageSignerGetAuthenticatedAttributes(Handle : THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageSignerGetAuthenticatedAttributes(DllRoutines[135])(Handle);
end;

function SBMessageSignerGetUnauthenticatedAttributes(Handle : THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageSignerGetUnauthenticatedAttributes(DllRoutines[136])(Handle);
end;

{ Message Verifier interface }
function SBMessageVerifierCreate : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageVerifierCreate(DllRoutines[137]);
end;

function SBMessageVerifierDestroy(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMessageVerifierDestroy(DllRoutines[138])(Handle);
end;

function SBMessageVerifierSetCertStorage(Handle : THandle; StorageHandle : THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMessageVerifierSetCertStorage(DllRoutines[139])(Handle, StorageHandle);
end;

function SBMessageVerifierGetCertStorage(Handle : Thandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageVerifierGetCertStorage(DllRoutines[140])(Handle);
end;

function SBMessageVerifierGetHashAlgorithm(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageVerifierGetHashAlgorithm(DllRoutines[141])(Handle);
end;

function SBMessageVerifierGetCertificates(Handle : THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageVerifierGetCertificates(DllRoutines[142])(Handle);
end;

function SBMessageVerifierVerify(Handle : THandle; InBuffer : pointer; InSize :
  integer; OutBuffer : pointer; var OutSize : integer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageVerifierVerify(DllRoutines[143])(Handle, InBuffer, InSize,
      OutBuffer, OutSize);
end;

function SBMessageVerifierVerifyDetached(Handle : THandle; InBuffer: pointer;
  InSize : integer; SigBuffer: pointer; SigSize : integer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBMessageVerifierVerifyDetached(DllRoutines[144])(Handle,
      InBuffer, InSize, SigBuffer, SigSize);
end;

function SBMessageVerifierIsSignatureDetached(SigBuffer: pointer; SigSize: integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBMessageVerifierIsSignatureDetached(DllRoutines[145])(SigBuffer,
      SigSize);
end;

function SBMessageVerifierGetCertID(Handle : THandle; Index: integer) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageVerifierGetCertID(DllRoutines[146])(Handle, Index);
end;

function SBMessageVerifierGetCertIDCount(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageVerifierGetCertIDCount(DllRoutines[147])(Handle);
end;

function SBMessageVerifierGetAttributes(Handle : THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBMessageVerifierGetAttributes(DllRoutines[148])(Handle);
end;

{ Random interface }
function SBRandomCreate : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBRandomCreate(DllRoutines[149]);
end;

function SBRandomDestroy(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRandomDestroy(DllRoutines[150])(Handle);
end;

function SBRandomRandomize(Handle : THandle; Buffer : pointer; Size : integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRandomRandomize(DllRoutines[151])(Handle, Buffer, Size);
end;

function SBRandomGenerate(Handle : THandle; Buffer : pointer; Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRandomGenerate(DllRoutines[152])(Handle, Buffer, Size);
end;

{ PKCS7 Issuer interface }
function SBPKCS7IssuerGetIssuer(Handle: THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBPKCS7IssuerGetIssuer(DllRoutines[153])(Handle);
end;

function SBPKCS7IssuerGetSerialNumber(Handle : THandle; Buffer : pointer; var Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBPKCS7IssuerGetSerialNumber(DllRoutines[154])(Handle, Buffer, Size);
end;

function SBPKCS7IssuerSetSerialNumber(Handle : THandle; Buffer : pointer; Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBPKCS7IssuerSetSerialNumber(DllRoutines[155])(Handle, Buffer, Size);
end;

{ ArrayList interface }
function SBArrayListGetCount(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBArrayListGetCount(DllRoutines[156])(Handle);
end;

function SBArrayListGetValue(Handle : THandle; Index : integer; Buffer : pointer;
  var Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBArrayListGetValue(DllRoutines[157])(Handle, Index, Buffer, Size);
end;

function SBArrayListSetValue(Handle : THandle; Index : integer; Buffer : pointer;
  Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBArrayListSetValue(DllRoutines[158])(Handle, Index, Buffer, Size);
end;

function SBArrayListAdd(Handle : THandle; Buffer : pointer; Size : integer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBArrayListAdd(DllRoutines[159])(Handle, Buffer, Size);
end;

function SBArrayListRemove(Handle : THandle; Index : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBArrayListRemove(DllRoutines[160])(Handle, Index);
end;

{ PKCS7 Attributes interface }
function SBPKCS7AttributesRemove(Handle : THandle; Index : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBPKCS7AttributesRemove(DllRoutines[161])(Handle, Index);
end;

function SBPKCS7AttributesGetAttribute(Handle : THandle; Index : integer;
  Buffer : pointer; var Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBPKCS7AttributesGetAttribute(DllRoutines[162])(Handle, Index,
      Buffer, Size);
end;

function SBPKCS7AttributesSetAttribute(Handle : THandle; Index : integer;
  Buffer : pointer; Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBPKCS7AttributesSetAttribute(DllRoutines[163])(Handle, Index,
      Buffer, Size);
end;

function SBPKCS7AttributesGetValues(Handle : THandle; Index : integer) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBPKCS7AttributesGetValues(DllRoutines[164])(Handle, Index);
end;

function SBPKCS7AttributesGetCount(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBPKCS7AttributesGetCount(DllRoutines[165])(Handle);
end;

function SBPKCS7AttributesSetCount(Handle : THandle; Count : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBPKCS7AttributesSetCount(DllRoutines[166])(Handle, Count);
end;

function SBFormatAttributeValue(TagID : integer; InBuffer: pointer; InSize : integer;
  OutBuffer: pointer; var OutSize : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBFormatAttributeValue(DllRoutines[167])(TagID, InBuffer, InSize,
      OutBuffer, OutSize);
end;

function SBUnformatAttributeValue(InBuffer: pointer; InSize : integer; OutBuffer:
  pointer; var OutSize : integer; var TagID : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBUnformatAttributeValue(DllRoutines[168])(InBuffer, InSize,
      OutBuffer, OutSize, TagID);
end;

{ Certificate Revocation Lists interface }
function SBCertificateRevocationListCreate : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBCertificateRevocationListCreate(DllRoutines[169])();
end;

function SBCertificateRevocationListDestroy(Handle: THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRevocationListDestroy(DllRoutines[170])(Handle);
end;

function SBCertificateRevocationListAdd(Handle : THandle; CertHandle : THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRevocationListAdd(DllRoutines[171])(Handle, CertHandle);
end;

function SBCertificateRevocationListRemoveByIndex(Handle : THandle; Index : integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRevocationListRemoveByIndex(DllRoutines[172])(Handle, Index);
end;

function SBCertificateRevocationListRemove(Handle : THandle; CertHandle : THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRevocationListRemove(DllRoutines[173])(Handle, CertHandle);
end;

function SBCertificateRevocationListIsPresent(Handle: THandle; CertHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRevocationListIsPresent(DllRoutines[174])(Handle, CertHandle);
end;

function SBCertificateRevocationListLoadFromBuffer(Handle : THandle; Buffer: pointer;
  Size : integer) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBCertificateRevocationListLoadFromBuffer(DllRoutines[175])(Handle, Buffer, Size);
end;

function SBCertificateRevocationListSaveToBuffer(Handle : THandle; Buffer: pointer;
  var Size : integer; CertHandle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBCertificateRevocationListSaveToBuffer(DllRoutines[176])(Handle,
      Buffer, Size, CertHandle);
end;

function SBCertificateRevocationListValidate(Handle : THandle; CertHandle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBCertificateRevocationListValidate(DllRoutines[177])(Handle, CertHandle);
end;

function SBCertificateRevocationListGetIssuer(Handle : THandle) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBCertificateRevocationListGetIssuer(DllRoutines[178])(Handle);
end;

function SBCertificateRevocationListGetThisUpdate(Handle : THandle; var Date : TDateTime): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRevocationListGetThisUpdate(DllRoutines[179])(Handle, Date);
end;

function SBCertificateRevocationListGetNextUpdate(Handle : THandle; var Date : TDateTime): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRevocationListGetNextUpdate(DllRoutines[180])(Handle, Date);
end;

function SBCertificateRevocationListSetThisUpdate(Handle : THandle; Date : TDateTime): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRevocationListSetThisUpdate(DllRoutines[181])(Handle, Date);
end;

function SBCertificateRevocationListSetNextUpdate(Handle : THandle; Date : TDateTime): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBCertificateRevocationListSetNextUpdate(DllRoutines[182])(Handle, Date);
end;

function SBCertificateRevocationListGetSignatureAlgorithm(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBCertificateRevocationListGetSignatureAlgorithm(DllRoutines[183])(Handle);
end;

function SBCertificateRevocationListGetItem(Handle : THandle; Index: integer) : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBCertificateRevocationListGetItem(DllRoutines[184])(Handle, Index);
end;

function SBCertificateRevocationListGetCount(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := -1
  else
    Result := TSBCertificateRevocationListGetCount(DllRoutines[185])(Handle);
end;

function SBRevocationItemGetSerialNumber(Handle : THandle; Buffer: pointer;
  var Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRevocationItemGetSerialNumber(DllRoutines[186])(Handle, Buffer, Size);
end;

function SBRevocationItemSetSerialNumber(Handle : THandle; Buffer: pointer;
  Size : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRevocationItemSetSerialNumber(DllRoutines[187])(Handle, Buffer, Size);
end;

function SBRevocationItemGetRevocationDate(Handle : THandle; var Date :
  TDateTime) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRevocationItemGetRevocationDate(DllRoutines[188])(Handle, Date);
end;

function SBRevocationItemSetRevocationDate(Handle : THandle; Date : TDateTime) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBRevocationItemSetRevocationDate(DllRoutines[189])(Handle, Date);
end;

function SBPKISetLicenseKey(Key : PChar) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
  begin
    TSBSetLicenseKey(DllRoutines[190])(Key);
    Result := true;
  end;
end;

end.
