unit SSLBBoxCliWrapper;

interface

uses
  Windows;

const
  // SSL Protocol Versions
  SB_VERSION_FIRST = 0;
  SB_VERSION_SSL2 = 0;
  SB_VERSION_SSL3 = 1;
  SB_VERSION_TLS1 = 2;

  // Cipher Suites
  SB_SUITE_FIRST                        = $0000;
  SB_SUITE_NULL_NULL_NULL               = $0000;
  SB_SUITE_RSA_NULL_MD5                 = $0001;
  SB_SUITE_RSA_NULL_SHA                 = $0002;
  SB_SUITE_RSA_RC4_MD5                  = $0003;
  SB_SUITE_RSA_RC4_SHA                  = $0004;
  SB_SUITE_RSA_RC2_MD5                  = $0005;
  SB_SUITE_RSA_IDEA_MD5                 = $0006;
  SB_SUITE_RSA_IDEA_SHA                 = $0007;
  SB_SUITE_RSA_DES_MD5                  = $0008;
  SB_SUITE_RSA_DES_SHA                  = $0009;
  SB_SUITE_RSA_3DES_MD5                 = $000A;
  SB_SUITE_RSA_3DES_SHA                 = $000B;
  SB_SUITE_RSA_AES128_SHA               = $000C;
  SB_SUITE_RSA_AES256_SHA               = $000D;
  SB_SUITE_DH_DSS_DES_SHA               = $000E;
  SB_SUITE_DH_DSS_3DES_SHA              = $000F;
  SB_SUITE_DH_DSS_AES128_SHA            = $0010;
  SB_SUITE_DH_DSS_AES256_SHA            = $0011;
  SB_SUITE_DH_RSA_DES_SHA               = $0012;
  SB_SUITE_DH_RSA_3DES_SHA              = $0013;
  SB_SUITE_DH_RSA_AES128_SHA            = $0014;
  SB_SUITE_DH_RSA_AES256_SHA            = $0015;
  SB_SUITE_DHE_DSS_DES_SHA              = $0016;
  SB_SUITE_DHE_DSS_3DES_SHA             = $0017;
  SB_SUITE_DHE_DSS_AES128_SHA           = $0018;
  SB_SUITE_DHE_DSS_AES256_SHA           = $0019;
  SB_SUITE_DHE_RSA_DES_SHA              = $001A;
  SB_SUITE_DHE_RSA_3DES_SHA             = $001B;
  SB_SUITE_DHE_RSA_AES128_SHA           = $001C;
  SB_SUITE_DHE_RSA_AES256_SHA           = $001D;
  SB_SUITE_DH_ANON_RC4_MD5              = $001E;
  SB_SUITE_DH_ANON_DES_SHA              = $001F;
  SB_SUITE_DH_ANON_3DES_SHA             = $0020;
  SB_SUITE_DH_ANON_AES128_SHA           = $0021;
  SB_SUITE_DH_ANON_AES256_SHA           = $0022;
  SB_SUITE_RSA_RC2_MD5_EXPORT           = $0023;
  SB_SUITE_RSA_RC4_MD5_EXPORT           = $0024;
  SB_SUITE_RSA_DES_SHA_EXPORT           = $0025;
  SB_SUITE_DH_DSS_DES_SHA_EXPORT        = $0026;
  SB_SUITE_DH_RSA_DES_SHA_EXPORT        = $0027;
  SB_SUITE_DHE_DSS_DES_SHA_EXPORT       = $0028;
  SB_SUITE_DHE_RSA_DES_SHA_EXPORT       = $0029;
  SB_SUITE_DH_ANON_RC4_MD5_EXPORT       = $002A;
  SB_SUITE_DH_ANON_DES_SHA_EXPORT       = $002B;

type
  TSBCloseReason = (crError, crClose);
  TClientCertificateType = (ccRSA, ccDSS, ccRSADH, ccDSSDH);

  TOnSendEventHandler = procedure (Handle: THandle; UserData: integer;
    Buffer: pointer; Size: longint); stdcall;
  TOnDataEventHandler = procedure (Handle: THandle; UserData: integer;
    Buffer: pointer; Size: longint); stdcall;
  TOnReceiveEventHandler = procedure (Handle: THandle; UserData: integer;
    Buffer: pointer; MaxSize: integer; out Written: integer); stdcall;
  TOnOpenConnectionEventHandler = procedure (Handle: THandle; UserData: integer); stdcall;
  TOnCloseConnectionEventHandler = procedure (Handle: THandle; UserData: integer;
    Reason : TSBCloseReason); stdcall;
  TOnCertificateValidateEventHandler = procedure (Handle : THandle; UserData : integer;
    x509Handle : THandle; var Validate : boolean); stdcall;
  TOnCertificateNeededEventHandler = procedure (Handle : THandle; UserData : integer;
    CertificateBuffer : pointer; var CertificateSize : longint;
    PrivateKeyBuffer : pointer; var PrivateKeySize : longint;
    CertificateType : TClientCertificateType); stdcall;
  TOnCertificateNeededExEventHandler = procedure(Handle : THandle; UserData : integer;
    var CertHandle : THandle); stdcall;

// Initializing/finalizing the library
function SBSSLCliInitialize: boolean;
procedure SBSSLCliFinalize;
function SBSSLCliSetLicenseKey(Key : PChar) : boolean;

// Secure Client interface - Methods
function SBClientClose(Handle: THandle): boolean;
function SBClientCreate: THandle;
function SBClientDataAvailable(Handle: THandle): boolean;
function SBClientDestroy(Handle: THandle): boolean;
function SBClientDisable(Handle: THandle): boolean;
function SBClientDisableCipherSuite(Handle: THandle; Suite: integer): boolean;
function SBClientDisableVersion(Handle: THandle; Version: integer): boolean;
function SBClientEnable(Handle: THandle): boolean;
function SBClientEnableCipherSuite(Handle: THandle; Suite: integer): boolean;
function SBClientEnableVersion(Handle: THandle; Version: integer): boolean;
function SBClientGetCurrentVersion(Handle: THandle): integer;
function SBClientIsCipherSuiteEnabled(Handle: THandle; Suite: integer): boolean;
function SBClientIsEnabled(Handle: THandle): boolean;
function SBClientIsActive(Handle: THandle): boolean;
function SBClientIsVersionEnabled(Handle: THandle; Version: integer): boolean;
function SBClientOpen(Handle: THandle): boolean;
function SBClientSendData(Handle: THandle; Buffer: pointer; Size: longint): boolean;
function SBClientGetCipherSuite(Handle : THandle):integer;
function SBClientInternalValidate(Handle : THandle; var Validity : integer;
  var Reason : integer):boolean;
function SBClientSetCertStorage(Handle : THandle; StorageHandle : THandle) : boolean;
function SBClientResume(Handle : THandle) : boolean;
function SBClientJoin(Handle : THandle; JoinHandle : THandle) : boolean;
function SBClientCloseSilent(Handle : THandle) : boolean;
// Secure Client interface - Callbacks
function SBClientSetOnData(Handle: THandle; Handler: TOnDataEventHandler;
  UserData: integer): boolean;
function SBClientSetOnReceive(Handle: THandle; Handler: TOnReceiveEventHandler;
  UserData: integer): boolean;
function SBClientSetOnSend(Handle: THandle; Handler: TOnSendEventHandler;
  UserData: integer): boolean;
function SBClientSetOnOpenConnection(Handle: THandle; Handler: TOnOpenConnectionEventHandler;
  UserData: integer): boolean;
function SBClientSetOnCloseConnection(Handle: THandle; Handler: TOnCloseConnectionEventHandler;
  UserData: integer): boolean;
function SBClientSetOnCertificateValidate(Handle: THandle; Handler: TOnCertificateValidateEventHandler;
  UserData: integer): boolean;
function SBClientSetOnCertificateNeeded(Handle: THandle; Handler: TOnCertificateNeededEventHandler;
  UserData: integer): boolean;
function SBClientSetOnCertificateNeededEx(Handle : THandle; Handler : TOnCertificateNeededExEventHandler;
  UserData : integer): boolean;

implementation

type
  TSBSetLicenseKey = procedure(Key : PChar); stdcall;
  TSBClientClose = function (Handle: THandle): longbool; stdcall;
  TSBClientCreate = function (): THandle; stdcall;
  TSBClientDataAvailable = function (Handle: THandle): longbool; stdcall;
  TSBClientDestroy = function (Handle: THandle): longbool; stdcall;
  TSBClientDisable = function (Handle: THandle): longbool; stdcall;
  TSBClientDisableCipherSuite = function (Handle: THandle; Suite: integer): longbool; stdcall;
  TSBClientDisableVersion = function (Handle: THandle; Version: integer): longbool; stdcall;
  TSBClientEnable = function (Handle: THandle): longbool; stdcall;
  TSBClientEnableCipherSuite = function (Handle: THandle; Suite: integer): longbool; stdcall;
  TSBClientEnableVersion = function (Handle: THandle; Version: integer): longbool; stdcall;
  TSBClientGetCurrentVersion = function (Handle: THandle): integer; stdcall;
  TSBClientIsCipherSuiteEnabled = function (Handle: THandle; Suite: integer): longbool; stdcall;
  TSBClientIsActive = function (Handle: THandle): longbool; stdcall;
  TSBClientIsEnabled = function (Handle: THandle): longbool; stdcall;
  TSBClientIsVersionEnabled = function (Handle: THandle; Version: integer): longbool; stdcall;
  TSBClientOpen = function (Handle: THandle): longbool; stdcall;
  TSBClientSendData = function (Handle: THandle; Buffer: pointer; Size: longint): longbool; stdcall;
  TSBClientInternalValidate = function(Handle : THandle; var Validity : integer;
    var Reason : integer): longbool; stdcall;
  TSBClientSetCertStorage = function(Handle : THandle; StorageHandle : THandle) : longbool; stdcall;
  TSBClientResume = function(Handle : THandle) : longbool; stdcall;
  TSBClientJoin = function(Handle : THandle; JoinHandle : THandle) : longbool; stdcall;
  TSBClientSetOnData = function (Handle: THandle; Handler: TOnDataEventHandler;
    UserData: integer): longbool; stdcall;
  TSBClientSetOnReceive = function (Handle: THandle; Handler: TOnReceiveEventHandler;
    UserData: integer): longbool; stdcall;
  TSBClientSetOnSend = function (Handle: THandle; Handler: TOnSendEventHandler;
    UserData: integer): longbool; stdcall;
  TSBClientSetOnOpenConnection = function (Handle: THandle; Handler: TOnOpenConnectionEventHandler;
    UserData: integer): longbool; stdcall;
  TSBClientSetOnCloseConnection = function (Handle: THandle; Handler: TOnCloseConnectionEventHandler;
    UserData: integer): longbool; stdcall;
  TSBClientSetOnCertificateValidate = function (Handle: THandle; Handler: TOnCertificateValidateEventHandler;
    UserData: integer): longbool; stdcall;
  TSBClientSetOnCertificateNeeded = function (Handle: THandle; Handler: TOnCertificateNeededEventHandler;
    UserData: integer): longbool; stdcall;
  TSBClientSetOnCertificateNeededEx = function (Handle: THandle; Handler: TOnCertificateNeededExEventHandler;
    UserData: integer): longbool; stdcall;
  TSBClientGetCipherSuite = function (Handle : THandle) : integer; stdcall;
  TSBClientCloseSilent = function(Handle : THandle) : longbool; stdcall;

const
  DllName = 'SSLBBoxCli.dll';
  DllRoutineCount = 32;
  DllRoutineNames : array[0..DllRoutineCount - 1] of PChar = (
    'SBClientClose',
    'SBClientCreate',
    'SBClientDataAvailable',
    'SBClientDestroy',
    'SBClientDisable',
    'SBClientDisableCipherSuite',
    'SBClientDisableVersion',
    'SBClientEnable',
    'SBClientEnableCipherSuite',
    'SBClientEnableVersion',
    'SBClientGetCurrentVersion',
    'SBClientIsActive',
    'SBClientIsCipherSuiteEnabled',
    'SBClientIsEnabled',
    'SBClientIsVersionEnabled',
    'SBClientOpen',
    'SBClientSendData',
    'SBClientSetOnData',
    'SBClientSetOnReceive',
    'SBClientSetOnSend',
    'SBClientGetCipherSuite',
    'SBClientSetOnOpenConnection',
    'SBClientSetOnCloseConnection',
    'SBClientSetOnCertificateValidate',
    'SBClientSetOnCertificateNeeded',
    'SBClientInternalValidate',
    'SBClientSetCertStorage',
    'SBClientResume',
    'SBClientJoin',
    'SBClientSetOnCertificateNeededEx',
    'SBClientCloseSilent',
    'SetLicenseKey'
  );

var
  DllHandle : THandle;
  DllRefs : integer = 0;
  DllRoutines : array[0..DllRoutineCount - 1] of pointer;

// Initializing/finalizing the library
function SBSSLCliInitialize: boolean;
var
  I : integer;
begin
  if InterlockedExchange(DllRefs, DllRefs) > 0 then
  begin
    InterlockedIncrement(DllRefs);
    Result := True
  end
  else
  begin
    Result := False;
    // loading the library
    DllHandle := LoadLibrary(DllName);
    if DllHandle = 0 then exit;
    InterlockedIncrement(DllRefs);
    // initializing links to routines
    for I := 0 to DllRoutineCount - 1 do
    begin
      DllRoutines[I] := GetProcAddress(DllHandle, DllRoutineNames[I]);
      if DllRoutines[I] = nil then
      begin
        SBSSLCliFinalize;
        exit;
      end;
    end;
    Result := True;
  end;
end;

procedure SBSSLCliFinalize;
begin
  if InterlockedExchange(DllRefs, DllRefs) > 0 then
    if InterlockedDecrement(DllRefs) = 0 then
    begin
      FreeLibrary(DllHandle);
      DllHandle := 0;
    end;
end;

function SBSSLCliSetLicenseKey(Key : PChar) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
  begin
    TSBSetLicenseKey(DllRoutines[31])(Key);
    Result := true;
  end;
end;

// Secure Client interface - Methods
function SBClientClose(Handle: THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientClose(DllRoutines[0])(Handle);
end;

function SBClientCreate: THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBClientCreate(DllRoutines[1])();
end;

function SBClientDataAvailable(Handle: THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientDataAvailable(DllRoutines[2])(Handle);
end;

function SBClientDestroy(Handle: THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientDestroy(DllRoutines[3])(Handle);
end;

function SBClientDisable(Handle: THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientDisable(DllRoutines[4])(Handle);
end;

function SBClientDisableCipherSuite(Handle: THandle; Suite: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientDisableCipherSuite(DllRoutines[5])(Handle, Suite);
end;

function SBClientDisableVersion(Handle: THandle; Version: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientDisableVersion(DllRoutines[6])(Handle, Version);
end;

function SBClientEnable(Handle: THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientEnable(DllRoutines[7])(Handle);
end;

function SBClientEnableCipherSuite(Handle: THandle; Suite: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientEnableCipherSuite(DllRoutines[8])(Handle, Suite);
end;

function SBClientEnableVersion(Handle: THandle; Version: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientEnableVersion(DllRoutines[9])(Handle, Version);
end;

function SBClientGetCurrentVersion(Handle: THandle): integer;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBClientGetCurrentVersion(DllRoutines[10])(Handle);
end;

function SBClientIsActive(Handle: THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientIsActive(DllRoutines[11])(Handle);
end;

function SBClientIsCipherSuiteEnabled(Handle: THandle; Suite: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientIsCipherSuiteEnabled(DllRoutines[12])(Handle, Suite);
end;

function SBClientIsEnabled(Handle: THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientIsEnabled(DllRoutines[13])(Handle);
end;

function SBClientIsVersionEnabled(Handle: THandle; Version: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientIsVersionEnabled(DllRoutines[14])(Handle, Version);
end;

function SBClientOpen(Handle: THandle): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientOpen(DllRoutines[15])(Handle);
end;

function SBClientSendData(Handle: THandle; Buffer: pointer; Size: longint): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientSendData(DllRoutines[16])(Handle, Buffer, Size);
end;

function SBClientSetOnData(Handle: THandle; Handler: TOnDataEventHandler;
  UserData: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientSetOnData(DllRoutines[17])(Handle, Handler, UserData);
end;

function SBClientSetOnReceive(Handle: THandle; Handler: TOnReceiveEventHandler;
  UserData: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientSetOnReceive(DllRoutines[18])(Handle, Handler, UserData);
end;

function SBClientSetOnSend(Handle: THandle; Handler: TOnSendEventHandler;
  UserData: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientSetOnSend(DllRoutines[19])(Handle, Handler, UserData);
end;

function SBClientGetCipherSuite(Handle : THandle):integer;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBClientGetCipherSuite(DllRoutines[20])(Handle);
end;

function SBClientSetOnOpenConnection(Handle: THandle; Handler: TOnOpenConnectionEventHandler;
  UserData: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientSetOnOpenConnection(DllRoutines[21])(Handle, Handler, UserData);
end;

function SBClientSetOnCloseConnection(Handle: THandle; Handler: TOnCloseConnectionEventHandler;
  UserData: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientSetOnCloseConnection(DllRoutines[22])(Handle, Handler, UserData);
end;

function SBClientSetOnCertificateValidate(Handle: THandle; Handler: TOnCertificateValidateEventHandler;
  UserData: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientSetOnCertificateValidate(DllRoutines[23])(Handle, Handler, UserData);
end;

function SBClientSetOnCertificateNeeded(Handle: THandle; Handler: TOnCertificateNeededEventHandler;
  UserData: integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientSetOnCertificateNeeded(DllRoutines[24])(Handle, Handler, UserData);
end;

function SBClientInternalValidate(Handle : THandle; var Validity : integer;
  var Reason : integer):boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientInternalValidate(DllRoutines[25])(Handle, Validity, Reason);
end;

function SBClientSetCertStorage(Handle : THandle; StorageHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientSetCertStorage(DllRoutines[26])(Handle, StorageHandle);
end;

function SBClientResume(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientResume(DllRoutines[27])(Handle);
end;

function SBClientJoin(Handle : THandle; JoinHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientJoin(DllRoutines[28])(Handle, JoinHandle);
end;

function SBClientSetOnCertificateNeededEx(Handle : THandle; Handler : TOnCertificateNeededExEventHandler;
  UserData : integer): boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientSetOnCertificateNeededEx(DllRoutines[29])(Handle, Handler, UserData);
end;

function SBClientCloseSilent(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBClientCloseSilent(DllRoutines[30])(Handle);
end;

end.
