unit SSLBBoxSrvWrapper;

interface

uses
  Windows;

const
  // SSL Protocol Versions
  SB_VERSION_FIRST = 0;
  SB_VERSION_SSL2 = 0;
  SB_VERSION_SSL3 = 1;
  SB_VERSION_TLS1 = 2;

  // Cipher Suites
  SB_SUITE_FIRST                        = $0000;
  SB_SUITE_NULL_NULL_NULL               = $0000;
  SB_SUITE_RSA_NULL_MD5                 = $0001;
  SB_SUITE_RSA_NULL_SHA                 = $0002;
  SB_SUITE_RSA_RC4_MD5                  = $0003;
  SB_SUITE_RSA_RC4_SHA                  = $0004;
  SB_SUITE_RSA_RC2_MD5                  = $0005;
  SB_SUITE_RSA_IDEA_MD5                 = $0006;
  SB_SUITE_RSA_IDEA_SHA                 = $0007;
  SB_SUITE_RSA_DES_MD5                  = $0008;
  SB_SUITE_RSA_DES_SHA                  = $0009;
  SB_SUITE_RSA_3DES_MD5                 = $000A;
  SB_SUITE_RSA_3DES_SHA                 = $000B;
  SB_SUITE_RSA_AES128_SHA               = $000C;
  SB_SUITE_RSA_AES256_SHA               = $000D;
  SB_SUITE_DH_DSS_DES_SHA               = $000E;
  SB_SUITE_DH_DSS_3DES_SHA              = $000F;
  SB_SUITE_DH_DSS_AES128_SHA            = $0010;
  SB_SUITE_DH_DSS_AES256_SHA            = $0011;
  SB_SUITE_DH_RSA_DES_SHA               = $0012;
  SB_SUITE_DH_RSA_3DES_SHA              = $0013;
  SB_SUITE_DH_RSA_AES128_SHA            = $0014;
  SB_SUITE_DH_RSA_AES256_SHA            = $0015;
  SB_SUITE_DHE_DSS_DES_SHA              = $0016;
  SB_SUITE_DHE_DSS_3DES_SHA             = $0017;
  SB_SUITE_DHE_DSS_AES128_SHA           = $0018;
  SB_SUITE_DHE_DSS_AES256_SHA           = $0019;
  SB_SUITE_DHE_RSA_DES_SHA              = $001A;
  SB_SUITE_DHE_RSA_3DES_SHA             = $001B;
  SB_SUITE_DHE_RSA_AES128_SHA           = $001C;
  SB_SUITE_DHE_RSA_AES256_SHA           = $001D;
  SB_SUITE_DH_ANON_RC4_MD5              = $001E;
  SB_SUITE_DH_ANON_DES_SHA              = $001F;
  SB_SUITE_DH_ANON_3DES_SHA             = $0020;
  SB_SUITE_DH_ANON_AES128_SHA           = $0021;
  SB_SUITE_DH_ANON_AES256_SHA           = $0022;
  SB_SUITE_RSA_RC2_MD5_EXPORT           = $0023;
  SB_SUITE_RSA_RC4_MD5_EXPORT           = $0024;
  SB_SUITE_RSA_DES_SHA_EXPORT           = $0025;
  SB_SUITE_DH_DSS_DES_SHA_EXPORT        = $0026;
  SB_SUITE_DH_RSA_DES_SHA_EXPORT        = $0027;
  SB_SUITE_DHE_DSS_DES_SHA_EXPORT       = $0028;
  SB_SUITE_DHE_RSA_DES_SHA_EXPORT       = $0029;
  SB_SUITE_DH_ANON_RC4_MD5_EXPORT       = $002A;
  SB_SUITE_DH_ANON_DES_SHA_EXPORT       = $002B;

type
  TSBCloseReason = (crError, crClose);
  TOnSendEventHandler = procedure (Handle: THandle; UserData: integer;
    Buffer: pointer; Size: longint); stdcall;
  TOnDataEventHandler = procedure (Handle: THandle; UserData: integer;
    Buffer: pointer; Size: longint); stdcall;
  TOnReceiveEventHandler = procedure (Handle: THandle; UserData: integer;
    Buffer: pointer; MaxSize: integer; out Written: integer); stdcall;
  TOnOpenConnectionEventHandler = procedure (Handle: THandle; UserData: integer); stdcall;
  TOnCloseConnectionEventHandler = procedure (Handle: THandle; UserData: integer;
    Reason : TSBCloseReason); stdcall;
  TOnCertificateValidateEventHandler = procedure (Handle : THandle; UserData : integer;
    x509Handle : THandle; var Validate : boolean); stdcall;

// Initializing/finalizing the library
function SBSSLSrvInitialize: boolean;
procedure SBSSLSrvFinalize;
function SBSSLSrvSetLicenseKey(Key : PChar) : boolean;

// Session pool interface
function SBSessionPoolCreate : THandle;
function SBSessionPoolDestroy(Handle : THandle) : boolean;
function SBSessionPoolSetTimeout(Handle : THandle; Timeout : cardinal) : boolean;
function SBSessionPoolSetInterval(Handle : THandle; Interval : cardinal) : boolean;
function SBSessionPoolSetMaxCount(Handle : THandle; MaxCount : cardinal) : boolean;

// Secure Server interface - methods
function SBServerCreate : THandle;
function SBServerDestroy(Handle : THandle) : boolean;
function SBServerOpen(Handle : THandle) : boolean;
function SBServerClose(Handle : THandle) : boolean;
function SBServerDataAvailable(Handle : THandle) : boolean;
function SBServerEnable(Handle : THandle) : boolean;
function SBServerDisable(Handle : THandle) : boolean;
function SBServerEnableCipherSuite(Handle : THandle; Suite : integer) : boolean;
function SBServerDisableCipherSuite(Handle : THandle; Suite : integer) : boolean;
function SBServerEnableVersion(Handle : THandle; Version : integer) : boolean;
function SBServerDisableVersion(Handle : THandle; Version : integer) : boolean;
function SBServerGetCurrentVersion(Handle : THandle) : integer;
function SBServerIsActive(Handle : THandle) : boolean;
function SBServerIsEnabled(Handle : THandle) : boolean;
function SBServerIsCipherSuiteEnabled(Handle : THandle; Suite : integer) : boolean;
function SBServerIsVersionEnabled(Handle : THandle; Version : integer) : boolean;
function SBServerGetCipherSuite(Handle : THandle) : integer;
function SBServerSendData(Handle : THandle; Buffer : pointer; Size : longint) : boolean;
function SBServerInternalValidate(Handle: THandle; var Validity : integer;
  var Reason : integer) : boolean;
function SBServerSetCertStorage(Handle : THandle; CertStorageHandle : THandle) : boolean;
function SBServerSetSessionPool(Handle : THandle; SessionPoolHandle : THandle) : boolean;
function SBServerSetClientCertStorage(Handle : THandle; ClientCertStorage : THandle) :
  boolean;
function SBServerEnableClientAuthentication(Handle : THandle) : boolean;
function SBServerDisableClientAuthentication(Handle : THandle) : boolean;
function SBServerIsClientAuthenticationEnabled(Handle : THandle) : boolean;
function SBServerEnableForceCertificateChain(Handle : THandle) : boolean;
function SBServerDisableForceCertificateChain(Handle : THandle) : boolean;
function SBServerIsForceCertificateChainEnabled(Handle : THandle) : boolean;
// Secure Server interface - callbacks
function SBServerSetOnData(Handle: THandle; Handler: TOnDataEventHandler;
  UserData: integer): longbool;
function SBServerSetOnReceive(Handle: THandle; Handler: TOnReceiveEventHandler;
  UserData: integer): longbool;
function SBServerSetOnSend(Handle: THandle; Handler: TOnSendEventHandler;
  UserData: integer): longbool;
function SBServerSetOnOpenConnection(Handle: THandle; Handler: TOnOpenConnectionEventHandler;
  UserData: integer): longbool;
function SBServerSetOnCloseConnection(Handle: THandle; Handler: TOnCloseConnectionEventHandler;
  UserData: integer): longbool;
function SBServerSetOnCertificateValidate(Handle: THandle; Handler: TOnCertificateValidateEventHandler;
  UserData: integer): longbool;

implementation

type
  TSBSessionPoolCreate = function : THandle; stdcall;
  TSBSessionPoolDestroy = function(Handle : THandle) : longbool; stdcall;
  TSBSessionPoolSetTimeout = function(Handle : THandle; Timeout : cardinal) : longbool; stdcall;
  TSBSessionPoolSetInterval = function(Handle : THandle; Interval : cardinal) : longbool; stdcall;
  TSBSessionPoolSetMaxCount = function(Handle : THandle; MaxCount : cardinal) : longbool; stdcall;
  TSBServerCreate = function: THandle; stdcall;
  TSBServerDestroy = function(Handle : THandle) : longbool; stdcall;
  TSBServerOpen = function(Handle : THandle) : longbool; stdcall;
  TSBServerClose = function(Handle : THandle) : longbool; stdcall;
  TSBServerDataAvailable = function(Handle : THandle) : longbool; stdcall;
  TSBServerEnable = function(Handle : THandle) : longbool; stdcall;
  TSBServerDisable = function(Handle : THandle) : longbool; stdcall;
  TSBServerEnableCipherSuite = function(Handle : THandle; Suite : integer) : longbool; stdcall;
  TSBServerDisableCipherSuite = function(Handle : THandle; Suite : integer) : longbool; stdcall;
  TSBServerEnableVersion = function(Handle : THandle; Version : integer) : longbool; stdcall;
  TSBServerDisableVersion = function(Handle : THandle; Version : integer) : longbool; stdcall;
  TSBServerGetCurrentVersion = function(Handle : THandle) : integer; stdcall;
  TSBServerIsActive = function(Handle : THandle) : longbool; stdcall;
  TSBServerIsEnabled = function(Handle : THandle) : longbool; stdcall;
  TSBServerIsCipherSuiteEnabled = function(Handle : THandle; Suite : integer) : longbool; stdcall;
  TSBServerIsVersionEnabled = function(Handle : THandle; Version : integer) : longbool; stdcall;
  TSBServerGetCipherSuite = function(Handle : THandle) : integer; stdcall;
  TSBServerSendData = function(Handle : THandle; Buffer : pointer; Size : longint) : longbool; stdcall;
  TSBServerSetOnData = function(Handle: THandle; Handler: TOnDataEventHandler;
    UserData: integer): longbool; stdcall;
  TSBServerSetOnReceive = function(Handle: THandle; Handler: TOnReceiveEventHandler;
    UserData: integer): longbool; stdcall;
  TSBServerSetOnSend = function(Handle: THandle; Handler: TOnSendEventHandler;
    UserData: integer): longbool; stdcall;
  TSBServerSetOnOpenConnection = function(Handle: THandle; Handler: TOnOpenConnectionEventHandler;
    UserData: integer): longbool; stdcall;
  TSBServerSetOnCloseConnection = function(Handle: THandle; Handler: TOnCloseConnectionEventHandler;
    UserData: integer): longbool; stdcall;
  TSBServerSetOnCertificateValidate = function(Handle: THandle; Handler: TOnCertificateValidateEventHandler;
    UserData: integer): longbool; stdcall;
  TSBServerInternalValidate = function(Handle: THandle; var Validity : integer;
    var Reason : integer) : longbool; stdcall;
  TSBServerSetCertStorage = function(Handle : THandle; CertStorageHandle : THandle) : longbool; stdcall;
  TSBServerSetSessionPool = function(Handle : THandle; SessionPoolHandle : THandle) : longbool; stdcall;
  TSBServerSetClientCertStorage = function(Handle : THandle; ClientCertStorage : THandle) :
    longbool; stdcall;
  TSBServerEnableClientAuthentication = function(Handle : THandle) : longbool; stdcall;
  TSBServerDisableClientAuthentication = function(Handle : THandle) : longbool; stdcall;
  TSBServerIsClientAuthenticationEnabled = function(Handle : THandle) : longbool; stdcall;
  TSBServerEnableForceCertificateChain = function(Handle : THandle) : longbool; stdcall;
  TSBServerDisableForceCertificateChain = function(Handle : THandle) : longbool; stdcall;
  TSBServerIsForceCertificateChainEnabled = function(Handle : THandle) : longbool; stdcall;
  TSBSetLicenseKey = procedure(Key : PChar); stdcall;

const
  DllName = 'SSLBBoxSrv.dll';
  DllRoutineCount = 40;
  DllRoutineNames : array[0..DllRoutineCount - 1] of PChar = (
    'SBSessionPoolCreate',
    'SBSessionPoolDestroy',
    'SBSessionPoolSetTimeout',
    'SBSessionPoolSetInterval',
    'SBSessionPoolSetMaxCount',
    'SBServerCreate',
    'SBServerDestroy',
    'SBServerOpen',
    'SBServerClose',
    'SBServerDataAvailable',
    'SBServerEnable',
    'SBServerDisable',
    'SBServerEnableCipherSuite',
    'SBServerDisableCipherSuite',
    'SBServerEnableVersion',
    'SBServerDisableVersion',
    'SBServerGetCurrentVersion',
    'SBServerIsActive',
    'SBServerIsEnabled',
    'SBServerIsCipherSuiteEnabled',
    'SBServerIsVersionEnabled',
    'SBServerGetCipherSuite',
    'SBServerSendData',
    'SBServerInternalValidate',
    'SBServerSetCertStorage',
    'SBServerSetSessionPool',
    'SBServerSetClientCertStorage',
    'SBServerEnableClientAuthentication',
    'SBServerDisableClientAuthentication',
    'SBServerIsClientAuthenticationEnabled',
    'SBServerEnableForceCertificateChain',
    'SBServerDisableForceCertificateChain',
    'SBServerIsForceCertificateChainEnabled',
    'SBServerSetOnData',
    'SBServerSetOnReceive',
    'SBServerSetOnSend',
    'SBServerSetOnOpenConnection',
    'SBServerSetOnCloseConnection',
    'SBServerSetOnCertificateValidate',
    'SetLicenseKey'
  );

var
  DllHandle: THandle = 0;
  DllRefs: integer = 0;
  DllRoutines: array [0..DllRoutineCount - 1] of pointer;

// Initializing/finalizing the library
function SBSSLSrvInitialize: boolean;
var
  I : integer;
begin
  if InterlockedExchange(DllRefs, DllRefs) > 0 then
  begin
    InterlockedIncrement(DllRefs);
    Result := True
  end
  else
  begin
    Result := False;
    // loading the library
    DllHandle := LoadLibrary(DllName);
    if DllHandle = 0 then exit;
    InterlockedIncrement(DllRefs);
    // initializing links to routines
    for I := 0 to DllRoutineCount - 1 do
    begin
      DllRoutines[I] := GetProcAddress(DllHandle, DllRoutineNames[I]);
      if DllRoutines[I] = nil then
      begin
        SBSSLSrvFinalize;
        exit;
      end;
    end;
    Result := True;
  end;
end;

procedure SBSSLSrvFinalize;
begin
  if InterlockedExchange(DllRefs, DllRefs) > 0 then
    if InterlockedDecrement(DllRefs) = 0 then
    begin
      FreeLibrary(DllHandle);
      DllHandle := 0;
    end;
end;

function SBSSLSrvSetLicenseKey(Key : PChar) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
  begin
    TSBSetLicenseKey(DllRoutines[39])(Key);
    Result := true;
  end;
end;

// Session pool interface
function SBSessionPoolCreate : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBSessionPoolCreate(DllRoutines[0]);
end;

function SBSessionPoolDestroy(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBSessionPoolDestroy(DllRoutines[1])(Handle);
end;

function SBSessionPoolSetTimeout(Handle : THandle; Timeout : cardinal) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBSessionPoolSetTimeout(DllRoutines[2])(Handle, Timeout);
end;

function SBSessionPoolSetInterval(Handle : THandle; Interval : cardinal) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBSessionPoolSetInterval(DllRoutines[3])(Handle, Interval);
end;

function SBSessionPoolSetMaxCount(Handle : THandle; MaxCount : cardinal) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBSessionPoolSetMaxCount(DllRoutines[4])(Handle, MaxCount);
end;

// Secure Server interface - methods
function SBServerCreate : THandle;
begin
  if DllHandle = 0 then
    Result := 0
  else
    Result := TSBServerCreate(DllRoutines[5]);
end;

function SBServerDestroy(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerDestroy(DllRoutines[6])(Handle);
end;

function SBServerOpen(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerOpen(DllRoutines[7])(Handle);
end;

function SBServerClose(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerClose(DllRoutines[8])(Handle);
end;

function SBServerDataAvailable(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerDataAvailable(DllRoutines[9])(Handle);
end;

function SBServerEnable(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerEnable(DllRoutines[10])(Handle);
end;

function SBServerDisable(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerDestroy(DllRoutines[11])(Handle);
end;

function SBServerEnableCipherSuite(Handle : THandle; Suite : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerEnableCipherSuite(DllRoutines[12])(Handle, Suite);
end;

function SBServerDisableCipherSuite(Handle : THandle; Suite : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerDisableCipherSuite(DllRoutines[13])(Handle, Suite);
end;

function SBServerEnableVersion(Handle : THandle; Version : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerEnableVersion(DllRoutines[14])(Handle, Version);
end;

function SBServerDisableVersion(Handle : THandle; Version : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerDisableVersion(DllRoutines[15])(Handle, Version);
end;

function SBServerGetCurrentVersion(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := SB_VERSION_FIRST - 1
  else
    Result := TSBServerGetCurrentVersion(DllRoutines[16])(Handle);
end;

function SBServerIsActive(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerIsActive(DllRoutines[17])(Handle);
end;

function SBServerIsEnabled(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerIsEnabled(DllRoutines[18])(Handle);
end;

function SBServerIsCipherSuiteEnabled(Handle : THandle; Suite : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerIsCipherSuiteEnabled(DllRoutines[19])(Handle, Suite);
end;

function SBServerIsVersionEnabled(Handle : THandle; Version : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerIsVersionEnabled(DllRoutines[20])(Handle, Version);
end;

function SBServerGetCipherSuite(Handle : THandle) : integer;
begin
  if DllHandle = 0 then
    Result := SB_SUITE_FIRST - 1
  else
    Result := TSBServerGetCipherSuite(DllRoutines[21])(Handle);
end;

function SBServerSendData(Handle : THandle; Buffer : pointer; Size : longint) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerSendData(DllRoutines[22])(Handle, Buffer, Size);
end;

function SBServerInternalValidate(Handle: THandle; var Validity : integer;
  var Reason : integer) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerInternalValidate(DllRoutines[23])(Handle, Validity, Reason);
end;

function SBServerSetCertStorage(Handle : THandle; CertStorageHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerSetCertStorage(DllRoutines[24])(Handle, CertStorageHandle);
end;

function SBServerSetSessionPool(Handle : THandle; SessionPoolHandle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerSetSessionPool(DllRoutines[25])(Handle, SessionPoolHandle);
end;

function SBServerSetClientCertStorage(Handle : THandle; ClientCertStorage : THandle) :
  boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerSetClientCertStorage(DllRoutines[26])(Handle, ClientCertStorage);
end;

function SBServerEnableClientAuthentication(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerEnableClientAuthentication(DllRoutines[27])(Handle);
end;

function SBServerDisableClientAuthentication(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerDisableClientAuthentication(DllRoutines[28])(Handle);
end;

function SBServerIsClientAuthenticationEnabled(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerIsClientAuthenticationEnabled(DllRoutines[29])(Handle);
end;

function SBServerEnableForceCertificateChain(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerEnableForceCertificateChain(DllRoutines[30])(Handle);
end;

function SBServerDisableForceCertificateChain(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerDisableForceCertificateChain(DllRoutines[31])(Handle);
end;

function SBServerIsForceCertificateChainEnabled(Handle : THandle) : boolean;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerIsForceCertificateChainEnabled(DllRoutines[32])(Handle);
end;

// Secure Server interface - callbacks
function SBServerSetOnData(Handle: THandle; Handler: TOnDataEventHandler;
  UserData: integer): longbool;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerSetOnData(DllRoutines[33])(Handle, Handler, UserData);
end;

function SBServerSetOnReceive(Handle: THandle; Handler: TOnReceiveEventHandler;
  UserData: integer): longbool;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerSetOnReceive(DllRoutines[34])(Handle, Handler, UserData);
end;

function SBServerSetOnSend(Handle: THandle; Handler: TOnSendEventHandler;
  UserData: integer): longbool;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerSetOnSend(DllRoutines[35])(Handle, Handler, UserData);
end;

function SBServerSetOnOpenConnection(Handle: THandle; Handler: TOnOpenConnectionEventHandler;
  UserData: integer): longbool;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerSetOnOpenConnection(DllRoutines[36])(Handle, Handler, UserData);
end;

function SBServerSetOnCloseConnection(Handle: THandle; Handler: TOnCloseConnectionEventHandler;
  UserData: integer): longbool;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerSetOnCloseConnection(DllRoutines[37])(Handle, Handler, UserData);
end;

function SBServerSetOnCertificateValidate(Handle: THandle; Handler: TOnCertificateValidateEventHandler;
  UserData: integer): longbool;
begin
  if DllHandle = 0 then
    Result := false
  else
    Result := TSBServerSetOnCertificateValidate(DllRoutines[38])(Handle, Handler, UserData);
end;

end.
