#ifndef __PKIBBOX_H__
#define __PKIBBOX_H__

#include <windows.h>

// Certificate Algorithms

#define		SB_CERT_ALGORITHM_ID_RSA_ENCRYPTION		0x0000
#define		SB_CERT_ALGORITHM_MD2_RSA_ENCRYPTION	0x0001
#define		SB_CERT_ALGORITHM_MD5_RSA_ENCRYPTION	0x0002
#define		SB_CERT_ALGORITHM_SHA1_RSA_ENCRYPTION	0x0003
#define		SB_CERT_ALGORITHM_ID_DSA				0x0004
#define		SB_CERT_ALGORITHM_ID_DSA_SHA1			0x0005
#define		SB_CERT_ALGORITHM_DH_PUBLIC				0x0006


// Certificate Validity constants

#define		SB_CERT_VALIDITY_OK						0x0001
#define		SB_CERT_VALIDITY_SELF_SIGNED			0x0002
#define		SB_CERT_VALIDITY_INVALID				0x0004
#define		SB_CERT_VALIDITY_STORAGE_ERROR			0x0008

#define		SB_CERT_VALIDITY_REASON_BAD_DATA			0x0001
#define		SB_CERT_VALIDITY_REASON_REVOKED				0x0002
#define		SB_CERT_VALIDITY_REASON_NOT_YET_VALID		0x0004
#define		SB_CERT_VALIDITY_REASON_EXPIRED				0x0008
#define		SB_CERT_VALIDITY_REASON_INVALID_SIGNATURE	0x0010
#define		SB_CERT_VALIDITY_REASON_UNKNOWN_CA			0x0020

// Algorithm constants

#define		SB_ALGORITHM_CNT_RC4						0x7001
#define		SB_ALGORITHM_CNT_DES						0x7002
#define		SB_ALGORITHM_CNT_3DES						0x7003
#define		SB_ALGORITHM_CNT_RC2						0x7004
#define		SB_ALGORITHM_CNT_AES128						0x7005
#define		SB_ALGORITHM_CNT_AES192						0x7006
#define		SB_ALGORITHM_CNT_AES256						0x7007
#define		SB_ALGORITHM_DGST_SHA1						0x7101
#define		SB_ALGORITHM_DGST_MD5						0x7102
#define		SB_ALGORITHM_PBE_SHA1_RC4_128				0x7201
#define		SB_ALGORITHM_PBE_SHA1_RC4_40				0x7202
#define		SB_ALGORITHM_PBE_SHA1_3DES					0x7203
#define		SB_ALGORITHM_PBE_SHA1_RC2_128				0x7204
#define		SB_ALGORITHM_PBE_SHA1_RC2_40				0x7205
#define		SB_ALGORITHM_UNKNOWN						0x7FFF

// Object identifiers which may be used in RelativeDistinguishedName

#define		SB_CERT_OID_COMMON_NAME						"\x55\x04\x03"
#define		SB_CERT_OID_SURNAME							"\x55\x04\x04"
#define		SB_CERT_OID_COUNTRY							"\x55\x04\x06"
#define		SB_CERT_OID_LOCALITY						"\x55\x04\x07"
#define		SB_CERT_OID_STATE_OR_PROVINCE				"\x55\x04\x08"
#define		SB_CERT_OID_ORGANIZATION					"\x55\x04\x0A"
#define		SB_CERT_OID_ORGANIZATION_UNIT				"\x55\x04\x0B"
#define		SB_CERT_OID_TITLE							"\x55\x04\x0C"
#define		SB_CERT_OID_NAME							"\x55\x04\x29"
#define		SB_CERT_OID_GIVEN_NAME						"\x55\x04\x2A"
#define		SB_CERT_OID_INITIALS						"\x55\x04\x2B"
#define		SB_CERT_OID_GENERATION_QUALIFIER			"\x55\x04\x2C"
#define		SB_CERT_OID_DN_QUALIFIER					"\x55\x04\x2E"

#ifdef __cplusplus 
extern "C" {
#endif

// Common routines

BOOL	SBPKIInitialize(void);
void	SBPKIFinalize(void);
BOOL	SBPKISetLicenseKey(PCHAR Key);

// X.509 Certificates interface

int		SBX509GetVersion(HANDLE hCert);
BOOL	SBX509GetSerialNumber(HANDLE hCert, void* pSerialNumber, int* Size);
HANDLE	SBX509Create(void);
BOOL	SBX509LoadFromBuffer(HANDLE hCert, void* pData, int Size);
BOOL	SBX509GetCertificateBinary(HANDLE hCert, void* pBinaryData, int* Size);
BOOL	SBX509GetSignatureValue(HANDLE hCert, void* pSignature, int* Size);
BOOL	SBX509GetIssuer(HANDLE hCert, void* pIssuer, int* Size);
BOOL	SBX509GetValidityFrom(HANDLE hCert, DATE *Date);
BOOL	SBX509GetValidityTo(HANDLE hCert, DATE *Date);
BOOL	SBX509GetSubject(HANDLE hCert, void* pSubject, int* Size);
int		SBX509GetSignatureAlgorithm(HANDLE hCert);
int		SBX509GetPublicKeyAlgorithm(HANDLE hCert);
BOOL	SBX509GetRSAParams(HANDLE hCert, void* pRSAModulus, int* nRSAModulusSize,
			void* pRSAPublicExponent, int* nRSAPublicExponentSize);
BOOL	SBX509GetDSSSignatureParams(HANDLE hCert, void* pDSSS, int* nDSSSSize,
			void* pDSSR, int* nDSSRSize);
BOOL	SBX509GetDSSParams(HANDLE hCert, void* pDSSP, int* nDSSPSize, void* pDSSQ, int* nDSSQSize, 
			void* pDSSG, int* nDSSGSize, void* pDSSY, int* nDSSYSize);
BOOL	SBX509GetDHParams(HANDLE hCert, void* pDHP, int* nDHPSize,
			void* pDHG, int* nDHGSize, void* pDHY, int* nDHYSize);
BOOL	SBX509IsCertificateSelfSigned(HANDLE hCert);
BOOL	SBX509Validate(HANDLE hCert);
BOOL	SBX509ValidateRSA(HANDLE hCert, void* pRSAModulus, int nRSAModulusSize,
			void* pRSAPublicExponent, int nRSAPublicExponentSize);
BOOL	SBX509ValidateDSS(HANDLE hCert, void* pDSSP, int nDSSPSize,
			void* pDSSQ, int nDSSQSize, void* pDSSG, int nDSSGSize,
			void* pDSSY, int nDSSYSize);
BOOL	SBX509ValidateWithCA(HANDLE hCert, void* pCACertificate, int nCACertificateSize);
BOOL	SBX509SetIssuer(HANDLE hCert, PCHAR Fields[]);
BOOL	SBX509SetSubject(HANDLE hCert, PCHAR Fields[]);
BOOL	SBX509SaveToBuffer(HANDLE hCert, void* Buffer, int* nSize);
BOOL	SBX509SaveKeyToBuffer(HANDLE hCert, void* Buffer, int* nSize);
BOOL	SBX509Generate(HANDLE hCert, int nAlgorithm, int nDwordsInEncryptKey);
BOOL	SBX509CAGenerate(HANDLE hCert, int nAlgorithm, int nDwordsInEncryptKey);
BOOL	SBX509SetCACertificate(HANDLE hCert, void* Buffer, int nSize);
BOOL	SBX509SetCAPrivateKey(HANDLE hCert, void* Buffer, int nSize);
BOOL	SBX509Destroy(HANDLE hCert);
BOOL	SBX509SetValidityFrom(HANDLE hCert, DATE Date);
BOOL	SBX509SetValidityTo(HANDLE hCert, DATE Date);
BOOL	SBX509LoadKeyFromBuffer(HANDLE hCert, void* pBuffer, int nSize);
BOOL	SBX509LoadFromBufferPEM(HANDLE hCert, void* Buffer, int nSize, PCHAR PassPhrase);
BOOL	SBX509LoadKeyFromBufferPEM(HANDLE hCert, void* Buffer, int nSize, PCHAR PassPhrase);
BOOL	SBX509SaveToBufferPEM(HANDLE hCert, void* Buffer, int* nSize, PCHAR PassPhrase);
BOOL	SBX509SaveKeyToBufferPEM(HANDLE hCert, void* Buffer, int* nSize, PCHAR PassPhrase);
BOOL	SBX509Clone(HANDLE hCert, HANDLE hDestCert);
BOOL	SBX509GetHashMD5(HANDLE hCert, void* pBuffer);
BOOL	SBX509GetHashSHA1(HANDLE hCert, void* pBuffer);
int		SBX509GetPublicKeySize(HANDLE hCert);
BOOL	SBX509GetPublicKeyBlob(HANDLE hCert, void* pBuffer, int* nSize);
BOOL	SBX509PrivateKeyExists(HANDLE hCert);
int		SBX509LoadFromBufferPFX(HANDLE hCert, void* pBuffer, int nSize, PCHAR Password);
int		SBX509SaveToBufferPFX(HANDLE hCert, void* pBuffer, int* nSize, PCHAR Password, int KeyEncryptionAlgorithm, int CertEncryptionAlgorithm);
int		SBX509Sign(HANDLE hCert, void* pInBuffer, int nInSize, void* pOutBuffer, int* nOutSize);
int		SBX509Verify(HANDLE hCert, void* pInBuffer, int nInSize, void* pSignature, int nSignatureSize);
BOOL	SBX509GenerateByRequest(HANDLE hCert, HANDLE hRequest, HANDLE hNewCert);
HANDLE	SBX509GetIssuerRDN(HANDLE hCert);
HANDLE	SBX509GetSubjectRDN(HANDLE hCert);
int		SBX509SignDetached(HANDLE hCert, void* pInBuffer, int nInSize, void* pOutBuffer, int* nOutSize);

// Certificate storages interface

BOOL	SBGetAvailableSystemStores(char** pBuffer, int *nSize);
BOOL	SBGetAvailablePhysicalStores(char* szSystemStore, char** pBuffer, int *nSize);
int		SBCertStorageFindByHashMD5(HANDLE hStorage, void* pHash);
int		SBCertStorageFindByHashSHA1(HANDLE hStorage, void* pHash);
int		SBCertStorageGetIssuerCertificate(HANDLE hStorage, HANDLE hCert);
HANDLE	SBWinCertStorageCreate(void);
BOOL	SBWinCertStorageDestroy(HANDLE hStorage);
int		SBWinCertStorageValidate(HANDLE hStorage, HANDLE hCert,
  int* nReason);
BOOL	SBWinCertStorageAdd(HANDLE hStorage, PCHAR StoreName);
BOOL	SBWinCertStorageRemove(HANDLE hStorage, PCHAR StoreName);
BOOL	SBWinCertStorageRemoveAll(HANDLE hStorage);
BOOL	SBWinCertStorageAddCertificate(HANDLE hStorage, HANDLE hCert);
BOOL	SBWinCertStorageAddCertificateTo(HANDLE hStorage, HANDLE hCert, PCHAR StoreName);
BOOL	SBWinCertStorageRemoveCertificate(HANDLE hStorage, int nIndex);
HANDLE	SBWinCertStorageGetCertificate(HANDLE hStorage, int nIndex);
int		SBWinCertStorageGetCount(HANDLE hStorage);
BOOL	SBWinCertStorageExportTo(HANDLE hStorage, HANDLE hExportToStorage);
HANDLE	SBFileCertStorageCreate(void);
BOOL	SBFileCertStorageDestroy(HANDLE hStorage);
int		SBFileCertStorageValidate(HANDLE hStorage, HANDLE hCert, 
  int* Reason);
BOOL	SBFileCertStorageSetFilename(HANDLE hStorage, PCHAR Filename); 
BOOL	SBFileCertStorageAdd(HANDLE hStorage, HANDLE hCert);
BOOL	SBFileCertStorageRemove(HANDLE hStorage, int nIndex);
HANDLE	SBFileCertStorageGetCertificate(HANDLE hStorage, int nIndex);
int		SBFileCertStorageGetCount(HANDLE hStorage);
BOOL	SBFileCertStorageSaveToFile(HANDLE hStorage, PCHAR Filename);
BOOL	SBFileCertStorageExportTo(HANDLE hStorage, HANDLE hExportToStorage);
HANDLE	SBMemoryCertStorageCreate(void);
BOOL	SBMemoryCertStorageDestroy(HANDLE hStorage);
int		SBMemoryCertStorageValidate(HANDLE hStorage, HANDLE hCert, int* Reason);
BOOL	SBMemoryCertStorageAdd(HANDLE hStorage, HANDLE hCert);
BOOL	SBMemoryCertStorageRemove(HANDLE hStorage, int nIndex);
HANDLE	SBMemoryCertStorageGetCertificate(HANDLE hStorage, int nIndex);
int		SBMemoryCertStorageGetCount(HANDLE hStorage);
BOOL	SBMemoryCertStorageExportTo(HANDLE hStorage, HANDLE hExportToStorage);

// Relative distinguished name interface

HANDLE	SBRelativeDistinguishedNameCreate(void);
BOOL	SBRelativeDistinguishedNameDestroy(HANDLE hName);
BOOL	SBRelativeDistinguishedNameSetCount(HANDLE hName, int Count);
int		SBRelativeDistinguishedNameGetCount(HANDLE hName);
BOOL	SBRelativeDistinguishedNameSetOID(HANDLE hName, int Index, void* pBuffer, int nSize);
BOOL	SBRelativeDistinguishedNameGetOID(HANDLE hName, int Index, void* pBuffer, int* nSize);
BOOL	SBRelativeDistinguishedNameSetValue(HANDLE hName, int Index, void* pBuffer, int nSize);
BOOL	SBRelativeDistinguishedNameGetValue(HANDLE hName, int Index, void* pBuffer, int* nSize);
BOOL	SBRelativeDistinguishedNameRemove(HANDLE hName, int Index);

// Certificate Request interface

HANDLE	SBCertificateRequestCreate(void);
BOOL	SBCertificateRequestDestroy(HANDLE hRequest);
BOOL	SBCertificateRequestLoadFromBuffer(HANDLE hRequest, void* pBuffer, int nSize);
BOOL	SBCertificateRequestGenerate(HANDLE hRequest, int PublicKeyAlgorithm, int Bits, int SignatureAlgorithm);
BOOL	SBCertificateRequestSaveToBuffer(HANDLE hRequest, void* pBuffer, int* nSize);
BOOL	SBCertificateRequestValidateSignature(HANDLE hRequest);
BOOL	SBCertificateRequestGetRSAParams(HANDLE hRequest, void* pPublicModulus, int* nPublicModulusSize, void* pPublicExponent, int* nPublicExponentSize);
BOOL	SBCertificateRequestGetDSAParams(HANDLE hRequest, void* pP, int* nPSize, void* pQ, int* nQSize, void* pG, int* nGSize, void* pY, int* nYSize);
BOOL	SBCertificateRequestGetPrivateKey(HANDLE hRequest, void* pBuffer, int* nSize);
HANDLE	SBCertificateRequestGetSubject(HANDLE hRequest);
HANDLE	SBCertificateRequestGetAttributes(HANDLE hRequest);
int		SBCertificateRequestGetPublicKeyAlgorithm(HANDLE hRequest);
int		SBCertificateRequestGetSignatureAlgorithm(HANDLE hRequest);

// Message Encryptor interface

HANDLE	SBMessageEncryptorCreate(void);
BOOL	SBMessageEncryptorDestroy(HANDLE hEncryptor);
BOOL	SBMessageEncryptorSetCertStorage(HANDLE hEncryptor, HANDLE hStorage);
HANDLE	SBMessageEncryptorGetCertStorage(HANDLE hEncryptor);
BOOL	SBMessageEncryptorSetAlgorithm(HANDLE hEncryptor, int Algorithm);
int		SBMessageEncryptorGetAlgorithm(HANDLE hEncryptor);
BOOL	SBMessageEncryptorSetBitsInKey(HANDLE hEncryptor, int Bits);
int		SBMessageEncryptorGetBitsInKey(HANDLE hEncryptor);
int		SBMessageEncryptorEncrypt(HANDLE hEncryptor, void* pInBuffer, int nInSize, void* pOutBuffer, int* nOutSize);

// Message Decryptor interface

HANDLE	SBMessageDecryptorCreate(void);
BOOL	SBMessageDecryptorDestroy(HANDLE hDecryptor);
BOOL	SBMessageDecryptorSetCertStorage(HANDLE hDecryptor, HANDLE hStorage);
HANDLE	SBMessageDecryptorGetCertStorage(HANDLE hDecryptor);
int		SBMessageDecryptorGetAlgorithm(HANDLE hDecryptor);
int		SBMessageDecryptorGetBitsInKey(HANDLE hDecryptor);
int		SBMessageDecryptorDecrypt(HANDLE hDecryptor, void* pInBuffer, int nInSize, void* pOutBuffer, int* nOutSize);
int		SBMessageDecryptorGetUsedCertificate(HANDLE hDecryptor);
HANDLE	SBMessageDecryptorGetCertID(HANDLE hDecryptor, int nIndex);
int		SBMessageDecryptorGetCertIDCount(HANDLE hDecryptor);

// Message Signer interface

HANDLE	SBMessageSignerCreate(void);
BOOL	SBMessageSignerDestroy(HANDLE hSigner);
BOOL	SBMessageSignerSetHashAlgorithm(HANDLE hSigner, int Algorithm);
int		SBMessageSignerGetHashAlgorithm(HANDLE hSigner);
BOOL	SBMessageSignerSetCertStorage(HANDLE hSigner, HANDLE hStorage);
HANDLE	SBMessageSignerGetCertStorage(HANDLE hSigner);
int		SBMessageSignerSign(HANDLE hSigner, void* pInBuffer, int nInSize, void* pOutBuffer, int* nOutSize);
int		SBMessageSignerSignDetached(HANDLE hSigner, void* pInBuffer, int nInSize, void* pOutBuffer, int* nOutSize);
HANDLE	SBMessageSignerGetAuthenticatedAttributes(HANDLE hSigner);
HANDLE	SBMessageSignerGetUnauthenticatedAttributes(HANDLE hSigner);

// Message Verifier interface

HANDLE	SBMessageVerifierCreate(void);
BOOL	SBMessageVerifierDestroy(HANDLE hVerifier);
BOOL	SBMessageVerifierSetCertStorage(HANDLE hVerifier, HANDLE hStorage);
HANDLE	SBMessageVerifierGetCertStorage(HANDLE hVerifier);
int		SBMessageVerifierGetHashAlgorithm(HANDLE hVerifier);
HANDLE	SBMessageVerifierGetCertificates(HANDLE hVerifier);
int		SBMessageVerifierVerify(HANDLE hVerifier, void* pInBuffer, int nInSize, void* pOutBuffer, int* nOutSize);
int		SBMessageVerifierVerifyDetached(HANDLE hVerifier, void* pInBuffer, int nInSize, void* pSigBuffer, int nSigSize);
BOOL	SBMessageVerifierIsSignatureDetached(void* pSigBuffer, int nSigSize);
HANDLE	SBMessageVerifierGetCertID(HANDLE hVerifier, int nIndex);
int		SBMessageVerifierGetCertIDCount(HANDLE hVerifier);
HANDLE	SBMessageVerifierGetAttributes(HANDLE hVerifier);

// Random interface

HANDLE	SBRandomCreate(void);
BOOL	SBRandomDestroy(HANDLE hRandom);
BOOL	SBRandomRandomize(HANDLE hRandom, void* pBuffer, int nSize);
BOOL	SBRandomGenerate(HANDLE hRandom, void* pBuffer, int nSize);

// PKCS#7 issuer interface

HANDLE	SBPKCS7IssuerGetIssuer(HANDLE hIssuer);
BOOL	SBPKCS7IssuerGetSerialNumber(HANDLE hIssuer, void* pBuffer, int* nSize);
BOOL	SBPKCS7IssuerSetSerialNumber(HANDLE hIssuer, void* pBuffer, int nSize);

// Array List interface

int		SBArrayListGetCount(HANDLE hList) ;
BOOL	SBArrayListGetValue(HANDLE hList, int nIndex, void* pBuffer, int* nSize);
BOOL	SBArrayListSetValue(HANDLE hList, int nIndex, void* pBuffer, int nSize);
int		SBArrayListAdd(HANDLE hList, void* pBuffer, int nSize);
BOOL	SBArrayListRemove(HANDLE hList, int nIndex);

// PKCS#7 attributes interface

BOOL	SBPKCS7AttributesRemove(HANDLE hAttrs, int nIndex);
BOOL	SBPKCS7AttributesGetAttribute(HANDLE hAttrs, int nIndex, void* pBuffer, int* nSize);
BOOL	SBPKCS7AttributesSetAttribute(HANDLE hAttrs, int nIndex, void* pBuffer, int nSize);
HANDLE	SBPKCS7AttributesGetValues(HANDLE hAttrs, int nIndex);
int		SBPKCS7AttributesGetCount(HANDLE hAttrs);
BOOL	SBPKCS7AttributesSetCount(HANDLE hAttrs, int nCount);
BOOL	SBFormatAttributeValue(int nTagID, void* pInBuffer, int nInSize, void* pOutBuffer,
							   int* nOutSize);
BOOL	SBUnformatAttributeValue(void* pInBuffer, int nInSize, void* pOutBuffer,
							   int* nOutSize, int* nTagID);

// Certificate revocation lists interface

HANDLE	SBCertificateRevocationListCreate(void);
BOOL	SBCertificateRevocationListDestroy(HANDLE hCRL);
BOOL	SBCertificateRevocationListAdd(HANDLE hCRL, HANDLE hCert);
BOOL	SBCertificateRevocationListRemoveByIndex(HANDLE hCRL, int nIndex);
BOOL	SBCertificateRevocationListRemove(HANDLE hCRL, HANDLE hCert);
BOOL	SBCertificateRevocationListIsPresent(HANDLE hCRL, HANDLE hCert);
int		SBCertificateRevocationListLoadFromBuffer(HANDLE hCRL, void* pBuffer, int nSize);
int		SBCertificateRevocationListSaveToBuffer(HANDLE hCRL, void* pBuffer, int* nSize, HANDLE hCert);
int		SBCertificateRevocationListValidate(HANDLE hCRL, HANDLE hCert);
HANDLE	SBCertificateRevocationListGetIssuer(HANDLE hCRL);
BOOL	SBCertificateRevocationListGetThisUpdate(HANDLE hCRL, DATE* date);
BOOL	SBCertificateRevocationListGetNextUpdate(HANDLE hCRL, DATE* date);
BOOL	SBCertificateRevocationListSetThisUpdate(HANDLE hCRL, DATE date);
BOOL	SBCertificateRevocationListSetNextUpdate(HANDLE hCRL, DATE date);
int		SBCertificateRevocationListGetSignatureAlgorithm(HANDLE hCRL);
HANDLE	SBCertificateRevocationListGetItem(HANDLE hCRL, int nIndex);
int		SBCertificateRevocationListGetCount(HANDLE hCRL);
BOOL	SBRevocationItemGetSerialNumber(HANDLE hItem, void* pBuffer, int* nSize);
BOOL	SBRevocationItemSetSerialNumber(HANDLE hItem, void* pBuffer, int nSize);
BOOL	SBRevocationItemGetRevocationDate(HANDLE hItem, DATE* date);
BOOL	SBRevocationItemSetRevocationDate(HANDLE hItem, DATE date);

#ifdef __cplusplus 
}
#endif


#endif