#include "SSLBBoxCli.h"

#define DLL_NAME	"SSLBBoxCli.dll"

#define DLL_ROUTINE_COUNT	32

static HMODULE	hDll = NULL;
static LONG nDllRefs = 0;

static FARPROC	aDllRoutines[DLL_ROUTINE_COUNT] = {
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
	NULL, NULL,	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
	NULL, NULL
};

static LPTSTR	aDllRoutineNames[DLL_ROUTINE_COUNT] = {
    TEXT("SBClientClose"),
    TEXT("SBClientCreate"),
    TEXT("SBClientDataAvailable"),
    TEXT("SBClientDestroy"),
    TEXT("SBClientDisable"),
    TEXT("SBClientDisableCipherSuite"),
    TEXT("SBClientDisableVersion"),
    TEXT("SBClientEnable"),
    TEXT("SBClientEnableCipherSuite"),
    TEXT("SBClientEnableVersion"),
    TEXT("SBClientGetCurrentVersion"),
    TEXT("SBClientIsActive"),
    TEXT("SBClientIsCipherSuiteEnabled"),
    TEXT("SBClientIsEnabled"),
    TEXT("SBClientIsVersionEnabled"),
    TEXT("SBClientOpen"),
    TEXT("SBClientSendData"),
    TEXT("SBClientSetOnData"),
    TEXT("SBClientSetOnReceive"),
    TEXT("SBClientSetOnSend"),
    TEXT("SBClientGetCipherSuite"),
    TEXT("SBClientSetOnOpenConnection"),
    TEXT("SBClientSetOnCloseConnection"),
    TEXT("SBClientSetOnCertificateValidate"),
    TEXT("SBClientSetOnCertificateNeeded"),
    TEXT("SBClientInternalValidate"),
    TEXT("SBClientSetCertStorage"),
    TEXT("SBClientResume"),
    TEXT("SBClientJoin"),
    TEXT("SBClientSetOnCertificateNeededEx"),
    TEXT("SBClientCloseSilent"),
    TEXT("SetLicenseKey")
};

typedef BOOL (__stdcall *SBClientCloseProc)(HANDLE hClient);
typedef HANDLE (__stdcall *SBClientCreateProc)(void);
typedef BOOL (__stdcall *SBClientDataAvailableProc)(HANDLE hClient);
typedef BOOL (__stdcall *SBClientDestroyProc)(HANDLE hClient);
typedef BOOL (__stdcall *SBClientDisableProc)(HANDLE hClient);
typedef BOOL (__stdcall *SBClientDisableCipherSuiteProc)(HANDLE hClient, int nSuite);
typedef BOOL (__stdcall *SBClientDisableVersionProc)(HANDLE hClient, int nVersion);
typedef BOOL (__stdcall *SBClientEnableProc)(HANDLE hClient);
typedef BOOL (__stdcall *SBClientEnableCipherSuiteProc)(HANDLE hClient, int nSuite);
typedef BOOL (__stdcall *SBClientEnableVersionProc)(HANDLE hClient, int nVersion);
typedef int (__stdcall *SBClientGetCurrentVersionProc)(HANDLE hClient);
typedef BOOL (__stdcall *SBClientIsCipherSuiteEnabledProc)(HANDLE hClient, int nSuite);
typedef BOOL (__stdcall *SBClientIsEnabledProc)(HANDLE hClient);
typedef BOOL (__stdcall *SBClientIsActiveProc)(HANDLE hClient);
typedef BOOL (__stdcall *SBClientIsVersionEnabledProc)(HANDLE hClient, int nVersion);
typedef BOOL (__stdcall *SBClientOpenProc)(HANDLE hClient);
typedef BOOL (__stdcall *SBClientSendDataProc)(HANDLE hClient, void* pBuffer, int nSize);
typedef int (__stdcall *SBClientGetCipherSuiteProc)(HANDLE hClient);
typedef BOOL (__stdcall *SBClientInternalValidateProc)(HANDLE hClient, int* Validity, int* Reason);
typedef BOOL (__stdcall *SBClientSetCertStorageProc)(HANDLE hClient, HANDLE hCertStorage);
typedef BOOL (__stdcall *SBClientResumeProc)(HANDLE hClient);
typedef BOOL (__stdcall *SBClientJoinProc)(HANDLE hClient, HANDLE hJoinClient);
typedef BOOL (__stdcall *SBClientSetOnDataProc)(HANDLE hClient, OnSendProc pHandler, int nUserData);
typedef BOOL (__stdcall *SBClientSetOnReceiveProc)(HANDLE hClient, OnReceiveProc pHandler, int nUserData);
typedef BOOL (__stdcall *SBClientSetOnSendProc)(HANDLE hClient, OnSendProc pHandler, int nUserData);
typedef BOOL (__stdcall *SBClientSetOnOpenConnectionProc)(HANDLE hClient, OnOpenConnectionProc pHandler, int nUserData);
typedef BOOL (__stdcall *SBClientSetOnCloseConnectionProc)(HANDLE hClient, OnCloseConnectionProc pHandler, int nUserData);
typedef BOOL (__stdcall *SBClientSetOnCertificateValidateProc)(HANDLE hClient, OnCertificateValidateProc pHandler, int nUserData);
typedef BOOL (__stdcall *SBClientSetOnCertificateNeededProc)(HANDLE hClient, OnCertificateNeededProc pHandler, int nUserData);
typedef BOOL (__stdcall *SBClientSetOnCertificateNeededExProc)(HANDLE hClient, OnCertificateNeededExProc pHandler, int nUserData);
typedef BOOL (__stdcall *SBClientCloseSilentProc)(HANDLE hClient);
typedef BOOL (__stdcall *SetLicenseKeyProc)(PCHAR Key);


BOOL SBSSLCliInitialize()
{
	if (InterlockedExchange(&nDllRefs, nDllRefs) > 0)
	{
		InterlockedIncrement(&nDllRefs);
		return TRUE;
	}
	else
	{
		hDll = LoadLibrary(DLL_NAME);
		if (hDll == NULL)
			return FALSE;
		InterlockedIncrement(&nDllRefs);
		for (int i = 0; i < DLL_ROUTINE_COUNT; i++)
		{
			aDllRoutines[i] = GetProcAddress(hDll, aDllRoutineNames[i]);
		}
		return TRUE;
	}
}

void SBSSLCliFinalize()
{
	if (InterlockedExchange(&nDllRefs, nDllRefs) > 0)
	{
		if (InterlockedDecrement(&nDllRefs) == 0)
		{
			FreeLibrary(hDll);
			hDll = NULL;
		}
	}
}

BOOL SBSSLCliSetLicenseKey(PCHAR Key)
{
	if (hDll == NULL) 
		return FALSE;
	else {
		((SetLicenseKeyProc)aDllRoutines[31])(Key);
		return TRUE;
	}
}

BOOL	SBClientClose(HANDLE hClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientCloseProc)aDllRoutines[0])(hClient);
}

HANDLE	SBClientCreate(void)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientCreateProc)aDllRoutines[1])();
}

BOOL	SBClientDataAvailable(HANDLE hClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientDataAvailableProc)aDllRoutines[2])(hClient);
}

BOOL	SBClientDestroy(HANDLE hClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientDestroyProc)aDllRoutines[3])(hClient);
}

BOOL	SBClientDisable(HANDLE hClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientDisableProc)aDllRoutines[4])(hClient);
}

BOOL	SBClientDisableCipherSuite(HANDLE hClient, int nSuite)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientDisableCipherSuiteProc)aDllRoutines[5])(hClient, nSuite);
}

BOOL	SBClientDisableVersion(HANDLE hClient, int nVersion)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientDisableVersionProc)aDllRoutines[6])(hClient, nVersion);
}

BOOL	SBClientEnable(HANDLE hClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientEnableProc)aDllRoutines[7])(hClient);
}

BOOL	SBClientEnableCipherSuite(HANDLE hClient, int nSuite)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientEnableCipherSuiteProc)aDllRoutines[8])(hClient, nSuite);
}

BOOL	SBClientEnableVersion(HANDLE hClient, int nVersion)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientEnableVersionProc)aDllRoutines[9])(hClient, nVersion);
}

int		SBClientGetCurrentVersion(HANDLE hClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientGetCurrentVersionProc)aDllRoutines[10])(hClient);
}

BOOL	SBClientIsActive(HANDLE hClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientIsActiveProc)aDllRoutines[11])(hClient);
}

BOOL	SBClientIsCipherSuiteEnabled(HANDLE hClient, int nSuite)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientIsCipherSuiteEnabledProc)aDllRoutines[12])(hClient, nSuite);
}

BOOL	SBClientIsEnabled(HANDLE hClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientIsEnabledProc)aDllRoutines[13])(hClient);
}

BOOL	SBClientIsVersionEnabled(HANDLE hClient, int nVersion)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientIsVersionEnabledProc)aDllRoutines[14])(hClient, nVersion);
}

BOOL	SBClientOpen(HANDLE hClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientOpenProc)aDllRoutines[15])(hClient);
}

BOOL	SBClientSendData(HANDLE hClient, void* pBuffer, int nSize)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientSendDataProc)aDllRoutines[16])(hClient, pBuffer, nSize);
}

BOOL	SBClientSetOnData(HANDLE hClient, OnDataProc pHandler, int nUserData)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientSetOnDataProc)aDllRoutines[17])(hClient, pHandler, nUserData);
}

BOOL	SBClientSetOnReceive(HANDLE hClient, OnReceiveProc pHandler, int nUserData)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientSetOnReceiveProc)aDllRoutines[18])(hClient, pHandler, nUserData);
}

BOOL	SBClientSetOnSend(HANDLE hClient, OnSendProc pHandler, int nUserData)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientSetOnSendProc)aDllRoutines[19])(hClient, pHandler, nUserData);
}

int		SBClientGetCipherSuite(HANDLE hClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientGetCipherSuiteProc)aDllRoutines[20])(hClient);
}

BOOL	SBClientSetOnOpenConnection(HANDLE hClient, OnOpenConnectionProc pHandler, int nUserData)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientSetOnOpenConnectionProc)aDllRoutines[21])(hClient, pHandler, nUserData);
}

BOOL	SBClientSetOnCloseConnection(HANDLE hClient, OnCloseConnectionProc pHandler, int nUserData)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientSetOnCloseConnectionProc)aDllRoutines[22])(hClient, pHandler, nUserData);
}

BOOL	SBClientSetOnCertificateValidate(HANDLE hClient, OnCertificateValidateProc pHandler, int nUserData)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientSetOnCertificateValidateProc)aDllRoutines[23])(hClient, pHandler, nUserData);
}

BOOL	SBClientSetOnCertificateNeeded(HANDLE hClient, OnCertificateNeededProc pHandler, int nUserData)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientSetOnCertificateNeededProc)aDllRoutines[24])(hClient, pHandler, nUserData);
}

BOOL	SBClientInternalValidate(HANDLE hClient, int* Validity, int* Reason)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientInternalValidateProc)aDllRoutines[25])(hClient, Validity, Reason);
}

BOOL	SBClientSetCertStorage(HANDLE hClient, HANDLE hCertStorage)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientSetCertStorageProc)aDllRoutines[26])(hClient, hCertStorage);
}

BOOL	SBClientResume(HANDLE hClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientResumeProc)aDllRoutines[27])(hClient);
}

BOOL	SBClientJoin(HANDLE hClient, HANDLE hJoinClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientJoinProc)aDllRoutines[28])(hClient, hJoinClient);
}

BOOL	SBClientSetOnCertificateNeededEx(HANDLE hClient, OnCertificateNeededExProc pHandler, int nUserData)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientSetOnCertificateNeededExProc)aDllRoutines[29])(hClient, pHandler, nUserData);
}

BOOL	SBClientCloseSilent(HANDLE hClient)
{
	if (hDll == NULL)
		return FALSE;
	else
		return ((SBClientCloseSilentProc)aDllRoutines[30])(hClient);
}
