#ifndef __SSLBBOXCLI_H__
#define __SSLBBOXCLI_H__

#include <windows.h>

// SSL Protocol Versions

#define SB_VERSION_SSL2			0
#define SB_VERSION_SSL3			1
#define SB_VERSION_TLS1			2

// Cipher Suites

#define		SB_SUITE_NULL_NULL_NULL			0x0000
#define		SB_SUITE_RSA_NULL_MD5			0x0001
#define		SB_SUITE_RSA_NULL_SHA           0x0002
#define		SB_SUITE_RSA_RC4_MD5            0x0003
#define		SB_SUITE_RSA_RC4_SHA            0x0004
#define		SB_SUITE_RSA_RC2_MD5            0x0005
#define		SB_SUITE_RSA_IDEA_MD5           0x0006
#define		SB_SUITE_RSA_IDEA_SHA           0x0007
#define		SB_SUITE_RSA_DES_MD5            0x0008
#define		SB_SUITE_RSA_DES_SHA            0x0009
#define		SB_SUITE_RSA_3DES_MD5           0x000A
#define		SB_SUITE_RSA_3DES_SHA           0x000B
#define		SB_SUITE_RSA_AES128_SHA         0x000C
#define		SB_SUITE_RSA_AES256_SHA         0x000D
#define		SB_SUITE_DH_DSS_DES_SHA         0x000E
#define		SB_SUITE_DH_DSS_3DES_SHA        0x000F
#define		SB_SUITE_DH_DSS_AES128_SHA      0x0010
#define		SB_SUITE_DH_DSS_AES256_SHA      0x0011
#define		SB_SUITE_DH_RSA_DES_SHA         0x0012
#define		SB_SUITE_DH_RSA_3DES_SHA        0x0013
#define		SB_SUITE_DH_RSA_AES128_SHA      0x0014
#define		SB_SUITE_DH_RSA_AES256_SHA      0x0015
#define		SB_SUITE_DHE_DSS_DES_SHA        0x0016
#define		SB_SUITE_DHE_DSS_3DES_SHA       0x0017
#define		SB_SUITE_DHE_DSS_AES128_SHA     0x0018
#define		SB_SUITE_DHE_DSS_AES256_SHA     0x0019
#define		SB_SUITE_DHE_RSA_DES_SHA        0x001A
#define		SB_SUITE_DHE_RSA_3DES_SHA       0x001B
#define		SB_SUITE_DHE_RSA_AES128_SHA     0x001C
#define		SB_SUITE_DHE_RSA_AES256_SHA     0x001D
#define		SB_SUITE_DH_ANON_RC4_MD5        0x001E
#define		SB_SUITE_DH_ANON_DES_SHA        0x001F
#define		SB_SUITE_DH_ANON_3DES_SHA       0x0020
#define		SB_SUITE_DH_ANON_AES128_SHA     0x0021
#define		SB_SUITE_DH_ANON_AES256_SHA     0x0022
#define		SB_SUITE_RSA_RC2_MD5_EXPORT     0x0023
#define		SB_SUITE_RSA_RC4_MD5_EXPORT     0x0024
#define		SB_SUITE_RSA_DES_SHA_EXPORT     0x0025
#define		SB_SUITE_DH_DSS_DES_SHA_EXPORT  0x0026
#define		SB_SUITE_DH_RSA_DES_SHA_EXPORT  0x0027
#define		SB_SUITE_DHE_DSS_DES_SHA_EXPORT 0x0028
#define		SB_SUITE_DHE_RSA_DES_SHA_EXPORT 0x0029
#define		SB_SUITE_DH_ANON_RC4_MD5_EXPORT 0x002A
#define		SB_SUITE_DH_ANON_DES_SHA_EXPORT 0x002B

// Connection close reasons

#define		SB_CR_CLOSE						0x00
#define		SB_CR_ERROR						0x01

#ifdef _cplusplus
extern "C" {
#endif

// Callbacks prototypes

typedef void (__stdcall *OnSendProc)(HANDLE hClient, int nUserData, void* pBuffer, int nSize);
typedef void (__stdcall *OnReceiveProc)(HANDLE hClient, int nUserData, void* pBuffer, int nMaxSize, int* pWritten);
typedef void (__stdcall *OnDataProc)(HANDLE hClient, int nUserData, void* pBuffer, int nSize);
typedef void (__stdcall *OnOpenConnectionProc)(HANDLE hClient, int nUserData);
typedef void (__stdcall *OnCloseConnectionProc)(HANDLE hClient, int nUserData, int Reason);
typedef void (__stdcall *OnCertificateValidateProc)(HANDLE hClient, int nUserData, HANDLE hX509handle, int* Validate);
typedef void (__stdcall *OnCertificateNeededProc)(HANDLE hClient, int nUserData, void* pCertificateBuffer,
												  int* nCertificateSize, void* pPrivateKeyBuffer, 
												  int* nPrivateKeySize, int CertificateType);
typedef void (__stdcall *OnCertificateNeededExProc)(HANDLE hClient, int nUserData,
													HANDLE* hCert);


BOOL SBSSLCliInitialize();
void SBSSLCliFinalize();
BOOL SBSSLCliSetLicenseKey(PCHAR Key);

BOOL	SBClientClose(HANDLE hClient);
BOOL	SBClientCloseSilent(HANDLE hClient);
HANDLE	SBClientCreate(void);
BOOL	SBClientDataAvailable(HANDLE hClient);
BOOL	SBClientDestroy(HANDLE hClient);
BOOL	SBClientDisable(HANDLE hClient);
BOOL	SBClientDisableCipherSuite(HANDLE hClient, int nSuite);
BOOL	SBClientDisableVersion(HANDLE hClient, int nVersion);
BOOL	SBClientEnable(HANDLE hClient);
BOOL	SBClientEnableCipherSuite(HANDLE hClient, int nSuite);
BOOL	SBClientEnableVersion(HANDLE hClient, int nVersion);
int		SBClientGetCurrentVersion(HANDLE hClient);
BOOL	SBClientIsCipherSuiteEnabled(HANDLE hClient, int nSuite);
BOOL	SBClientIsEnabled(HANDLE hClient);
BOOL	SBClientIsActive(HANDLE hClient);
BOOL	SBClientIsVersionEnabled(HANDLE hClient, int nVersion);
BOOL	SBClientOpen(HANDLE hClient);
int		SBClientGetCipherSuite(HANDLE hClient);
BOOL	SBClientInternalValidate(HANDLE hClient, int* Validity, int* Reason);
BOOL	SBClientSetCertStorage(HANDLE hClient, HANDLE hCertStorage);
BOOL	SBClientResume(HANDLE hClient);
BOOL	SBClientJoin(HANDLE hClient, HANDLE hJoinClient);
BOOL	SBClientSendData(HANDLE hClient, void* pBuffer, int nSize);
BOOL	SBClientSetOnData(HANDLE hClient, OnDataProc pHandler, int nUserData);
BOOL	SBClientSetOnReceive(HANDLE hClient, OnReceiveProc pHandler, int nUserData);
BOOL	SBClientSetOnSend(HANDLE hClient, OnSendProc pHandler, int nUserData);
BOOL	SBClientSetOnOpenConnection(HANDLE hClient, OnOpenConnectionProc pHandler, int nUserData);
BOOL	SBClientSetOnCloseConnection(HANDLE hClient, OnCloseConnectionProc pHandler, int nUserData);
BOOL	SBClientSetOnCertificateValidate(HANDLE hClient, OnCertificateValidateProc pHandler, int nUserData);
BOOL	SBClientSetOnCertificateNeeded(HANDLE hClient, OnCertificateNeededProc pHandler, int UserData);
BOOL	SBClientSetOnCertificateNeededEx(HANDLE hClient, OnCertificateNeededExProc pHandler, int UserData);

#ifdef _cplusplus
}
#endif

#endif