#ifndef __SSLBBOXSRV_H__
#define __SSLBBOXSRV_H__

#include <windows.h>

// SSL Protocol Versions

#define SB_VERSION_SSL2			0
#define SB_VERSION_SSL3			1
#define SB_VERSION_TLS1			2

// Cipher Suites

#define		SB_SUITE_NULL_NULL_NULL			0x0000
#define		SB_SUITE_RSA_NULL_MD5			0x0001
#define		SB_SUITE_RSA_NULL_SHA           0x0002
#define		SB_SUITE_RSA_RC4_MD5            0x0003
#define		SB_SUITE_RSA_RC4_SHA            0x0004
#define		SB_SUITE_RSA_RC2_MD5            0x0005
#define		SB_SUITE_RSA_IDEA_MD5           0x0006
#define		SB_SUITE_RSA_IDEA_SHA           0x0007
#define		SB_SUITE_RSA_DES_MD5            0x0008
#define		SB_SUITE_RSA_DES_SHA            0x0009
#define		SB_SUITE_RSA_3DES_MD5           0x000A
#define		SB_SUITE_RSA_3DES_SHA           0x000B
#define		SB_SUITE_RSA_AES128_SHA         0x000C
#define		SB_SUITE_RSA_AES256_SHA         0x000D
#define		SB_SUITE_DH_DSS_DES_SHA         0x000E
#define		SB_SUITE_DH_DSS_3DES_SHA        0x000F
#define		SB_SUITE_DH_DSS_AES128_SHA      0x0010
#define		SB_SUITE_DH_DSS_AES256_SHA      0x0011
#define		SB_SUITE_DH_RSA_DES_SHA         0x0012
#define		SB_SUITE_DH_RSA_3DES_SHA        0x0013
#define		SB_SUITE_DH_RSA_AES128_SHA      0x0014
#define		SB_SUITE_DH_RSA_AES256_SHA      0x0015
#define		SB_SUITE_DHE_DSS_DES_SHA        0x0016
#define		SB_SUITE_DHE_DSS_3DES_SHA       0x0017
#define		SB_SUITE_DHE_DSS_AES128_SHA     0x0018
#define		SB_SUITE_DHE_DSS_AES256_SHA     0x0019
#define		SB_SUITE_DHE_RSA_DES_SHA        0x001A
#define		SB_SUITE_DHE_RSA_3DES_SHA       0x001B
#define		SB_SUITE_DHE_RSA_AES128_SHA     0x001C
#define		SB_SUITE_DHE_RSA_AES256_SHA     0x001D
#define		SB_SUITE_DH_ANON_RC4_MD5        0x001E
#define		SB_SUITE_DH_ANON_DES_SHA        0x001F
#define		SB_SUITE_DH_ANON_3DES_SHA       0x0020
#define		SB_SUITE_DH_ANON_AES128_SHA     0x0021
#define		SB_SUITE_DH_ANON_AES256_SHA     0x0022
#define		SB_SUITE_RSA_RC2_MD5_EXPORT     0x0023
#define		SB_SUITE_RSA_RC4_MD5_EXPORT     0x0024
#define		SB_SUITE_RSA_DES_SHA_EXPORT     0x0025
#define		SB_SUITE_DH_DSS_DES_SHA_EXPORT  0x0026
#define		SB_SUITE_DH_RSA_DES_SHA_EXPORT  0x0027
#define		SB_SUITE_DHE_DSS_DES_SHA_EXPORT 0x0028
#define		SB_SUITE_DHE_RSA_DES_SHA_EXPORT 0x0029
#define		SB_SUITE_DH_ANON_RC4_MD5_EXPORT 0x002A
#define		SB_SUITE_DH_ANON_DES_SHA_EXPORT 0x002B

// Connection close reasons

#define		SB_CR_CLOSE						0x00
#define		SB_CR_ERROR						0x01

#ifdef _cplusplus
extern "C" {
#endif

typedef void (__stdcall *OnSendProc)(HANDLE hClient, int nUserData, void* pBuffer, int nSize);
typedef void (__stdcall *OnReceiveProc)(HANDLE hClient, int nUserData, void* pBuffer, int nMaxSize, int* pWritten);
typedef void (__stdcall *OnDataProc)(HANDLE hClient, int nUserData, void* pBuffer, int nSize);
typedef void (__stdcall *OnOpenConnectionProc)(HANDLE hClient, int nUserData);
typedef void (__stdcall *OnCloseConnectionProc)(HANDLE hClient, int nUserData, int Reason);
typedef void (__stdcall *OnCertificateValidateProc)(HANDLE hClient, int nUserData, HANDLE hX509handle, int* Validate);

BOOL SBSSLSrvInitialize();
void SBSSLSrvFinalize();
BOOL SBSSLSrvSetLicenseKey(PCHAR Key);

// Session pool interface

HANDLE	SBSessionPoolCreate(void);
BOOL	SBSessionPoolDestroy(HANDLE hSessionPool);
BOOL	SBSessionPoolSetTimeout(HANDLE hSessionPool, int nTimeout);
BOOL	SBSessionPoolSetInterval(HANDLE hSessionPool, int nInterval);
BOOL	SBSessionPoolSetMaxCount(HANDLE hSessionPool, int nMaxCount);

// Secure Server interface

HANDLE	SBServerCreate(void);
BOOL	SBServerDestroy(HANDLE hServer);
BOOL	SBServerOpen(HANDLE hServer);
BOOL	SBServerClose(HANDLE hServer);
BOOL	SBServerDataAvailable(HANDLE hServer);
BOOL	SBServerEnable(HANDLE hServer);
BOOL	SBServerDisable(HANDLE hServer);
BOOL	SBServerEnableCipherSuite(HANDLE hServer, int nSuite);
BOOL	SBServerDisableCipherSuite(HANDLE hServer, int nSuite);
BOOL	SBServerEnableVersion(HANDLE hServer, int nVersion);
BOOL	SBServerDisableVersion(HANDLE hServer, int nVersion);
int		SBServerGetCurrentVersion(HANDLE hServer);
BOOL	SBServerIsEnabled(HANDLE hServer);
BOOL	SBServerIsActive(HANDLE hServer);
BOOL	SBServerIsCipherSuiteEnabled(HANDLE hServer, int nSuite);
BOOL	SBServerIsVersionEnabled(HANDLE hServer, int nVersion);
int		SBServerGetCipherSuite(HANDLE hServer);
BOOL	SBServerSendData(HANDLE hServer, void* pBuffer, int nSize);
BOOL	SBServerSetOnData(HANDLE hServer, OnDataProc pHandler, int nUserData);
BOOL	SBServerSetOnReceive(HANDLE hServer, OnReceiveProc pHandler, int nUserData);
BOOL	SBServerSetOnSend(HANDLE hServer, OnSendProc pHandler, int nUserData);
BOOL	SBServerSetOnOpenConnection(HANDLE hServer, OnOpenConnectionProc pHandler, int nUserData);
BOOL	SBServerSetOnCloseConnection(HANDLE hServer, OnCloseConnectionProc pHandler, int nUserData);
BOOL	SBServerSetOnCertificateValidate(HANDLE hServer, OnCertificateValidateProc pHandler, int nUserData);
BOOL	SBServerInternalValidate(HANDLE hServer, int* Validity, int* Reason);
BOOL	SBServerSetCertStorage(HANDLE hServer, HANDLE hCertStorage);
BOOL	SBServerSetSessionPool(HANDLE hServer, HANDLE hSessionPool);
BOOL	SBServerSetClientCertStorage(HANDLE hServer, HANDLE hCertStorage);
BOOL	SBServerEnableClientAuthentication(HANDLE hServer);
BOOL	SBServerDisableClientAuthentication(HANDLE hServer);
BOOL	SBServerIsClientAuthenticationEnabled(HANDLE hServer);
BOOL	SBServerEnableForceCertificateChain(HANDLE hServer);
BOOL	SBServerDisableForceCertificateChain(HANDLE hServer);
BOOL	SBServerIsForceCertificateChainEnabled(HANDLE hServer);

#ifdef _cplusplus
}
#endif


#endif